/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass;

import org.eclipse.jface.action.IAction;

import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

/**
 * Interface, das von Clients implementiert werden muss, damit das KonfigAss Plug-In
 * den registrierten Teilkonfigurationsassistenten das Konfig-Datenmodell bergeben kann.
 * Es werden zustzlich Methoden bereitgestellt um Informationen an das KonfigAss Plug-In
 * zu bergeben.  
 *  
 * @author Pittner
 *
 */
public interface IKonfigAssDatenModell
{
  /**
   * Holt die PluginId des Plugins welches der KonfigAss verwendet, wird vom Contributor geliefert
   * 
   * @return Id
   */
  public String holePluginId();
  
  /**
   * Holt den Namen des Plugins welches der KonfigAss verwendet, wird vom Contributor geliefert
   * 
   * @return Name
   */
  public String holeName();

  /**
   * Holt die Beschreibung des Plugins welches der KonfigAss verwendet, wird vom Contributor geliefert
   * 
   * @return Beschreibung
   */
  public String holeBeschreibung();
  
  /**
   * Holt die Action des Plugins welches der KonfigAss verwendet, wird vom Contributor geliefert
   * 
   * @return Aktion
   */
  public IAction holeAktion();
  
//  /**
//   * Holt die Action des Plugins welches der KonfigAss verwendet, wird vom Contributor geliefert
//   * 
//   * @return Aktion
//   */
//  public String holePfad();

//  /**
//   * Holt die Action des Plugins welches der KonfigAss verwendet, wird vom Contributor geliefert
//   * 
//   * @return Aktion
//   */
//  public String holeSecId();
 
  
  /**
   * bergibt den Plug-In welche den Erweiterungspunkt verwenden das Konfig-Datenmodell, 
   * wird vom Enabler geliefert
   * 
   * @param dataModell 
   *            das Konfig-Datenmodell
   * @param pfad 
   *            der Pfad zu Verwaltungsdatei der Konfiguration aus der das Konfig-Datenmodell erzeugt wurde
   */
  public void setzeKonfigDatenModell(ConfigDataModel dataModel, String pfad);
  
  /**
   *  Informiert die Plug-In welche den Erweiterungspunkt verwenden und bereits ein Datenmodell erhalten
   *  haben darber, dass eine Konfiguration geladen wird.
   * 
   * @param pfad 
   *            der Pfad zu Verwaltungsdatei der Konfiguration aus der das Konfig-Datenmodell erzeugt wird
   */
  public void ladeKonfigDatenModell(String pfad); 
}
