/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.konfigass.assistenten;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;

import de.bsvrz.buv.plugin.konfigass.KonfigAssDateiOperationen;
import de.bsvrz.buv.plugin.konfigass.KonfigBereichDaten;

/**
 * Class representing the first page of the wizard
 */

public class TKAStartenStartSeite extends WizardPage implements Listener
{

  public static final String copyright = "(c) Copyright IBM Corporation 2002.";

  IWorkbench workbench;

  IStructuredSelection selection;

  Button kvButton;

  Button aoeButton;

  // status variable for the possible errors on this page
  // timeStatus holds an error if the return date is before the departure date
  IStatus timeStatus;

  // holds an error is the destination and departure fields are the same
  IStatus destinationStatus;

  private Label labelName;

  private Label labelBeschreibung;

  private Text textName;

  private Label labelQuelle;

  private Text textPid;

  private Label labelQuelle1;

  private Text textOrtKonfig;

  private Button button;

  private String _pidAoe;

  protected String _name;

  private Label labelQuelle2;

  private Text textOrtOrdner;

  private Button button2;

  private Label labelKodierung;

  private Text textKodierung;

  //
  // final static String[] dates = { "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15",
  // "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30", "31" };
  //
  // final static String[] months = { "January", "February", "March", "April", "May", "June", "July", "August",
  // "September", "October", "November", "December" };
  //
  // final static String[] years;
  //
  // final static int startingYear;

  private Shell shell;

  private File _source;

  private File _dest;

  private TableViewer _tvZufluesse1;

  private List<KonfigBereichDaten> _listKonfigBereichDaten;

  private String _tka;

  // static
  // {
  // Calendar cal = Calendar.getInstance();
  // cal.setTime(new Date());
  // startingYear = cal.get(Calendar.YEAR);
  // years = new String[3];
  // for (int i = -1; i < 2; i++)
  // {
  // years[i + 1] = String.valueOf(startingYear + i);
  // }
  // }

  /**
   * Constructor for HolidayMainPage.
   */
  public TKAStartenStartSeite(String tka, List<KonfigBereichDaten> list)
  {
    super("Page1");
    _tka = tka;
    _listKonfigBereichDaten = list;

    setTitle("Starten von: " + tka);
    setDescription("Es muss ein Konfigurationsbereich ausgewhlt werden:");

    destinationStatus = new Status(IStatus.OK, "not_used", 0, "", null);
    timeStatus = new Status(IStatus.OK, "not_used", 0, "", null);
  }

  /**
   * @see IDialogPage#createControl(Composite)
   */
  public void createControl(Composite parent)
  {

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = false;
    gridData.verticalAlignment = GridData.CENTER;

    shell = parent.getShell();

    // create the composite to hold the widgets
    Composite composite = new Composite(parent, SWT.NULL);

    // create the desired layout for this wizard page
    GridLayout gl = new GridLayout(1, false);
    composite.setLayout(gl);

    _tvZufluesse1 = new TableViewer(composite, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL
        | SWT.V_SCROLL);

    TableViewerColumn tc1 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
    tc1.getColumn().setWidth(100);
    tc1.getColumn().setText("AOE Konfiguration");
    // tc1.getColumn().pack();
    TableViewerColumn tc2 = new TableViewerColumn(_tvZufluesse1, SWT.NONE);
    tc2.getColumn().setWidth(100);
    tc2.getColumn().setText("Konfigurationsbereich");

    _tvZufluesse1.getTable().setLinesVisible(true);
    _tvZufluesse1.getTable().setHeaderVisible(true);

    _tvZufluesse1.setContentProvider(new TabelleInhaltsVersorger());
    _tvZufluesse1.setLabelProvider(new TabelleDarstellungsVersorger());

    GridData gd = new GridData(GridData.FILL_HORIZONTAL);
    _tvZufluesse1.getTable().setLayout(new GridLayout(1, false));
    _tvZufluesse1.getTable().setLayoutData(gd);

    createLine(composite, 1);

    setControl(composite);

    if (_listKonfigBereichDaten != null)
    {
      _tvZufluesse1.setInput(_listKonfigBereichDaten);
    }

  }

  private void erzeugeTabelleTeilkonfigurationsassistenten(Composite parent)
  {

  }

  private void addListeners()
  {
    // textKodierung.addListener(SWT.Modify, this);
    textName.addListener(SWT.Modify, this);
    // textPid.addListener(SWT.Modify, this);
    textOrtKonfig.addListener(SWT.Modify, this);
    button.addListener(SWT.Selection, this);
    textOrtOrdner.addListener(SWT.Modify, this);
    button2.addListener(SWT.Selection, this);

    // kvButton.addListener(SWT.Selection, this);
    // aoeButton.addListener(SWT.Selection, this);
  }

  /**
   * @see Listener#handleEvent(Event)
   */
  public void handleEvent(Event event)
  {
    if ((event.widget == button) || (event.widget == button2))
    {
      DirectoryDialog dlg = new DirectoryDialog(shell);

      dlg.setText("Ordner-Auswahl Dialog");

      dlg.setMessage("Ordner auswhlen");

      String dir = dlg.open();
      if (dir != null)
      {
        if (event.widget == button)
        {
          _source = new File(dir);

          textOrtKonfig.setText(dir);
        }
        if (event.widget == button2)
        {
          _dest = new File(dir);
          textOrtOrdner.setText(dir);
        }
      }
    }
    getWizard().getContainer().updateButtons();
  }

  /*
   * Returns the next page. Saves the values from this page in the model associated with the wizard. Initializes the
   * widgets on the next page.
   */

  public IWizardPage getNextPage()
  {
    saveDataToModel();

    if (_source != null && _dest != null)
    {
      _dest = new File(_dest.getAbsolutePath() + "/" + textName.getText());

      final File fsource = _source;
      final File fdest = _dest;

      try
      {
        new ProgressMonitorDialog(shell).run(true, true, new IRunnableWithProgress() {

          @Override
          public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException
          {

            try
            {

              KonfigAssDateiOperationen.kopiereOrdnerDatei(fsource, fdest, monitor, new Integer(0));

            }
            catch (IOException e)
            {
              // TODO Auto-generated catch block
              e.printStackTrace();
            }

          }
        });
      }
      catch (InvocationTargetException e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
      catch (InterruptedException e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }

    }

    KvAoeNeuSeite1 page = ((KvAoeNeuAssistent)getWizard()).seite1;
    page.onEnterPage();
    return page;

  }

  /**
   * @see IWizardPage#canFlipToNextPage()
   */
  public boolean canFlipToNextPage()
  {

    if (getErrorMessage() != null)
      return false;

    if (isTextNonEmpty(textName) && isTextNonEmpty(textOrtKonfig) && isTextNonEmpty(textOrtOrdner))
    {
      return true;

    }
    return false;
  }

  /*
   * Saves the uses choices from this page to the model. Called on exit of the page
   */
  private void saveDataToModel()
  {
    // Gets the model
    KvAoeNeuAssistent wizard = (KvAoeNeuAssistent)getWizard();
    KvAoeNeuModell model = wizard.model;

    if (kvButton.getSelection())
      model.useKV = true;
    else
      model.useKV = false;

    if (aoeButton.getSelection())
      model.useAOE = true;
    else
      model.useAOE = false;

    model._pid = textName.getText();
    model._quellVerzeichnis = textOrtKonfig.getText();
    model._zielVerzeichnis = textOrtOrdner.getText();

  }

  private static boolean isTextNonEmpty(Text t)
  {
    if (t != null)
    {
      String s = t.getText();
      if ((s != null) && (s.trim().length() > 0))
        return true;
    }
    return false;
  }

  private void createLine(Composite parent, int ncol)
  {
    Label line = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL | SWT.BOLD);
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);
    gridData.horizontalSpan = ncol;
    line.setLayoutData(gridData);
  }

  /**
   * @author Pittner
   * 
   */
  private class TabelleInhaltsVersorger implements IStructuredContentProvider
  {

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object inputElement)
    {
      java.util.List list = (java.util.List)inputElement;

      Object[] re = new Object[list.size()];

      for (int i = 0; i < list.size(); i++)
      {
        re[i] = list.get(i);
      }

      return re;
    }

    /*
     * (non-Javadoc) *
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose()
    {

    }

    /*
     * (non-Javadoc) *
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object,
     * java.lang.Object)
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {

    }
  }

  /**
   * {@link LabelProvider} fr die Tabelle.
   * 
   * @author HofmannJ
   * 
   */
  private class TabelleDarstellungsVersorger implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    @Override
    public String getColumnText(Object element, int columnIndex)
    {
      String returnString = null;

      switch (columnIndex)
      {
        case 0:
          returnString = ((KonfigBereichDaten)element).getPidAOE();
          break;
        case 1:
          returnString = ((KonfigBereichDaten)element).getName();
          break;
        default:
          returnString = "default";
      }
      return returnString;
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

}
