/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkaaq;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnBeobachter;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.EingabeVorbelegung;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

public class BildungsRegelnBWAQ extends BildungsRegelnAllgemeinAQ
{
  protected Map<String, EingabeVorbelegung> _vorbelegungen = new HashMap<String, EingabeVorbelegung>();

  protected Set<BildungsRegelnBeobachter> _beobachter = new LinkedHashSet<BildungsRegelnBeobachter>();

  protected void initVorbelegungen()
  {
    _vorbelegungen.put("Bezeichnung", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("Strae", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("Knotennummer", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("Typ", new EingabeVorbelegung("HFB", new String[] { "HFB", "NFB", "Aus", "Ein" }));
    _vorbelegungen.put("Fahrtrichtung", new EingabeVorbelegung(new String[] { "N", "O", "S", "W" }));
    _vorbelegungen.put("Fernziel", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("TypName", new EingabeVorbelegung(null, true));
  }

  @Override
  protected void initPraefixe()
  {
    super.initPraefixe();
  }

  @Override
  protected void initFelder()
  {
    super.initFelder();
    _felder.put(HOT_AQ.AnzeigeQuerschnitt, new String[] { "Bezeichnung", "Strae", "Knotennummer", "Typ",
        "Fahrtrichtung", "Fernziel", "TypName" });
    // _felder.put(HOT_AQ.AnzeigeQuerschnittSub, new String[] { "Bezeichnung", "Strae", "Knotennummer", "Typ",
    // "Fahrtrichtung", "Fernziel", "TypName" });
  }

  public BildungsRegelnBWAQ()
  {
    super();

    initVorbelegungen();
  }

  public EingabeVorbelegung getEingabeVorbelegung(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld,
      HierarchieObjekt verbundenesHO)
  {
    if (hotNeuesObjekt == HOT_AQ.AnzeigeQuerschnitt)
    {
      if (Arrays.asList(_felder.get(hotNeuesObjekt)).contains(eingabeFeld))
      {
        String text = null;
        EingabeVorbelegung result = _vorbelegungen.get(eingabeFeld);

        if (Arrays.asList(_felder.get(HOT_AQ.AnzeigeQuerschnitt)).contains(eingabeFeld))
        {
// Keine Vererbung der Feldinhalte  		
//          Map<String, String> werte = Collections.emptyMap();

//          if (verbundenesHO != null)
//          {
//            werte = _historie.get(verbundenesHO.getSystemObjekte().get(0).getPid());
//          }
//
//          if (werte != null)
//          {
//            text = werte.get(eingabeFeld);            
//          }
        }

        if (text != null && text.length() > 0)
        {
          String[] liste = result.getListe();
          boolean leerErlaubt = result.isLeerErlaubt();
          return liste.length > 0 ? new EingabeVorbelegung(text, liste, leerErlaubt) : new EingabeVorbelegung(text,
              leerErlaubt);
        }
        else
        {
          return result;
        }
      }
    }
    else
    {
      return super.getEingabeVorbelegung(hotNeuesObjekt, eingabeFeld, verbundenesHO);
    }

    return BildungsRegelnTools.LEERE_VORBELEGUNG;
  }

  @Override
  public boolean setWerte(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO,
      HierarchieObjekt vorlageHO)
  {

    // if (hotNeuesObjekt.equals(HOT_AQ.AnzeigeQuerschnitt))
    // {
    // if (!(vorlageHO.getHierarchieObjektTyp().equals(hotNeuesObjekt)))
    // {
    // throw new IllegalArgumentException("Als HOT des Vorlage-HO ist nur der Typ " + hotNeuesObjekt + " erlaubt");
    // }
    // String felder[] = _felder.get(hotNeuesObjekt);
    //      
    // if (felder != null)
    // {
    // for (String eingabeFeld: felder)
    // {
    // if ("Typ".equals(eingabeFeld))
    // {
    // // // Typ wird aus dem Attribut Lage bernommen
    // // SystemObject fs = vorlageHO.getSystemObjekt(0);
    // // Data data = fs.getConfigurationData(fs.getDataModel().getAttributeGroup("atg.fahrStreifen"));
    // // if (data != null)
    // // {
    // // String lage = data.getItem("Lage").asTextValue().getText();
    // // setWert(hotNeuesObjekt, eingabeFeld, lage);
    // // }
    // List<HierarchieObjekt> kinder = getHierarchie().getUntergeordneteObjekte(uebergeordnetesHO);
    // int anzFS = kinder.size();
    // if (anzFS == 0)
    // {
    // setWert(hotNeuesObjekt, eingabeFeld, "HFS");
    // }
    // else
    // {
    // setWert(hotNeuesObjekt, eingabeFeld, anzFS + "FS");
    // }
    // }
    // else
    // {
    // EingabeVorbelegung belegung = getEingabeVorbelegung(hotNeuesObjekt, eingabeFeld, uebergeordnetesHO);
    // setWert(hotNeuesObjekt, eingabeFeld, belegung.getWert());
    // }
    // }
    // }
    // }
    // else
    {
//      String felder[] = _felder.get(hotNeuesObjekt);
//
//      if (felder != null)
//      {
//        for (String eingabeFeld : felder)
//        {
//          EingabeVorbelegung belegung = getEingabeVorbelegung(hotNeuesObjekt, eingabeFeld, uebergeordnetesHO);
//          setWert(hotNeuesObjekt, eingabeFeld, belegung.getWert());
//        }
//      }
      
      if (!HOT_AQ.AnzeigeQuerschnitt.equals(hotNeuesObjekt))
      {  
        return super.setWerte(hotNeuesObjekt, uebergeordnetesHO, vorlageHO);
      }  
    }

    return false;
  }

  @Override
  public String getPidRegelText(IHierarchieObjektTyp hot)
  {
    String result;
    
    if (hot.equals(HOT_AQ.AnzeigeQuerschnitt))
    {
      result = getPraefix(hot.getTypen().get(0)) + ".Strae" + ".Knotennummer" + ".Typ" + ".Fahrtrichtung";
    }
    else
    {
      result = super.getPidRegelText(hot);
    }

    return result;
  }

  @Override
  public String getNameRegelText(IHierarchieObjektTyp hot)
  {
    String result;
    
    if (hot.equals(HOT_AQ.AnzeigeQuerschnitt))
    {
      result = "AQ Bezeichnung" + " Fahrtrichtung" + " FR" + " Fernziel" + " [TypName]";
    }
    else
    {
      result = super.getNameRegelText(hot);
    }

    return result;
  }

  @Override
  public String[] getPids(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO)
  {
    
    if (hotNeuesObjekt == HOT_AQ.AnzeigeQuerschnitt)
    {
      String text = "";
      // if (hotNeuesObjekt == HOT_AQ.AnzeigeQuerschnitt || hotNeuesObjekt == HOT_AQ.AnzeigeQuerschnittSub)
      {
        text += getWert(hotNeuesObjekt, "Strae");
        text += ".";
        text += getWert(hotNeuesObjekt, "Knotennummer");
        text += ".";
        text += getWert(hotNeuesObjekt, "Typ");
        text += ".";
        text += getWert(hotNeuesObjekt, "Fahrtrichtung");
      }
      
      String[] result = new String[1];

      String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);

      StringBuffer buffer = new StringBuffer();

      buffer.append(getPraefix(pidTypNeuesObjekt));
      buffer.append('.');

      buffer.append(BildungsRegelnTools.getPidText(text, true, true));

      String pid = buffer.toString();
      Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
      if (werte != null && !werte.isEmpty())
      {
        _historie.put(pid, new HashMap<String, String>(werte));
      }

      result[0] = pid;

      return result;
    }
    else
    {
      return super.getPids(hotNeuesObjekt, uebergeordnetesHO);
    }

  }

  public String getName(SystemObject objekt)
  {
    String pid = objekt.getPid();

    Map<String, String> werte = _historie.get(pid);

    if (werte != null)
    {
      SystemObjectType typ = objekt.getType();
      StringBuffer buffer = new StringBuffer();

      if (typ.getPid().equals("typ.anzeigeQuerschnitt"))
      {
        buffer.append("AQ ");
        buffer.append(werte.get("Bezeichnung"));

        buffer.append(" ");
        buffer.append(werte.get("Fahrtrichtung"));

        buffer.append(" FR ");
        buffer.append(werte.get("Fernziel"));

        String typName = werte.get("TypName");
        if (typName != null && !typName.isEmpty())
        {
          buffer.append(' ');
          buffer.append(typName);
        }

        return buffer.toString();
      }
    }

    // return BildungsRegelnTools.LEERER_WERT;
    return super.getName(objekt);
  }

  @Override
  public Data getDatensatz(HierarchieObjekt ho, SystemObject objekt, AttributeGroup atg, Data daten,
      boolean nurPflichtFelder)
  {
    // if (objekt.getType().getPid().equals("typ.anzeigeQuerschnitt")
    // || objekt.getType().getPid().equals("typ.messQuerschnittVirtuell")
    // || objekt.getType().getPid().equals("typ.messQuerschnittHandZhlung"))
    // {
    // if (!objekt.getType().getAttributeGroups().contains(atg))
    // {
    // throw new IllegalArgumentException("Attributgruppe " + atg.getName() + " kann nicht mit Systemobjekt "
    // + objekt.getPid() + "verwendet werden ");
    // }
    //
    // Data data;
    // if (daten == null)
    // {
    // data = AttributeBaseValueDataFactory.createAdapter(atg, AttributeHelper.getAttributesValues(atg));
    // // Default und undefiniert Werte setzen
    // data.setToDefault();
    // }
    // else
    // {
    // data = daten.createModifiableCopy();
    // }
    //
    // if (atg.getPid().equals("atg.messQuerschnittAllgemein"))
    // {
    // data.getItem("Typ").asTextValue().setText("HauptFahrbahn");
    // if (atg.getPid().equals("atg.messQuerschnittAllgemein"))
    // {
    // data.getItem("ErsatzMessQuerschnitt").asReferenceValue().setSystemObject(null);
    // }
    //
    // return data;
    // }
    // }

    return super.getDatensatz(ho, objekt, atg, daten, nurPflichtFelder);
  }

  @Override
  public void beobachterAnmelden(BildungsRegelnBeobachter beobachter)
  {
    _beobachter.add(beobachter);
  }

  @Override
  public void beobachterAbmelden(BildungsRegelnBeobachter beobachter)
  {
    _beobachter.remove(beobachter);
  }

  @Override
  public void alleBeobachterInformieren(String feldName, String wert)
  {
    for (BildungsRegelnBeobachter beobachter : _beobachter)
    {
      beobachter.wertGeaendert(feldName, wert);
    }
  }

  @Override
  public void wertGeaendert(String feldName, String wert)
  {
    if (feldName.equals("Typ"))
    {
      if (wert.equals("HFB"))
      {
        alleBeobachterInformieren("TypName", "");
      }
      else if (wert.equals("NFB"))
      {
        alleBeobachterInformieren("TypName", "NFB");
      }
      else if (wert.equals("Aus"))
      {
        alleBeobachterInformieren("TypName", "Ausfahrt");
      }
      else if (wert.equals("Ein"))
      {
        alleBeobachterInformieren("TypName", "Einfahrt");
      }
    }
  }

}
