/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis;

import java.util.Iterator;

import org.eclipse.core.runtime.Assert;

import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeBaseValueDataFactory;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeHelper;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.ReferenceAttributeType;
import de.bsvrz.dav.daf.main.config.ReferenceType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

/**
 * Factory-Methoden fr Datenstze vom Typ Data
 * 
 * @author Sans
 */
public class DataFactory
{
  /**
   * Erzeugen eines Datensatzes fr eine Attributgruppe
   * 
   * @param atg die Attributgruppe, fr die ein Datensatz erzeugt werden soll
   * 
   * @return einen Datensatz fr die bergebene Attributgruppe
   */
  public static Data createData(AttributeGroup atg)
  {
    return AttributeBaseValueDataFactory.createAdapter(atg, AttributeHelper.getAttributesValues(atg));
  }
  
  /**
   * Erzeugen einer nderbaren Kopie eines Datensatzes
   * 
   * @param data der zu kopierende Datensatz
   * @param lookUp Lookup-Objekt zum Zugriff auf neue Objekte
   * 
   * @return den kopierten Datensatz
   */
  public static Data createModifiableCopy(Data data, ObjectLookup lookUp)
  {
    Data result = data.createModifiableCopy();
    bereinige(data, result, lookUp);
    
    return result;
  }
  
  /**
   * Es wird der Umstand bereinigt, dass beim Kopieren eines Datensatzes mit {@link Data#createModifiableCopy()}
   * fr nichtauflsbare, optionale assoziative Referenzen die Pid des referenzierten Objekts nicht mitkopiert 
   * wird.
   * 
   * @param src Quelldatensatz
   * @param dest Zieldatensatz
   * @param lookUp Lookup-Objekt zum Zugriff auf neue Objekte
   */
  private static void bereinige(Data src, Data dest, ObjectLookup lookUp)
  {
//    Assert.isLegal(src.getAttributeType().equals(dest.getAttributeType()),
//        "Die Attributtypen beider bergebener Datenstze mssen gleich sein");
    
    if (src.isList())
    {  
      Assert.isTrue(dest.isList());
    
      Iterator srcIt = src.iterator();
      Iterator destIt = dest.iterator();
      
      while (srcIt.hasNext())
      {
        Assert.isTrue(destIt.hasNext());
        Data srcNext = (Data)srcIt.next();
        Data destNext = (Data)destIt.next();
        bereinige(srcNext, destNext, lookUp);
      }
    }
    else if (src.isArray())
    {
      Assert.isTrue(dest.isArray());
      
      Array srcArray = src.asArray();
      Array destArray = dest.asArray();
      
      Assert.isTrue(srcArray.getLength() == destArray.getLength());
      
      for (int i = 0; i < srcArray.getLength(); i += 1)
      {
        bereinige(srcArray.getItem(i), destArray.getItem(i), lookUp);
      }
    }
    else if (src.isPlain())
    {
      Assert.isTrue(dest.isPlain());
      
      if (src.getAttributeType() instanceof ReferenceAttributeType)
      {
        ReferenceAttributeType srcAttType = (ReferenceAttributeType)src.getAttributeType();
       
        if (srcAttType.isUndefinedAllowed() && srcAttType.getReferenceType() == ReferenceType.ASSOCIATION)
        {  
          Assert.isTrue(dest.getAttributeType() instanceof ReferenceAttributeType);
          ReferenceAttributeType destAttType = (ReferenceAttributeType)dest.getAttributeType();
          
          Assert.isTrue(destAttType.isUndefinedAllowed() && 
              destAttType.getReferenceType() == ReferenceType.ASSOCIATION);
          
          ReferenceValue srcRef = src.asReferenceValue();
                String pid = srcRef.getSystemObjectPid();
          
          if (pid != null && ! pid.isEmpty())
          {
            dest.asReferenceValue().setSystemObjectPid(pid, lookUp);
          }
        }  
      }
    }
  }  
  
}
