/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis;


import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.konfigass.MeldungenSpeicher;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.DataEditorComposite;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.DatenBereichEditorKapsel;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.DatenBereichSOKapsel;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.ScrolledDatenBereichEditorComposite;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.TabbedDatenBereichEditorComposite;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.sys.funclib.dataSerializer.Deserializer;
import de.bsvrz.sys.funclib.dataSerializer.NoSuchVersionException;
import de.bsvrz.sys.funclib.dataSerializer.Serializer;
import de.bsvrz.sys.funclib.dataSerializer.SerializingFactory;

/**
 * ber den Dialog {@link DefaultParameter} werden Defaultparameter-Datenstzr an den
 * einzelenen Systemobjekten eines Hierarchieobjekts angezeigt und verwaltet, d .h. neu-
 * angelegt, editiert und gelscht.
 * 
 * @author Sans
 *
 */
public class DefaultParameter extends Dialog implements SelectionListener
{
  private Shell shell = null; // @jve:decl-index=0:visual-constraint="96,108"

  private Composite composite = null;

  private Composite compositeBottom = null;

  private Button buttonNeu = null;
  
  private Button buttonAllesUebernehmen = null;
  
  private Button buttonUebernehmen = null;
  
  private Button buttonLoeschen = null;
  
  private Button buttonAbbrechen = null;
  
  private DataModel _dataModel;

  private String _secondViewId;

  private TabbedDatenBereichEditorComposite _editor;

  private HierarchieObjekt _hierarchieObjekt;
  
  private Map<String, Integer> _serialIds = new HashMap<String, Integer>();
  
  private Map<String, SystemObjectType> _parentType = new HashMap<String, SystemObjectType>();
  
  /**
   * Konstruktor fr eine neue Instanz der Klasse {@link DefaultParameter}
   *
   */
  public DefaultParameter()
  {
    super(Display.getDefault().getActiveShell());
  }
  
 /**
  * ffnet den modalen Dialog {@link DefaultParameter} zum Verwalten von Defaultparameter-Datenstzen
  * an den Systemobjketen eines Hierarchieobjekts.
  * 
  * @param hierarchieObjekt das Hierarchieobjekt, fr das Parameterdatenstze bearbeitet werden sollen
  * @param dataModel das Datenmodell zum Zugriff auf die Konfiguration 
  * @param secId die sekundre ID zum Zugriff auf die zughrige TKABasisView-Instanz 
  * @param editable bestimmt, ob die Daten editierbar sind oder nicht
  * @return Liefert <code>true</code>, wenn der Dialog korrekt angezeigt wurde
  */
  public boolean oeffneDialog(HierarchieObjekt hierarchieObjekt, DataModel dataModel, String secId,
      boolean editable)
  {
    _hierarchieObjekt = hierarchieObjekt;
    _dataModel = dataModel;
    _secondViewId = secId;

    Shell parent = getParent();
    shell = new Shell(parent, SWT.TITLE | SWT.APPLICATION_MODAL | SWT.RESIZE | SWT.MAX | SWT.CLOSE);
    shell.setText("Default Parameter - Hierarchieobjekt " + hierarchieObjekt);
    shell.setLayout(new GridLayout());
    createComposite();
    createCompositeBottom();
    
    _editor = new TabbedDatenBereichEditorComposite(composite, SWT.NONE, _dataModel, _secondViewId, false, buttonUebernehmen);
    
    boolean erzeugt = erzeugeDatenbereich();
    
    if (!erzeugt)
    {
//      System.out.println("Datenbereich nicht erzeugt, weil Hierarchieobjekt null");
      return false;
    }
    
    if ( _editor != null && _editor.getAktuelleTextPid() != null)
    {
      _editor.setEditable(editable);   
      _editor.getAktuelleTextPid().setEditable(false);
      _editor.getAktuellenTextName().setEditable(false);
    }
    else
    {
//      System.out.println("Editor: " + _editor);
//      System.out.println("Aktuelle Pid: " + _editor.getAktuelleTextPid());
      return false;      
    }

    
    shell.addListener(SWT.Close, new Listener() 
    {
      public void handleEvent(Event event)
      {
        if (_editor.isEditable() && !hatKeineDaten() && (buttonAllesUebernehmen.isEnabled() || buttonUebernehmen.isEnabled()))
        {  
          MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), 
              SWT.OK | SWT.CANCEL | SWT.ICON_QUESTION);
          msg.setText("Schlieen des Defaultparameter-Editors");
          msg.setMessage("Sollen evtl. nicht bernommene nderungen in den Daten verworfen werden?"); 
          
          event.doit = msg.open() == SWT.OK;
        }  
      }
    });  
    
    _editor.getTabFolder().addSelectionListener(this);
    CTabItem[] items = _editor.getTabFolder().getItems();
    for (CTabItem cTabItem : items)
    {
      int indexOf = _editor.getTabFolder().indexOf(cTabItem);
      _editor.getTabFolderSo(indexOf).addSelectionListener(this);  
    }
     
    buttonSteuerung();

    shell.open();

    Display display = parent.getDisplay();

    while (!shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return true;
  }

  private boolean hatKeineDaten()
  {
    boolean result = true;
    
    for (DatenBereichEditorKapsel kapsel: _editor.getKapselnZumSichern())
    {
      if (kapsel != null &&  kapsel.getDataEmpfangen() != null)
      {
        // Daten vorhanden
        result = false;
        break;
      }
    }
    
    return result;
  }
  
  private void buttonSteuerung()
  {
    DatenBereichEditorKapsel kapsel = getSelektierteKapsel();

    if (_editor != null && kapsel != null)
    {
      boolean panelDirty = _editor.isSelectedAtgPanelDirty();
    
      boolean aenderbar = _editor.isEditable() && kapsel != null;
//      System.out.println("ButtonSteuerung: nderbar: " + aenderbar + " Kapsel.DataEmpfangen: " + kapsel.getDataEmpfangen());
      buttonNeu.setEnabled(aenderbar && kapsel.getDataEmpfangen() == null);
//      System.out.println("buttonAllesUebernehmen setzen: " + aenderbar + " && " + !hatKeineDaten() + " && " +_editor.isDirty());
      buttonAllesUebernehmen.setEnabled(aenderbar && !hatKeineDaten() && _editor.isDirty());
      buttonUebernehmen.setEnabled(aenderbar && kapsel.getDataEmpfangen() != null && panelDirty);
      buttonLoeschen.setEnabled(aenderbar && kapsel.getDataEmpfangen() != null);
      buttonAbbrechen.setEnabled(true);
    }
    else
    {
      buttonNeu.setEnabled(false);
      buttonAllesUebernehmen.setEnabled(false);
      buttonUebernehmen.setEnabled(false);
      buttonLoeschen.setEnabled(false);
      buttonAbbrechen.setEnabled(true);      
    }
   
  }
  
  
  /**
   * Datenbereich erzeugen
   * 
   * @return Flag, fr Datenbereich neu erzeugt
   */
  private boolean erzeugeDatenbereich()
  {
    // Datenbereich leeren wird in createTabPage gemacht
//    if (_editor != null)
//    {
//      CTabFolder tabFolder = _editor.getTabFolder();
//            
//      CTabItem[] items = tabFolder.getItems();
//      for (CTabItem tabItem : items)
//      {
//        tabItem.getControl().dispose();
//        tabItem.dispose();
//      }
//    }
  
    // Datenbereich mit den Daten neu erzeugen
    if (_hierarchieObjekt != null)
    {
      List<SystemObject> systemObjekte = _hierarchieObjekt.getSystemObjekte();
      int numSo = 0;
      for (SystemObject so: systemObjekte)
      {
        DatenBereichSOKapsel kapselSo = new DatenBereichSOKapsel(so, _hierarchieObjekt);
        
        SystemObjectType typ = so.getType();
        List<AttributeGroup> attributGruppen = new Vector<AttributeGroup>(typ.getAttributeGroups());
        
        Collections.sort(attributGruppen, 
            new Comparator<AttributeGroup>()
            {
              public int compare(AttributeGroup atg1, AttributeGroup atg2)
              {
                return String.CASE_INSENSITIVE_ORDER.compare(atg1.getPidOrId(), atg2.getPidOrId());
              }
            });
        
        String aspPid = "asp.parameterDefault";
        
        int numAtg = 0;
        for (AttributeGroup atg: attributGruppen)
        {
          if (atg.isParameter())
          {
            DatenBereichEditorKapsel kapsel = new DatenBereichEditorKapsel(kapselSo, atg.getPid(), aspPid);
            ScrolledDatenBereichEditorComposite page;
            if ( numAtg == 0 && numSo == 0 )
              page = _editor.createTabPage(kapsel, true, true);  
            else if ( numAtg == 0 && numSo > 0 )
              page = _editor.createTabPage(kapsel, false, true);  
            else
              page = _editor.createTabPage(kapsel, false, false); 
            
            //falls die virtuelle Darstellung zu Problemen fhrt, kann dies hier und an einer weiteren Stelle ausgeschaltet werden
//            page.setVirtuell(false);
            
            Data paramSatz = getDefParamSatzObjekt(kapsel);

            if (paramSatz != null)
            {  
              DataEditorComposite panel = page.getDataEditorPanel();
              if (panel != null)
              {
                ObjectLookup lookUp = LookupFactory.createLookupForModifiableVersion((ConfigDataModel)_dataModel);
                Data data = DataFactory.createModifiableCopy(paramSatz, lookUp);
                
                kapsel.setDataEmpfangen(data);
                panel.setData(kapsel.getDataEmpfangen());
                _editor.setDirty(panel, false);
//                _editor.addListenersRecursive(panel);
                if (numAtg == 0)
                  page.erzeugeEditorInhalt();
              }
            }  
            numAtg++;
          }
        }
        numSo++;
      }
      if (_editor != null && numSo > 0)
        _editor.setSelektionTab(0, 0);
      
      return true;
    }

    else
    {
      _editor.loescheDarstellungsBereich();
    }
    return false;
  }

  private void aktionNeu()
  {
    DatenBereichEditorKapsel kapsel = getSelektierteKapsel();

    ScrolledDatenBereichEditorComposite page = _editor.getAktuellesScrollPanel();
//    page.setVirtuell(false);
    DataEditorComposite panel = _editor.getAktuellesPanel();
    if (panel != null)
    {
      panel.setVisible(false);
      Data paramSatz = getDefParamSatzVorbelegung(kapsel);
      
      ObjectLookup lookUp = LookupFactory.createLookupForModifiableVersion((ConfigDataModel)_dataModel);
      Data data = DataFactory.createModifiableCopy(paramSatz, lookUp);
      
      kapsel.setDataEmpfangen(paramSatz != null ? data : paramSatz);

      panel.setDataGroup(kapsel.getDataEmpfangen());
      
      _editor.setDirty(panel, true);
      panel.setVisible(true);
    }
    
    buttonSteuerung();
  }

  private DatenBereichEditorKapsel getSelektierteKapsel()
  {
//    DatenBereichEditorKapsel result = null;
//    
//    CTabItem sel = _editor.getTabFolderSo().getSelection();
//    if (sel != null)
//    {
//      Object object = sel.getData("kapsel");      
//      if (object instanceof DatenBereichEditorKapsel)
//      {
//        result = (DatenBereichEditorKapsel)object;
//      }
//    }
    return _editor.getSelektierteKapsel();
  }
  
  private void aktionAllesUebernehmen()
  {
    boolean error = false;
    MultiStatus status = null;
    
    for (DatenBereichEditorKapsel kapsel: _editor.getKapselnZumSichern())
    {
      if (kapsel.getDataEmpfangen() != null)
      {  
        if (status == null)
        {
          status = new MultiStatus(Activator.PLUGIN_ID, IStatus.OK,
              "bernehme alle Defaultparameter fr Hierarchieobjekt " + _hierarchieObjekt, null);
        }
        status.merge(schreibeDaten(kapsel));
     
        if (status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
          error = true;
        }
      }  
    }
    
    if (status != null)
    {
      speichernMeldung(status, true);
    }
    
    if (error)
    {
      MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), 
          SWT.OK | SWT.ICON_ERROR);
      msg.setText("Fehler beim bernehmen");
      msg.setMessage("Beim bernehmen von Defaultparametern sind Fehler aufgetreten");
      msg.open();
    }
    else
    {
//      System.out.println("aktionAllesUebernehmen: Set Dirty");
      _editor.setDirty(false);
      buttonSteuerung();      
    }
    
  }

  private void aktionUebernehmen()
  {
    DatenBereichEditorKapsel kapsel = getSelektierteKapsel();
    
    //synchronisiere Bearbeitung
//    Map<String, ScrolledDatenBereichEditorComposite> map = _editor.getMapKapselEditorZuordnung();
//    ScrolledDatenBereichEditorComposite page = map.get(kapsel.getId());
//    DataEditorComposite panel = page.getDataEditorPanel();
//    kapsel.setDataEmpfangen(panel.getData());
   
    IStatus status = schreibeDaten(kapsel);
    
    speichernMeldung("Starte Aktion bernehmen. ", status);
    
    if (status.matches(IStatus.ERROR | IStatus.CANCEL))
    {
      MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), 
          SWT.OK | SWT.ICON_ERROR);
      msg.setText("Fehler beim bernehmen");
      msg.setMessage("Beim bernehmen der Defaultparameter der Attributgruppe " + 
          kapsel.getName() + " fr das Konfigurationsobjekt " + kapsel.getPidSystemObjekt() +
          " ist ein Fehler aufgetreten");
      msg.open();
    }
    else
    {
      DataEditorComposite panel = _editor.getAktuellesPanel();
      if ( panel != null )
        _editor.setDirty(panel, false);

      buttonSteuerung();
    }
  }
  
  private IStatus schreibeDaten(DatenBereichEditorKapsel kapsel)
  {
    IStatus result;
    
    if (kapsel.getDataEmpfangen().isDefined())
    {
      ConfigurationObject ko = getKonfigObjekt(kapsel);
      AttributeGroup atgDefault = _dataModel.getAttributeGroup("atg.defaultParameterdatenstze");
      Data data = getDefParamKonfigDatensatz(kapsel);
      
      try
      {
        ko.setConfigurationData(atgDefault, data);
        result = new Status(Status.OK, Activator.PLUGIN_ID, 
            "Defaultparameter der ATG " + kapsel.getName() + " fr das Objekt " + 
            kapsel.getPidSystemObjekt() + " erfolgreich in die Konfiguration bernommen");
      }
      catch (ConfigurationChangeException e)
      {
        result = new Status(Status.ERROR, Activator.PLUGIN_ID, 
            "Fehler beim Speichern der Defaultparameter der ATG " + kapsel.getName() + 
            " fr das Konfigurationsobjekt " + kapsel.getPidSystemObjekt(), e);
      }
    }
    else
    {
      result = new Status(Status.ERROR, Activator.PLUGIN_ID, 
          "Default-Parameter-Datensatz der ATG " + kapsel.getName() + 
          " fr das Konfigurationsobjekt " + kapsel.getPidSystemObjekt() + 
          " ist nicht vollstndig definiert");
    }
    
    return result;
  }
  
  private void aktionLoeschen()
  {
    DatenBereichEditorKapsel kapsel = getSelektierteKapsel();
    
    MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), 
        SWT.OK | SWT.CANCEL | SWT.ICON_QUESTION);
    msg.setText("Lschen von Defaultparametern");
    msg.setMessage("Sollen die Defaultparameter der Attributgruppe " + 
        kapsel.getName() + " fr das Konfigurationsobjekt " + kapsel.getPidSystemObjekt() +
        " gelscht werden?");
    
    if (msg.open() != SWT.CANCEL)
    { 
      MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Lschen Default Parameter fr Hierarchieobjekt " + _hierarchieObjekt, null);
      MeldungenSpeicher.getInstanz().setStatus(status);
      
      ConfigurationObject ko = getKonfigObjekt(kapsel);
      AttributeGroup atg = _dataModel.getAttributeGroup(kapsel.getName());
      AttributeGroup atgDefault = _dataModel.getAttributeGroup("atg.defaultParameterdatenstze");
      
      Data data = ko.getConfigurationData(atgDefault);
      if (data != null)
      {
        SystemObjectType typ = _parentType.get(kapsel.getId());
        data = bereinigeDefParamKonfigDatensatz(ko, typ, atg, data, false);
        try
        {
          ko.setConfigurationData(atgDefault, data);
          status.merge(new Status(Status.OK, Activator.PLUGIN_ID, 
              "Defaultparameter der ATG " + atg.getPid() + " fr das Objekt " + ko.getPid() + 
              " erfolgreich aus der Konfiguration gelscht"));
          speichernMeldung(status, true);
        }
        catch (ConfigurationChangeException e)
        {
          status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, 
              "Fehler beim Lschen der Defaultparameter der ATG " + atg.getPid() + 
              " fr das Konfigurationsobjekt " + ko.getPid(), e));
          speichernMeldung(status, false);          
          
          msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), 
              SWT.OK | SWT.ICON_ERROR);
          msg.setText("Fehler beim Lschen");
          msg.setMessage("Beim Lschen der Defaultparameter der Attributgruppe " + 
              kapsel.getName() + " fr das Konfigurationsobjekt " + kapsel.getPidSystemObjekt() +
              " ist ein Fehler aufgetreten");
          msg.open();
          
          return;
        }
      }
      
      
      DataEditorComposite panel = _editor.getAktuellesPanel();
      kapsel.setDataEmpfangen(null);
      if ( panel != null )
      {
        panel.setDataGroup(null);
        //Setzen auf False, weil die Daten bereits gleich gespeichert werden
        _editor.setDirty(panel, false);
      }
      
      _serialIds.remove(kapsel.getId());
      
//      System.out.println("Lsche Daten des Typs: " + _parentType.get(kapsel.getId()));
      _parentType.remove(kapsel.getId());
      
      buttonSteuerung();
    }  
  }

  /**
   * Schliet den Dialog {@link DefaultParameter} ohne bernahme der Einstellungen.
   * 
   */
  private void aktionAbbrechen()
  {
     shell.close();
  }

  /**
   * This method initializes composite
   * 
   */
  private void createComposite()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;
    FillLayout fillLayout = new FillLayout();
    fillLayout.type = org.eclipse.swt.SWT.HORIZONTAL;
    composite = new Composite(shell, SWT.None);
    composite.setLayout(fillLayout);  
    composite.setLayoutData(gridData1);
    
    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(composite, Activator.PLUGIN_ID + ".defaultparameter");
      
  }

  /**
   * This method initializes compositeBottom
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(shell, SWT.NONE);
    compositeBottom.setLayoutData(gridData3);
    compositeBottom.setLayout(rowLayout);
    
    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(compositeBottom, Activator.PLUGIN_ID + ".defaultparameter");
    
        
    buttonNeu = new Button(compositeBottom, SWT.NONE);
    buttonNeu.setText("Neu");
    buttonNeu.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionNeu();

      }
    
    });
    
    buttonAllesUebernehmen = new Button(compositeBottom, SWT.NONE);
    buttonAllesUebernehmen.setText("Alle bernehmen");
    buttonAllesUebernehmen.setEnabled(false);
    buttonAllesUebernehmen.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionAllesUebernehmen();

      }

    });
    buttonUebernehmen = new Button(compositeBottom, SWT.NONE);
    buttonUebernehmen.setText("bernehmen");
    buttonUebernehmen.setEnabled(false);
    buttonUebernehmen.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionUebernehmen();

      }
    
    });
    
    buttonLoeschen = new Button(compositeBottom, SWT.NONE);
    buttonLoeschen.setText("Lschen");
    buttonLoeschen.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionLoeschen();

      }
    
    });

    buttonAbbrechen = new Button(compositeBottom, SWT.NONE);
    buttonAbbrechen.setText("Abbrechen");
    buttonAbbrechen.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionAbbrechen();

      }
    
    });

  }
  
  /**
   * Serialisiert die Empfangs-Daten von kapsel und stellt sie als Array-Element
   * des Arrays Default-Parameterdatensatz der ATG Default-Parameterdatenstze
   * zur Verfgung.
   * 
   * @param kapsel kapselt die Daten (ATG-PID, Systemobject-PID), die zur Spezifikation 
   * bentigt werden.
   * 
   * @return Datensatz, der ein Array-Element des Arrays Default-Parameterdatensatz 
   * der ATG Default-Parameterdatenstze darstellt.
   */
  private Data getDefParamKonfigDatensatz(DatenBereichEditorKapsel kapsel)
  {
    Data result;
    Data data = kapsel.getDataEmpfangen();
    ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
    Serializer serializer;
    Integer serialId = _serialIds.get(kapsel.getId());
    
    try
    {
      serializer = SerializingFactory.createSerializer(serialId != null ? serialId.intValue(): 3, outputStream);
      serializer.writeData(data);
    }
    catch (NoSuchVersionException e)
    {
      throw new RuntimeException(e);
    }
    catch (IOException e)
    {
      throw new RuntimeException(e);
    }
   
    ConfigurationObject ko = getKonfigObjekt(kapsel);
    AttributeGroup atg = _dataModel.getAttributeGroup(kapsel.getName());
    int serialisierer = serializer.getVersion();
    byte[] byteArray = outputStream.toByteArray();
    
    AttributeGroup atgDefault = _dataModel.getAttributeGroup("atg.defaultParameterdatenstze");
    Data dataDefault = ko.getConfigurationData(atgDefault);
    Array arrayDefault;
    
    SystemObjectType typ = _parentType.get(kapsel.getId());
//  System.out.println("Schreibe Daten mit Typ: " + typ.getPid());
         
    if (dataDefault == null)
    {
      result = DataFactory.createData(atgDefault);
      arrayDefault = result.getItem("Default-Parameterdatensatz").asArray();
      arrayDefault.setLength(1);
    }
    else
    {
      result = bereinigeDefParamKonfigDatensatz(ko, typ, atg, dataDefault, true);
      arrayDefault = result.getItem("Default-Parameterdatensatz").asArray();
    }  
    
    Data arrayElem = arrayDefault.getItem(0);
    arrayElem.getItem("typ").asReferenceValue().setSystemObject(typ != null ? typ : ko.getType());
    arrayElem.getItem("attributgruppe").asReferenceValue().setSystemObject(atg);
    arrayElem.getItem("serialisierer").asUnscaledValue().set(serialisierer);
    Array array = arrayElem.getItem("datensatz").asArray();
    array.setLength(byteArray.length);
    for (int i = 0; i < array.getLength(); i += 1)
    {
      array.getItem(i).asScaledValue().set(byteArray[i]);
    }
    
    return result;
  }
  
  private Data bereinigeDefParamKonfigDatensatz(SystemObject so, SystemObjectType parentTyp, AttributeGroup atg, 
      Data data, boolean reserviereErstenPlatz)
  {
    AttributeGroup atgDefault = _dataModel.getAttributeGroup("atg.defaultParameterdatenstze");
    Data result = DataFactory.createData(atgDefault);
    Array array = result.getItem("Default-Parameterdatensatz").asArray();
    array.setLength(reserviereErstenPlatz ? 1 : 0);
    Array arrayDefault = data.getItem("Default-Parameterdatensatz").asArray();
    
    for (int i = 0; i < arrayDefault.getLength(); i += 1)
    {
      Data arrayDefaultEl = arrayDefault.getItem(i);
      SystemObject typ = arrayDefaultEl.getItem("typ").asReferenceValue().getSystemObject();
      SystemObject attributgruppe  = arrayDefaultEl.getItem("attributgruppe").asReferenceValue().getSystemObject();
      
      if (!(atg.equals(attributgruppe) && parentTyp.equals(typ)))
      {
        int len = array.getLength() + 1;
        array.setLength(len);
        Data arrayEl = array.getItem(len - 1);
        
        arrayEl.getItem("typ").asReferenceValue().setSystemObject(typ);
        arrayEl.getItem("attributgruppe").asReferenceValue().setSystemObject(attributgruppe);
        
        int serialisierer = arrayDefaultEl.getItem("serialisierer").asUnscaledValue().intValue();
        byte[] datensatz = arrayDefaultEl.getItem("datensatz").asScaledArray().getByteArray();
        
        arrayEl.getItem("serialisierer").asScaledValue().set(serialisierer);
        Array byteArray = arrayEl.getItem("datensatz").asArray();
        byteArray.setLength(datensatz.length);
        for (int j = 0; j < byteArray.getLength(); j += 1)
        {
          byteArray.getItem(j).asScaledValue().set(datensatz[j]);
        }
      }
    }
    
    if (array.getLength() == 0)
    {
      result = null;  
    }
    
    return result;
  }
  
  private Data findeDefaultSatzInDaten(Data data, SystemObjectType typ, AttributeGroup atg)
  {
    Data result = null;
    
    Array array = data.getItem("Default-Parameterdatensatz").asArray();
    
    if (array != null)
    {  
      for (int i = 0; i < array.getLength(); i +=1)
      {
        Data elem = array.getItem(i);
        SystemObject soTyp = elem.getItem("typ").asReferenceValue().getSystemObject();
        if (typ.equals(soTyp) || (soTyp instanceof SystemObjectType && typ.inheritsFrom( (SystemObjectType)soTyp)))
        {  
          SystemObject paramAtg = elem.getItem("attributgruppe").asReferenceValue().getSystemObject();
          if (atg.equals(paramAtg))
          {
            result = elem;
            break;
          }
        }  
      }
    }  
    return result;
  }
  
  private Data findeDefaultSatzInSupertypen(SystemObjectType typ, AttributeGroup atg)
  {
    Data result = null;
    
    if (!typ.isBaseType())
    {  
      for (SystemObjectType superTyp: typ.getSuperTypes())
      {
//        Data dataDefault = findeDefaultSatz(superTyp, atg);
//        if (dataDefault != null)
//        {
//          Data dataSatz = findeDefaultSatzInDaten(dataDefault, superTyp, atg);
//          if (dataSatz != null)
//          {
//            result = dataSatz;
//            break;
//          }
//        }
        Data data = findeDefaultSatz(superTyp, atg);
        if (data != null)
        {
          result = data;
          break;
        }
      }
    }  
    return result;
  }
  
  private Data findeDefaultSatz(SystemObjectType typ, AttributeGroup atg)
  {
    Data result = null;
    AttributeGroup atgDefault = typ.getDataModel().getAttributeGroup("atg.defaultParameterdatenstze");
    
    Data data = typ.getConfigurationData(atgDefault);
    
    if (data == null)
    {
      result = findeDefaultSatzInSupertypen(typ, atg);
    }
    else
    {
      result = findeDefaultSatz(data, typ, atg);
      if (result == null)
      {
        result = findeDefaultSatzInSupertypen(typ, atg);
      }
    }  
     
    return result;
  }
  
  private Data findeDefaultSatz(Data data, SystemObjectType typ, AttributeGroup atg)
  {
    Data result = null;
    
    Array array = data.getItem("Default-Parameterdatensatz").asArray();
    
    if (array != null)
    {  
      for (int i = 0; i < array.getLength(); i +=1)
      {
        Data elem = array.getItem(i);
        SystemObject soTyp = elem.getItem("typ").asReferenceValue().getSystemObject();
        SystemObject paramAtg = elem.getItem("attributgruppe").asReferenceValue().getSystemObject();
        if (typ.equals(soTyp) && atg.equals(paramAtg))
        {
          result = elem;
          break;
        }
      }
    }  
    return result;
  }
  
  private Data deserialisiereDefaultSatz(Data data, AttributeGroup atg, int serialisierer)
  {
    Data result = null;
  
    byte[] datensatz = data.getItem("datensatz").asScaledArray().getByteArray();
    if (datensatz != null) 
    {
      try 
      {
        Deserializer deserializer = SerializingFactory.createDeserializer(serialisierer,
            new ByteArrayInputStream(datensatz));
        result = deserializer.readData(atg, _dataModel);
      } 
      catch (NoSuchVersionException e)
      {
        throw new RuntimeException(e);
      }
      catch (IOException e) 
      {
        throw new RuntimeException(e);
      }
    }
    
    return result;
      
  }
  
  private Data getDefParamSatzVorbelegung(DatenBereichEditorKapsel kapsel)
  {
    Data result = null;
    SystemObject so = getKonfigObjekt(kapsel);
    SystemObjectType typ = so.getType();
    AttributeGroup atg = _dataModel.getAttributeGroup(kapsel.getName());
    
    Data data = findeDefaultSatz(typ, atg);
    if (data != null)
    {
      int serialisierer = data.getItem("serialisierer").asUnscaledValue().intValue();
      result = deserialisiereDefaultSatz(data, atg, serialisierer);
      if (result != null)
      {
        _serialIds.put(kapsel.getId(), new Integer(serialisierer));
        SystemObject soTyp = data.getItem("typ").asReferenceValue().getSystemObject();
        if (soTyp instanceof SystemObjectType)
        {
//          System.out.println("Lese Daten des Typs: " + soTyp.getPid());
          _parentType.put(kapsel.getId(), (SystemObjectType)soTyp); 
        }
        else
        {
          _parentType.remove(kapsel.getId());
        }
      }
    }
    
    if (result == null)
    {
      result = DataFactory.createData(atg);
      // Default und undefinierte Werte setzen
      result.setToDefault();
    }
    
    return result;
  }
  
  private Data getDefParamSatzObjekt(DatenBereichEditorKapsel kapsel)
  {
    Data result = null;
    SystemObject so = getKonfigObjekt(kapsel);
    SystemObjectType typ = so.getType();
    AttributeGroup atg = _dataModel.getAttributeGroup(kapsel.getName());
    AttributeGroup atgDefault = typ.getDataModel().getAttributeGroup("atg.defaultParameterdatenstze");
    
    Data data = so.getConfigurationData(atgDefault);
    
    if (data != null)
    {
      Data satz = findeDefaultSatzInDaten(data, typ, atg);
      if (satz != null)
      {
        int serialisierer = satz.getItem("serialisierer").asUnscaledValue().intValue();
        result = deserialisiereDefaultSatz(satz, atg, serialisierer);
        if (result != null)
        {
          _serialIds.put(kapsel.getId(), new Integer(serialisierer));
          SystemObject soTyp = satz.getItem("typ").asReferenceValue().getSystemObject();
          if (soTyp instanceof SystemObjectType)
          {
//            System.out.println("Lese Daten des Typs: " + soTyp.getPid());
            _parentType.put(kapsel.getId(), (SystemObjectType)soTyp); 
          }
          else
          {
            _parentType.remove(kapsel.getId());
          }
        }
      }  
    }
    
    return result;
  }
  
  
  
  private ConfigurationObject getKonfigObjekt(DatenBereichEditorKapsel kapsel)
  {
    return getKonfigObjekt(kapsel.getElternKapsel());
  }
  
  private ConfigurationObject getKonfigObjekt(DatenBereichSOKapsel kapsel)
  {
    ConfigurationObject result = null;
    
    for (SystemObject so: kapsel.getHierarchieObjekt().getSystemObjekte())
    {
      if (so.getPid().equals(kapsel.getPidSystemObjekt()))
      {
        result = (ConfigurationObject)so;
        break;
      }
    }
    
    return result;
  }
  
  private void speichernMeldung(IStatus status, boolean ausgabeErwartet)
  {
	  MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(status, System.currentTimeMillis(), ausgabeErwartet);  
  }
  
  private void speichernMeldung(String aktion, IStatus status)
  {
    MultiStatus multiStatus = new MultiStatus(Activator.PLUGIN_ID, IStatus.OK, aktion, null);
    multiStatus.merge(status);
    MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(multiStatus, System.currentTimeMillis(), true);
  }
  
  public void widgetSelected(SelectionEvent e)
  {
    buttonSteuerung();
  }

  public void widgetDefaultSelected(SelectionEvent e)
  {
    buttonSteuerung();
  }
  
  /**
   * Ermittelt, ob das HierarchieObjekt DefaultParameter hat
   * 
   * @param hierarchieObjekt  HierarchieObjekt
   * @param dataModel         Datenmodell
   * @return  hat DefaultParameter
   */
  public static boolean hatDefaultParameter(HierarchieObjekt hierarchieObjekt, DataModel dataModel)
  {
    boolean parameter = false;
    
    List<SystemObject> systemObjekte = hierarchieObjekt.getSystemObjekte();
    for (SystemObject so: systemObjekte)
    {
      SystemObjectType typ = so.getType();
      List<AttributeGroup> attributGruppen = typ.getAttributeGroups();
      
      for (AttributeGroup atg: attributGruppen)
      {
        if (atg.isParameter())
        {
          parameter = true;
          break;
        }
      }
    }
    
    return parameter;
  }
}
