/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.progress.UIJob;

import de.bsvrz.buv.plugin.konfigass.KonfigAssListDialog;
import de.bsvrz.buv.plugin.konfigass.MeldungenSpeicher;
import de.bsvrz.buv.plugin.konfigass.datenModellUpdateListener.DatenModellUpdateListener;
import de.bsvrz.buv.plugin.tkabasis.aktionen.ActionRegistry;
import de.bsvrz.buv.plugin.tkabasis.aktionen.MyAction;
import de.bsvrz.buv.plugin.tkabasis.aktionen.AktionFactory.TKABasisAktion;
import de.bsvrz.buv.plugin.tkabasis.assistenten.InfoKonfigAenderungAssistent2;
import de.bsvrz.buv.plugin.tkabasis.assistenten.InfoKonfigAenderungModell2;
import de.bsvrz.buv.plugin.tkabasis.assistenten.InfoSystemObjektAssistent;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.datenbasis.DatenidentifikationGtmOffline;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.DatenBereichEditorKapsel;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.DatenBereichSOKapsel;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.TabbedDatenBereichEditorComposite;
import de.bsvrz.buv.plugin.tkabasis.datenbereich.editor.DatenBereichEditorKapsel.TYP;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HODaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HOMitStatus;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.KBHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.SystemObjektDaten;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkabasis.navigator.NavigatorComposite;
import de.bsvrz.buv.plugin.tkabasis.navigator.NavigatorComposite.DNDModus;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.AbstractNavigatorElement;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.HONavigatorElement;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.INavigatorElement;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.KBNavigatorElement;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.NavigatorElement;
import de.bsvrz.buv.plugin.tkabasis.preferences.DefsPara;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.LoeschenChecker;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnMitKB;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.buv.plugin.tkabasis.tkaUpdateManagement.TKAUpdate;
import de.bsvrz.buv.plugin.tkabasis.zustaende.EndlicherAutomat;
import de.bsvrz.buv.plugin.tkabasis.zustaende.Zustand;
import de.bsvrz.buv.plugin.tkabasis.zustaende.ZustandsFactory;
import de.bsvrz.buv.plugin.tkabasis.zustaende.ZustandsFactory.Bearbeitungsmodus;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeBaseValueDataFactory;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeHelper;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ObjectSetUse;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;

public class TKABasisComposite extends Composite
{
  /**
   * ID von diesem Composites
   */
  public static final String ID = "de.bsvrz.buv.plugin.tkabasis.konfigAssKonfigurationComposite";

  /**
   * Debug-Logger fr Logging-Ausgaben
   */
//  private final static Debug _debug = Debug.getLogger();
    
  private static final Color[] BMFARBEN = { new Color(null, 173, 255, 47), // Hellgrn
      new Color(null, 250, 218, 94), // Hellgelb
      new Color(null, 228, 113, 122) // Hellrot
  };

  /**
   * Flag zum Debuggen
   */
  private static final boolean SYS_OUT = false;
  
  private Composite _composite;

  private Composite composite;

  private Composite composite1;

  private Composite composite2;

  private Composite composite4;

  private Composite composite5;

  private Composite composite7;

  private NavigatorComposite _navigatorComposite = null;

  private Group group1;

  private Group group4;

  private Group group5;

  private Group group6;

  private ScrolledComposite _sc;

  private SashForm _sf;

  private Label label1;

  private Button buttonNeu = null;

  private Button buttonNeuUO = null;

  private Button buttonKopieren = null;

  private Button buttonDefault = null;

  private Button buttonPruefen = null;

  private Button buttonUebernehmen = null;

  private Button buttonLoeschen = null;

  private Button[] radioButton = null;

  private ConfigDataModel _dataModel;

  private String _secondViewId = "";

  private TabbedDatenBereichEditorComposite _editor;

  private ActionRegistry _actionRegistry;

  /**
   * Die Zustandsmaschine
   */
  private EndlicherAutomat _zustandsmaschine;

  private enum KopierModus {
    KEIN, NORMAL, TIEF, VERSCHIEBEN
  };

  private KopierModus _kopierModus = KopierModus.KEIN;;

  private List<HONavigatorElement> _kopierteHoNes = new ArrayList<HONavigatorElement>();

  private boolean _aktualisiereEditor = true;

  private boolean _ausgabeWarnungAenderung = true;

  private INavigatorElement _rootNavigatorElement;
  
  /**
   * Aktion, die den bergang in den bergang in den aktuellen Zustand bewirkt hat
   */
  private TKABasisAktion _aktionDesLetztenUebergangs = null;
  
  // fr die Jobs
  private MultiStatus _status = null;

  private HONavigatorElement _neuesHONavigatorElement;
  
  TKABasisComposite(Composite parent, int style, String secondViewId)
  {
    super(parent, style);

    _secondViewId = secondViewId;

    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    this.setSize(new Point(770, 561));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.heightHint = -1;
    gridData3.widthHint = -1;
    gridData3.horizontalIndent = 0;
    gridData3.verticalAlignment = GridData.FILL;
    initialize();

  }

  private void initialize()
  {
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.heightHint = -1;
    gridData3.widthHint = -1;
    gridData3.horizontalIndent = 0;
    gridData3.verticalAlignment = GridData.FILL;
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(gridLayout);
    _composite.setLayoutData(gridData2);
    createSashComposite();

    // initialisieren der Zustandsmaschine
    // ZustandsFactory zustaende = ZustandsFactory.getInstanz();
    ZustandsFactory zustaende = new ZustandsFactory();
    _zustandsmaschine = zustaende.getZustandsmaschine();

    // alle mglichen Aktion setzen
    createActions();

    // Kontext-Men des Navigators mit den Aktionen versorgen
    versorgeKontextMenuNavigator();

    // Drag & Drop des Navigators mit den Aktionen versorgen
    versorgeDNDAktionen();

    // Defaulteinstellung des Bearbeitungsmodus
    aktionAendernBearbeitungsmodus(null, _zustandsmaschine.getMomentZustand());
  }

  /**
   * This method initializes composite
   * 
   */
  private void createSashComposite()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    composite = new Composite(_composite, SWT.NONE);

    _sf = new SashForm(composite, SWT.BORDER | SWT.HORIZONTAL);

    createNavigationsBereichComposite();
    createDarstellungsBereichComposite();

    composite.setLayout(new GridLayout(2, false));
    composite.setLayoutData(gridData);

    _sf.setWeights(new int[] { 100, 200 });

    _sf.setLayout(new GridLayout(1, false));
    _sf.setLayoutData(gridData2);
  }

  /**
   * This method initializes composite5
   * 
   */
  private void createNavigationsBereichComposite()
  {
    // System.out.println("=====>>>>> Navigator neu erzeugen");
    GridLayout gridLayout2 = new GridLayout();
    gridLayout2.numColumns = 1;
    composite5 = new Composite(_sf, SWT.NONE);
    composite5.setLayout(gridLayout2);
    createBearbeitungsModusGroup();
    _navigatorComposite = new NavigatorComposite(composite5, SWT.NONE, _secondViewId);

  }

  /**
   * This method initializes group1
   * 
   */
  private void createBearbeitungsModusGroup()
  {
    GridLayout gridLayout7 = new GridLayout();
    gridLayout7.numColumns = 1;
    gridLayout7.marginWidth = 5;
    gridLayout7.horizontalSpacing = 5;
    GridData gridData12 = new GridData();
    gridData12.horizontalAlignment = GridData.FILL;
    gridData12.grabExcessHorizontalSpace = false;
    gridData12.verticalAlignment = GridData.FILL;
    group1 = new Group(composite5, SWT.NONE);
    group1.setLayoutData(gridData12);
    group1.setLayout(gridLayout7);
    group1.setText("Bearbeitungsmodus");

    createBearbeitungsModusComposite();
  }

  /**
   * This method initializes composite4
   * 
   */
  private void createBearbeitungsModusComposite()
  {
    GridLayout gridLayout1 = new GridLayout();
    gridLayout1.numColumns = 3;
    GridData gridData14 = new GridData();
    gridData14.horizontalAlignment = GridData.FILL;
    gridData14.grabExcessHorizontalSpace = true;
    gridData14.grabExcessVerticalSpace = true;
    gridData14.verticalAlignment = GridData.FILL;
    composite4 = new Composite(group1, SWT.NONE);
    composite4.setLayoutData(gridData14);
    composite4.setLayout(gridLayout1);

    radioButton = new Button[3];

    radioButton[0] = new Button(composite4, SWT.RADIO);
    radioButton[0].setText("Darstellung");
    radioButton[0].addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        if (radioButton[0].getSelection())
        {
          aktionDarstellen();
        }
      }
    });

    radioButton[1] = new Button(composite4, SWT.RADIO);
    radioButton[1].setText("Bearbeitung");
    radioButton[1].addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        if (radioButton[1].getSelection())
        {
          aktionBearbeiten();
        }
      }
    });

    radioButton[2] = new Button(composite4, SWT.RADIO);
    radioButton[2].setText("Neuanlage");
    radioButton[2].addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        if (radioButton[2].getSelection())
        {
          aktionNeu();
        }
      }
    });

  }

  /**
   * This method initializes composite7
   * 
   */
  private void createDarstellungsBereichComposite()
  {
    GridLayout gridLayout2 = new GridLayout();
    gridLayout2.marginWidth = 5;
    gridLayout2.horizontalSpacing = 5;
    GridData gridData7 = new GridData();
    gridData7.horizontalAlignment = GridData.FILL;
    gridData7.grabExcessHorizontalSpace = true;
    gridData7.grabExcessVerticalSpace = true;
    gridData7.verticalAlignment = GridData.FILL;
    composite7 = new Composite(_sf, SWT.NONE);
    composite7.setLayout(gridLayout2);
    composite7.setLayoutData(gridData7);

    createTitelGroup();

    createDatenBereichGroup();
    createSteuerungsBereichGroup();

    erzeugeEditor();
  }

  /**
   * This method initializes group4
   * 
   */
  private void createTitelGroup()
  {
    GridLayout gridLayout4 = new GridLayout();
    gridLayout4.marginWidth = 5;
    GridData gridData8 = new GridData();
    gridData8.horizontalAlignment = GridData.FILL;
    gridData8.grabExcessVerticalSpace = false;
    gridData8.grabExcessHorizontalSpace = true;
    gridData8.heightHint = 28;
    gridData8.verticalAlignment = GridData.CENTER;
    group4 = new Group(composite7, SWT.NONE);
    group4.setText("Titel");
    group4.setLayoutData(gridData8);
    group4.setLayout(gridLayout4);
    label1 = new Label(group4, SWT.NONE);
    label1.setText("Teilkonfigurationsassistent ' " + _secondViewId + " '");
  }

  /**
   * This method initializes group6
   * 
   */
  private void createDatenBereichGroup()
  {
    GridData gridData16 = new GridData();
    gridData16.horizontalAlignment = GridData.FILL;
    gridData16.grabExcessHorizontalSpace = true;
    gridData16.grabExcessVerticalSpace = true;
    gridData16.verticalAlignment = GridData.FILL;
    group6 = new Group(composite7, SWT.NONE);
    group6.setLayout(new FillLayout());
    group6.setLayoutData(gridData16);
    group6.setText("Datenbereich");

  }

  private void erzeugeEditor()
  {
    FillLayout fillLayout = new FillLayout();
    fillLayout.type = org.eclipse.swt.SWT.HORIZONTAL;
    composite1 = new Composite(group6, SWT.None);
    composite1.setLayout(fillLayout);

    _editor = new TabbedDatenBereichEditorComposite(composite1, SWT.NONE, _dataModel, _secondViewId, true,
        buttonUebernehmen);

  }

  /**
   * This method initializes group5
   * 
   */
  private void createSteuerungsBereichGroup()
  {
    GridData gridData5 = new GridData();
    gridData5.horizontalAlignment = GridData.FILL;
    gridData5.grabExcessHorizontalSpace = true;
    gridData5.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout41 = new GridLayout();
    gridLayout41.marginWidth = 5;
    gridLayout41.numColumns = 1;
    group5 = new Group(composite7, SWT.NONE);
    group5.setText("Operationen");
    group5.setLayoutData(gridData5);
    group5.setLayout(gridLayout41);
    createComposite2();
  }

  /**
   * This method initializes composite2
   * 
   */
  private void createComposite2()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.fill = true;
    rowLayout.pack = false;
    rowLayout.wrap = false;
    rowLayout.justify = false;
    GridData gridData4 = new GridData();
    gridData4.horizontalAlignment = GridData.FILL;
    gridData4.grabExcessHorizontalSpace = true;
    gridData4.verticalAlignment = GridData.CENTER;

    GridData gridData22 = new GridData();
    gridData22.horizontalAlignment = GridData.FILL;
    gridData22.grabExcessHorizontalSpace = true;
    gridData22.grabExcessVerticalSpace = false;
    gridData22.verticalAlignment = GridData.BEGINNING;

    _sc = new ScrolledComposite(group5, SWT.H_SCROLL);
    _sc.setLayout(new GridLayout(1, false));
    _sc.setLayoutData(gridData22);
    composite2 = new Composite(_sc, SWT.NONE);
    composite2.setLayoutData(gridData4);
    composite2.setLayout(rowLayout);
    buttonNeu = new Button(composite2, SWT.NONE);
    buttonNeu.setText("Neu");
    buttonNeu.setToolTipText("Anlegen eines neuen Hierarchieobjektes vom gleichen Typ.");
    buttonNeu.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionNeu();
      }
    });
    buttonNeuUO = new Button(composite2, SWT.NONE);
    buttonNeuUO.setText("Neu (u.O.)");
    buttonNeuUO.setToolTipText("Anlegen eines neuen untergeordneten Hierarchieobjektes.");
    buttonNeuUO.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionNeuUO();
      }
    });
    buttonKopieren = new Button(composite2, SWT.NONE);
    buttonKopieren.setText("Kopieren mit u. O.");
    buttonKopieren.setToolTipText("Kopieren des Hierarchieobjektes mit allen untergeordneten Objekten.");
    buttonKopieren.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionKopieren();

      }
    });
    buttonDefault = new Button(composite2, SWT.NONE);
    buttonDefault.setText("Default Parameter");
    buttonDefault.setToolTipText("Default Parameter des Hierarchieobjektes setzen.");
    buttonDefault.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionDefaultParameter();

      }
    });
    buttonPruefen = new Button(composite2, SWT.NONE);
    buttonPruefen.setText("Prfen");
    buttonPruefen.setToolTipText("Prfen des Hierarchieobjektes.");
    buttonPruefen.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionPruefenJob();

      }
    });
    buttonUebernehmen = new Button(composite2, SWT.NONE);
    buttonUebernehmen.setText("bernehmen");
    buttonUebernehmen.setToolTipText("bernehmen des Hierarchieobjektes.");
    buttonUebernehmen.setEnabled(false);
    buttonUebernehmen.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionUebernehmenJob();
      }
    });
    buttonLoeschen = new Button(composite2, SWT.NONE);
    buttonLoeschen.setText("Lschen");
    buttonLoeschen.setToolTipText("Lschen des Hierarchieobjektes.");
    buttonLoeschen.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionLoeschenJob();
      }
    });

    _sc.setContent(composite2);
    composite2.layout();
    _sc.setExpandHorizontal(true);
    _sc.setExpandVertical(true);
    _sc.setMinSize(composite2.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    composite2.layout();

  }

  /**
   * Erzeugt die Aktionen zum Verwalten der Hierarchieobjekte
   */
  private void createActions()
  {
    _actionRegistry = new ActionRegistry();

    Action actionNeu = new Action() {
      public void run()
      {
        aktionNeu();
      }
    };
    actionNeu.setId(TKABasisAktion.NEU.getAktion().getAktionsID());
    actionNeu.setText("Neu");
    actionNeu.setToolTipText("Anlegen eines neuen Hierarchieobjektes vom gleichen Typ.");
    // actionNeu.setImageDescriptor(Activator.getImageDescriptor("icons/actions/add_16x16.png"));
    _actionRegistry.registerAction(actionNeu);

    Action actionNeuUO = new Action() {
      public void run()
      {
        aktionNeuUO();
      }
    };
    actionNeuUO.setId(TKABasisAktion.NEUUO.getAktion().getAktionsID());
    actionNeuUO.setText("Neu (untergeordnetes Objekt)");
    actionNeuUO.setToolTipText("Anlegen eines neuen untergeordneten Hierarchieobjektes.");
    _actionRegistry.registerAction(actionNeuUO);

    Action actionKopieren = new Action() {
      public void run()
      {
        aktionKopieren();
      }
    };
    actionKopieren.setId(TKABasisAktion.KOPIEREN.getAktion().getAktionsID());
    actionKopieren.setText("Kopieren");
    actionKopieren.setToolTipText("Kopieren des Hierarchieobjektes mit allen untergeordneten Objekten.");
    _actionRegistry.registerAction(actionKopieren);

    Action actionKopierenHO = new Action() {
      public void run()
      {
        aktionKopierenHO();
      }
    };
    actionKopierenHO.setId(TKABasisAktion.KOPIERENHO.getAktion().getAktionsID());
    actionKopierenHO.setText("Kopieren (nur Objekt)");
    actionKopierenHO.setToolTipText("Kopieren des Hierarchieobjektes ohne weitere Objekte.");
    _actionRegistry.registerAction(actionKopierenHO);

    Action actionKopierenHOmuO = new Action() {
      public void run()
      {
        aktionKopierenHOmuO();
      }
    };
    actionKopierenHOmuO.setId(TKABasisAktion.KOPIERENHOMUO.getAktion().getAktionsID());
    actionKopierenHOmuO.setText("Kopieren (mit untergeordneten Objekten)");
    actionKopierenHOmuO.setToolTipText("Kopieren des Hierarchieobjektes mit allen untergeordneten Objekten.");
    _actionRegistry.registerAction(actionKopierenHOmuO);

    Action actionVerschieben = new Action() {
      public void run()
      {
        aktionVerschieben();
      }
    };
    actionVerschieben.setId(TKABasisAktion.VERSCHIEBEN.getAktion().getAktionsID());
    actionVerschieben.setText("Verschieben");
    actionVerschieben.setToolTipText("Verschieben des Hierarchieobjektes.");
    _actionRegistry.registerAction(actionVerschieben);

    Action actionEinfuegen = new MyAction() {
      public void run()
      {
        aktionEinfuegen(this.getData());
        this.setData(null);
      }
    };
    actionEinfuegen.setId(TKABasisAktion.EINFUEGEN.getAktion().getAktionsID());
    actionEinfuegen.setText("Einfuegen");
    actionEinfuegen.setToolTipText("Einfuegen des kopierten oder verschobenen Hierarchieobjektes.");
    _actionRegistry.registerAction(actionEinfuegen);

    Action actionKonfigAenderungInfo = new Action() {
      public void run()
      {
        aktionKonfigAenderungInfo();
      }
    };
    actionKonfigAenderungInfo.setId(TKABasisAktion.KONFIGAENDERUNGINFO.getAktion().getAktionsID());
    actionKonfigAenderungInfo.setText("Info Konfigurationsnderung");
    actionKonfigAenderungInfo
        .setToolTipText("Eingabe Informationen zur nderung des Konfigurationsbereiches.");
    _actionRegistry.registerAction(actionKonfigAenderungInfo);

    Action actionSOInfo = new Action() {
      public void run()
      {
        aktionSystemObjektInfo();
      }
    };
    actionSOInfo.setId(TKABasisAktion.SOINFO.getAktion().getAktionsID());
    actionSOInfo.setText("Info SystemObjekt");
    actionSOInfo
        .setToolTipText("ndern der Informationen zum Systemobjekt.");
    _actionRegistry.registerAction(actionSOInfo);
    
    Action actionLoeschen = new Action() {
      public void run()
      {
        aktionLoeschenJob();
      }
    };
    actionLoeschen.setId(TKABasisAktion.LOESCHEN.getAktion().getAktionsID());
    actionLoeschen.setText("Lschen");
    actionLoeschen.setToolTipText("Lschen des Hierarchieobjektes.");
    _actionRegistry.registerAction(actionLoeschen);

    Action actionSelektieren = new Action() {
      public void run()
      {
        aktionSelektieren();
      }
    };
    actionSelektieren.setId(TKABasisAktion.SELEKTIEREN.getAktion().getAktionsID());
    actionSelektieren.setText("Selektieren");
    actionSelektieren.setToolTipText("Selektieren des Hierarchieobjektes.");
    _actionRegistry.registerAction(actionSelektieren);

    Action actionDarstellen = new Action() {
      public void run()
      {
        aktionDarstellen();
      }
    };
    actionDarstellen.setId(TKABasisAktion.DARSTELLEN.getAktion().getAktionsID());
    actionDarstellen.setText("Darstellen");
    actionDarstellen.setToolTipText("Darstellen des Hierarchieobjektes.");
    _actionRegistry.registerAction(actionDarstellen);

    Action actionBearbeiten = new Action() {
      public void run()
      {
        aktionBearbeiten();
      }
    };
    actionBearbeiten.setId(TKABasisAktion.BEARBEITEN.getAktion().getAktionsID());
    actionBearbeiten.setText("Bearbeiten");
    actionBearbeiten.setToolTipText("Bearbeiten des Hierarchieobjektes.");
    _actionRegistry.registerAction(actionBearbeiten);

    Action actionVorgaenger = new Action() {
      public void run()
      {
        aktionVorgaenger();
      }
    };
    actionVorgaenger.setId(TKABasisAktion.VORGAENGER.getAktion().getAktionsID());
    actionVorgaenger.setText("Vorgnger");
    actionVorgaenger.setToolTipText("Vorgnger des Hierarchieobjektes.");
    actionVorgaenger.setImageDescriptor(Activator.getImageDescriptor("icons/vorgaenger.png"));
    _actionRegistry.registerAction(actionVorgaenger);

    Action actionNachfolger = new Action() {
      public void run()
      {
        aktionNachfolger();
      }
    };
    actionNachfolger.setId(TKABasisAktion.NACHFOLGER.getAktion().getAktionsID());
    actionNachfolger.setText("Nachfolger");
    actionNachfolger.setToolTipText("Nachfolger des Hierarchieobjektes.");
    actionNachfolger.setImageDescriptor(Activator.getImageDescriptor("icons/nachfolger.png"));
    _actionRegistry.registerAction(actionNachfolger);

    Action actionVorgaengerEbene = new Action() {
      public void run()
      {
        aktionVorgaengerEbene();
      }
    };
    actionVorgaengerEbene.setId(TKABasisAktion.VORGAENGEREBENE.getAktion().getAktionsID());
    actionVorgaengerEbene.setText("Vorgnger Ebene");
    actionVorgaengerEbene.setToolTipText("Vorgnger Ebene des Hierarchieobjektes.");
    actionVorgaengerEbene.setImageDescriptor(Activator.getImageDescriptor("icons/vorgaenger_ebene.png"));
    _actionRegistry.registerAction(actionVorgaengerEbene);

    Action actionNachfolgerEbene = new Action() {
      public void run()
      {
        aktionNachfolgerEbene();
      }
    };
    actionNachfolgerEbene.setId(TKABasisAktion.NACHFOLGEREBENE.getAktion().getAktionsID());
    actionNachfolgerEbene.setText("Nachfolger Ebene");
    actionNachfolgerEbene.setToolTipText("Nachfolger Ebene des Hierarchieobjektes.");
    actionNachfolgerEbene.setImageDescriptor(Activator.getImageDescriptor("icons/nachfolger_ebene.png"));
    _actionRegistry.registerAction(actionNachfolgerEbene);

    Action actionDefault = new Action() {
      public void run()
      {
        aktionDefaultParameter();
      }
    };
    actionDefault.setId(TKABasisAktion.DEFAULT.getAktion().getAktionsID());
    actionDefault.setText("Default Parameter");
    actionDefault.setToolTipText("Default Parameter des Hierarchieobjektes setzen.");
    _actionRegistry.registerAction(actionDefault);

    Action actionPruefen = new Action() {
      public void run()
      {
        aktionPruefenJob();
      }
    };
    actionPruefen.setId(TKABasisAktion.PRUEFEN.getAktion().getAktionsID());
    actionPruefen.setText("Prfen");
    actionPruefen.setToolTipText("Prfen des Hierarchieobjektes.");
    _actionRegistry.registerAction(actionPruefen);

    Action actionUebernehmen = new Action() {
      public void run()
      {
        aktionUebernehmenJob();
      }
    };
    actionUebernehmen.setId(TKABasisAktion.UEBERNEHMEN.getAktion().getAktionsID());
    actionUebernehmen.setText("bernehmen");
    actionUebernehmen.setToolTipText("bernehmen des Hierarchieobjektes.");
    _actionRegistry.registerAction(actionUebernehmen);

  }

  private void versorgeKontextMenuNavigator()
  {
    _navigatorComposite.addtoKontextMenue(getAction(TKABasisAktion.NEU.getAktion().getAktionsID()));
    _navigatorComposite.addtoKontextMenue(getAction(TKABasisAktion.NEUUO.getAktion().getAktionsID()));
    _navigatorComposite.addtoKontextMenue(getAction(TKABasisAktion.KOPIERENHO.getAktion().getAktionsID()));
    _navigatorComposite.addtoKontextMenue(getAction(TKABasisAktion.KOPIERENHOMUO.getAktion().getAktionsID()));
    _navigatorComposite.addtoKontextMenue(getAction(TKABasisAktion.VERSCHIEBEN.getAktion().getAktionsID()));
    _navigatorComposite.addtoKontextMenue(getAction(TKABasisAktion.EINFUEGEN.getAktion().getAktionsID()));
    _navigatorComposite.addtoKontextMenue(getAction(TKABasisAktion.KONFIGAENDERUNGINFO.getAktion().getAktionsID()));
    _navigatorComposite.addtoKontextMenue(getAction(TKABasisAktion.SOINFO.getAktion().getAktionsID()));
    _navigatorComposite.addtoKontextMenue(getAction(TKABasisAktion.LOESCHEN.getAktion().getAktionsID()));
    _navigatorComposite.addtoKontextMenue(getAction(TKABasisAktion.DARSTELLEN.getAktion().getAktionsID()));
    _navigatorComposite.addtoKontextMenue(getAction(TKABasisAktion.BEARBEITEN.getAktion().getAktionsID()));
  }

  /**
   * Zuordnen der Drag & Drop -Aktionen
   */
  private void versorgeDNDAktionen()
  {
    _navigatorComposite.setDragAction(DNDModus.COPY, getAction(TKABasisAktion.KOPIERENHO.getAktion().getAktionsID()));
    _navigatorComposite.setDragAction(DNDModus.COPY_DEEP, getAction(TKABasisAktion.KOPIERENHOMUO.getAktion()
        .getAktionsID()));
    _navigatorComposite.setDragAction(DNDModus.MOVE, getAction(TKABasisAktion.VERSCHIEBEN.getAktion().getAktionsID()));

    _navigatorComposite.setDropAction((MyAction)getAction(TKABasisAktion.EINFUEGEN.getAktion().getAktionsID()));

  }

  private void aktionNeu()
  {
    _neuesHONavigatorElement = null;
    
    if (ausgabeWarnungBeiAenderung(null) == SWT.CANCEL)
      return;

    // Achtung das Setzen der Buttons und Editable fr den Editor, wird hier fr das selektierte Objekt gemacht,
    // daher muss fr die neue Selektion nach der Neuanlage die momentane Bearbeitungsart im nicht Abbruch-Fall erneut gesetzt werden
    if (!doZustandsUebergang(TKABasisAktion.NEU))
      return;

    boolean abbruch = true;

    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object firstElement = selection.getFirstElement();

    // nur bei HONavigatorElement erlaubt
    if (firstElement instanceof HONavigatorElement)
    {
      HONavigatorElement ne = (HONavigatorElement)firstElement;
      HierarchieObjekt ho = ne.getHierarchieObjekt();

      if (ho != null)
      {
        Object parentObj = ne.getParent();
        IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();

        if (parentObj instanceof HONavigatorElement)
        {
          HONavigatorElement parentNe = (HONavigatorElement)parentObj;
          HierarchieObjekt vater = parentNe.getHierarchieObjekt();

          if (vater != null && anlegenNeuesHO(hot, parentNe, vater))
          {
            abbruch = false;
          }
        }
        else if (parentObj instanceof KBNavigatorElement)
        {
          KBNavigatorElement parentNe = (KBNavigatorElement)parentObj;
          Object kb = parentNe.getKonfigurationsBereich();

          if (kb != null && anlegenNeuesHO(hot, parentNe, kb))
          {
            abbruch = false;
          }
        }
      }
    }

    boolean hoNeAenderbar = true;
    if (_neuesHONavigatorElement != null)
      hoNeAenderbar = _neuesHONavigatorElement.isAenderbar();
    
    if (abbruch || !hoNeAenderbar)
      doZustandsUebergang(TKABasisAktion.ABBRUCH);
    else
    {
      // Bearbeitungszustand inkl. Buttons und Editable muss hier nach dem Neuanlegen und Selektieren des neuen Objektes erneut gesetzt werden
      aktionAendernBearbeitungsmodus(null, _zustandsmaschine.getMomentZustand());

      if (buttonUebernehmen.isEnabled())
        buttonUebernehmen.setFocus();
    }
  }

  private boolean anlegenNeuesHO(IHierarchieObjektTyp hot, INavigatorElement parentNe, Object vater)
  {   
    boolean ret = false;
    TKABasisView view = TKABasisView.getView(_secondViewId);
    IBildungsRegeln bildungsRegeln = view.getModelInfo().getBildungsRegeln();
    
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Erzeuge neues Hierarchieobjekt vom Typ " + hot, null);
    MeldungenSpeicher.getInstanz().setStatus(status);
    BildungsRegelnMitKB modell = new BildungsRegelnMitKB(bildungsRegeln);
    
    if (!hot.isSystemObjektVorhanden())
    {     
      int nret = BildungsRegelnTools.oeffneAssistent(modell, hot, vater);
      
      if (nret == 0)
      {
        String[] ss = bildungsRegeln.getPids(hot, vater instanceof HierarchieObjekt ? (HierarchieObjekt)vater : null);
        
        IHierarchieManager hm = view.getModelInfo().getHierarchieManager();
        
        HierarchieObjekt neu = null;
        HOMitStatus hoMitStatus = null;
        final TKAUpdate tkaUpdate = new TKAUpdate();
        hm.getHierarchie().beobachterAnmelden(tkaUpdate);
        
        try
        {        
          hoMitStatus = hm.erzeugeNeuesHO(modell.getKB(), vater, ss, hot, true);
          status.merge(hoMitStatus.getStatus());        
          neu = hoMitStatus.getHO();
        }
        finally
        {
          hm.getHierarchie().beobachterAbmelden(tkaUpdate);
        }
        
        DatenModellUpdateListener.getInstanz().notifyDatenModellUpdateListener(this,
            tkaUpdate.getListeAktualisiert(), tkaUpdate.getListeGeloescht(), tkaUpdate.getListeNeu());
        
        if (neu != null)
        {
          _neuesHONavigatorElement = sucheSelektiereNeuesHONavigatorElement(parentNe, neu);
                    
          if (_neuesHONavigatorElement != null)
          {
            ret =  true;
          }
        }   
        
        speichernMeldung(status, true);
        ergebnisAusgabe("Neuanlegen", hoMitStatus.getStatus());
      }    

    }
    else
    {    
      ArrayList<SystemObjectType> types = new ArrayList<SystemObjectType>();
      SystemObjectType type = _dataModel.getType(hot.getTypen().get(0));
      types.add(type);
           
      DatenidentifikationGtmOffline dia = new DatenidentifikationGtmOffline(PlatformUI.getWorkbench().getDisplay().getActiveShell(), null, _dataModel);
      LinkedList<SystemObject> sos = dia.oeffnen(types, 1);
      
      //fr alle ausgewhlten SO Hierarchieobjekte anlegen
      for (SystemObject so : sos)
      {
        String[] ss = new String[1];
        ss[0] = so.getPid();

        IHierarchieManager hm = view.getModelInfo().getHierarchieManager();
        
        HOMitStatus hoMitStatus = hm.erzeugeNeuesHO(modell.getKB(), vater, ss, hot, true);
        status.merge(hoMitStatus.getStatus());       
        HierarchieObjekt neu = hoMitStatus.getHO();
        
        if (neu != null)
        {
          _neuesHONavigatorElement = sucheSelektiereNeuesHONavigatorElement(parentNe, neu);       
          
          if (_neuesHONavigatorElement != null)
          {
            ret =  true;
          }
        }   
        
        speichernMeldung(status, true);
        ergebnisAusgabe("Neuanlegen", hoMitStatus.getStatus());
      }
    }    
    
    return ret;
  }

  private HONavigatorElement sucheSelektiereNeuesHONavigatorElement(INavigatorElement parentNe, HierarchieObjekt neu)
  {
    HONavigatorElement result = _navigatorComposite.sucheItem(parentNe, neu, false);
    
    _navigatorComposite.expandNavigator(parentNe);
    _navigatorComposite.refreshNavigator(parentNe);
    _ausgabeWarnungAenderung = false;
    if (result != null)
    {
      _navigatorComposite.selektiereItem(result);
    }
    else
    {
      _navigatorComposite.selektiereItem(parentNe);
    }
    _ausgabeWarnungAenderung = true;
    return result;
  }
  
  private void aktionNeuUO()
  {
    _neuesHONavigatorElement = null;
    
    if (ausgabeWarnungBeiAenderung(null) == SWT.CANCEL)
      return;

    // Achtung das Setzen der Buttons und Editable fr den Editor, wird hier fr das selektierte Objekt gemacht,
    // daher muss fr die neue Selektion nach der Neuanlage die momentane Bearbeitungsart im nicht Abbruch-Fall erneut gesetzt werden
    if (!doZustandsUebergang(TKABasisAktion.NEUUO))
      return;

    boolean abbruch = true;

    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object firstElement = selection.getFirstElement();

    if (firstElement instanceof HONavigatorElement)
    {
      HONavigatorElement parentNe = (HONavigatorElement)firstElement;
      HierarchieObjekt ho = parentNe.getHierarchieObjekt();

      if (ho != null && anlegenNeuesHOzumVater(parentNe, ho))
      {
        abbruch = false;
      }
    }
    else if (firstElement instanceof KBNavigatorElement)
    {
      KBNavigatorElement parentNe = (KBNavigatorElement)firstElement;
      ConfigurationArea kb = parentNe.getKonfigurationsBereich();

      if (kb != null && anlegenNeuesHOzumVater(parentNe, kb))
      {
        abbruch = false;
      }
    }
    
    boolean hoNeAenderbar = true;
    if (_neuesHONavigatorElement != null)
      hoNeAenderbar = _neuesHONavigatorElement.isAenderbar();
    
    if (abbruch || !hoNeAenderbar)
      doZustandsUebergang(TKABasisAktion.ABBRUCH);
    else
    {
      aktionAendernBearbeitungsmodus(null, _zustandsmaschine.getMomentZustand());

      if (buttonUebernehmen.isEnabled())
        buttonUebernehmen.setFocus();
    }

  }

  private boolean anlegenNeuesHOzumVater(INavigatorElement parentNe, Object vater)
  {
    IHierarchieObjektTyp hot = auswahlHOT(vater);

    if (hot != null)
      return anlegenNeuesHO(hot, parentNe, vater);

    return false;
  }

  private void aktionKopieren()
  {
    _neuesHONavigatorElement = null;
    
    if (ausgabeWarnungBeiAenderung("Kopieren des Hierarchieobjekts") == SWT.CANCEL)
      return;

//    // Achtung das Setzen der Buttons und Editable fr den Editor, wird hier fr das selektierte Objekt gemacht,
//    // daher muss fr die neue Selektion nach der Kopie die momentane Bearbeitungsart im nicht Abbruch-Fall erneut gesetzt werden
//    if (!doZustandsUebergang(TKABasisAktion.KOPIEREN))
//      return;

    boolean abbruch = true;

    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object firstElement = selection.getFirstElement();

    // nur bei HONavigatorElement erlaubt
    if (firstElement instanceof HONavigatorElement)
    {
      HONavigatorElement ne = (HONavigatorElement)firstElement;
      HierarchieObjekt ho = ne.getHierarchieObjekt();

      if (ho != null)
      {
        Object parentObj = ne.getParent();

        if (parentObj instanceof HONavigatorElement)
        {
          HONavigatorElement parentNe = (HONavigatorElement)parentObj;
          HierarchieObjekt vater = parentNe.getHierarchieObjekt();
          if (vater != null && kopierenHO(ho, parentNe, vater, true))
          {
            abbruch = false;
          }
        }
        else if (parentObj instanceof KBNavigatorElement)
        {
          KBNavigatorElement parentNe = (KBNavigatorElement)parentObj;
          Object vater = parentNe.getKonfigurationsBereich();
          if (vater != null && kopierenHO(ho, parentNe, vater, true))
          {
            abbruch = false;
          }
        }
      }
    }

    boolean hoNeAenderbar = true;
    if (_neuesHONavigatorElement != null)
      hoNeAenderbar = _neuesHONavigatorElement.isAenderbar();
    
    if (abbruch || !hoNeAenderbar)
      doZustandsUebergang(TKABasisAktion.ABBRUCH);
    else
    {
      // Bearbeitungszustand inkl. Buttons und Editable muss hier nach dem Kopieren und Selektieren des neuen Objektes erneut gesetzt werden
//      aktionAendernBearbeitungsmodus(null, _zustandsmaschine.getMomentZustand());
      doZustandsUebergang(TKABasisAktion.KOPIEREN);
      if (buttonUebernehmen.isEnabled())
        buttonUebernehmen.setFocus();
    }

  }

  /**
   * HierarchieObjekt kopieren
   * 
   * @param src       zu kopierendes HO
   * @param parentNe  neues Elternteil im Navigator
   * @param vater     neues Vater-Objekt in der Hierarchie
   * @param tiefeKopie tiefe (rekursive) Kopie mit allen Kindern
   */
  private boolean kopierenHO(HierarchieObjekt src, INavigatorElement parentNe, Object vater, boolean tiefeKopie)
  {
    if(SYS_OUT)
      System.out.println("kopierenHO von HO " + src.getSystemObjekt(0).getPid() + " : " + tiefeKopie);
    
    boolean ret = false;
    TKABasisView view = TKABasisView.getView(_secondViewId);
    IBildungsRegeln bildungsRegeln = view.getModelInfo().getBildungsRegeln();
    IHierarchieObjektTyp hot = src.getHierarchieObjektTyp();
    IHierarchieObjektTyp neuesHot = hot;
    
    BildungsRegelnMitKB modell = new BildungsRegelnMitKB(bildungsRegeln);
    String[] ss = null;
    
    if (!hot.isSystemObjektVorhanden())
    {
      //berprfung, ob eine Neuanlage zum HOT erlaubt ist, ansonsten Alternativen anbieten
      IHierarchieManager hierarchieManager = view.getModelInfo().getHierarchieManager();
      
      if (hierarchieManager.pruefeNeuanlageVerboten(vater, hot))
      {
        //Neuanlage verboten, daher Anzeige aller mglichen HOT
        neuesHot = auswahlHOT(vater);
      }
      if ( neuesHot != null)
      {
        int nret = BildungsRegelnTools.oeffneAssistent(modell, neuesHot, vater, src);

        if (nret == 0)
        {
          ss = bildungsRegeln.getPids(neuesHot, vater instanceof HierarchieObjekt ? (HierarchieObjekt)vater : null);
          ret = true;
        }
      }
    }
    else
    {
      ss = new String[1];
      ss[0] = src.getSystemObjekt(0).getPid();
      ret = true;
    } 

    if (ret)
    {
      boolean doppeltePid;
      IStatus result;
      MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Kopiere Hierarchieobjekt " + src, null);
      MeldungenSpeicher.getInstanz().setStatus(status);
      
      do
      {
        doppeltePid = false;
        HOMitStatus hoMitStatus = kopiereHO(src, parentNe, modell.getKB(), vater, ss, tiefeKopie, true);
        HierarchieObjekt neu = hoMitStatus.getHO();
        result = hoMitStatus.getStatus();
        
        if (neu != null)
        {
          //eventuell neues HOT setzen
          if (neuesHot != null && neuesHot != hot)
            neu.setHierarchieObjektTyp(neuesHot);
          
          _navigatorComposite.refreshNavigator(parentNe);
          if (_neuesHONavigatorElement != null)
          {
            _ausgabeWarnungAenderung = false;
            _navigatorComposite.selektiereItem(_neuesHONavigatorElement);
            _ausgabeWarnungAenderung = true;
          }
        }
        
        if (!hot.isSystemObjektVorhanden())
        {
          List<String> pidMeldungen = getMeldungenDoppeltePid(result);
          doppeltePid = !pidMeldungen.isEmpty();
          
          if (doppeltePid)
          {
            zeigeMehrzeiligeMeldung("Generierte Pid bereits verwendet", pidMeldungen, result.getSeverity());
            
            if (BildungsRegelnTools.oeffneAssistent(modell, neuesHot, vater, src, true) == 0)
            {
              ss = bildungsRegeln.getPids(neuesHot, vater instanceof HierarchieObjekt ? (HierarchieObjekt)vater : null);
            } 
            else
            {
              doppeltePid = false;
              result = new Status(Status.CANCEL, Activator.PLUGIN_ID, 
                  "Kopiervorgang vom Benutzer abgebrochen");
            }
          }
        }
      }
      while (doppeltePid);
      
      status.merge(result);
      speichernMeldung(status, true);
      
      ergebnisAusgabe("Kopieren", result);
    }
    return ret;
  }

  private void zeigeMehrzeiligeMeldung(String titel, List<String> liste, int severity)
  {
    int style = SWT.OK;
    
    if ((severity & IStatus.ERROR) != 0)
    {
      style |= SWT.ICON_ERROR;
    }
    else if ((severity & IStatus.WARNING) != 0)
    {
      style |= SWT.ICON_WARNING;
    }
    else
    {
      style |= SWT.ICON_INFORMATION;
    }
    
    String meldung = "";
    
    for (String text: liste)
    {
      if (!meldung.isEmpty())
      {
        meldung += "\n";
      }
      meldung += text;
    }
    
    MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), style);
    msg.setText(titel);
    msg.setMessage(meldung);
    msg.open();
  }
  
  /**
   * Liefert die Liste von Textmeldungen aus einer Instanz von Typ IStatus, 
   * die sich auf eine mehrfach verwendete Pid beziehen
   * 
   * @param status die IStatus-Instanz, die durchsucht werden soll
   * 
   * @return Liste entsprechender Textmeldung, wenn keine entsprechenden Meldungen in der Statusmeldung
   * enthalten sind, wird die leere Liste geliefert.
   */
  private List<String> getMeldungenDoppeltePid(IStatus status)
  {
    List<String> result = new Vector<String>();
    
    if ((status.getCode() & PruefungenTools.CODE_DOPPELTE_PID) != 0)
    {
      result.add(status.getMessage());
    }
    
    for (IStatus unterStatus: status.getChildren())
    {
      result.addAll(getMeldungenDoppeltePid(unterStatus));
    }
    
    return result;
  }
  
  /**
   * Auswahl-Dialog der gltigen HOTs zum HierarchieObjekt anzeigen, wenn dies mehr als eines ist
   * und ausgewhlten HOT zurckliefern.
   * 
   * @param vater Vater-Objektder Hierarchie, zu dem ein HOT ausgewhlt werden soll 
   * @return  ausgewhlter HOT
   */
  private IHierarchieObjektTyp auswahlHOT(Object vater)
  {
    IHierarchieObjektTyp hot = null;

    TKABasisView view = TKABasisView.getView(_secondViewId);
    IHierarchieManager hierarchieManager = view.getModelInfo().getHierarchieManager();
    IHierarchieObjektTyp[] temp = hierarchieManager.getUntergeordneteHOTs(vater);
    List<IHierarchieObjektTyp> list = new ArrayList<IHierarchieObjektTyp>();
    for (IHierarchieObjektTyp uhot : temp)
    {
      if (!hierarchieManager.pruefeNeuanlageVerboten(vater, uhot))
        list.add(uhot);
    }

    IHierarchieObjektTyp[] uos = list.toArray(new IHierarchieObjektTyp[list.size()]);

    if (uos != null && uos.length > 0)
    {
      if (uos.length > 1)
      {
        final KonfigAssListDialog listDialog = new KonfigAssListDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
        listDialog.setTitle("Auswahl untergeordnete Hierarchie");
        listDialog.setMessage("Es muss ein  HOT ausgewhlt werden");
        listDialog.setContentProvider(new ArrayContentProvider());
        listDialog.setLabelProvider(new LabelProvider());
        listDialog.setInput(uos);
        listDialog.setHelpContext(Activator.PLUGIN_ID + ".ho_neu");

        if (listDialog.open() == Dialog.OK)
        {
          Object[] result = listDialog.getResult();
          if (result.length > 0)
          {
            hot = (IHierarchieObjektTyp)result[0];
          }
        }
      }
      else
      {
        hot = uos[0];
      }
    }

    return hot;
  }
  
  private HOMitStatus kopiereHO(HierarchieObjekt src, INavigatorElement parentNe, ConfigurationArea kb,
      Object vater, String[] pid, boolean tiefeKopie, boolean isWurzel)
  {
    
    MultiStatus meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, 
        "Kopiere untergeordnetes Hierarchieobjekt " + src + " zu " + vater, null);
    
    TKABasisView view = TKABasisView.getView(_secondViewId);
    IHierarchieManager hm = view.getModelInfo().getHierarchieManager();
    
    HierarchieObjekt neu = null;
    HOMitStatus hoMitStatus = null;
    
    final TKAUpdate tkaUpdate = new TKAUpdate();
    hm.getHierarchie().beobachterAnmelden(tkaUpdate);
    
    try
    {
      hoMitStatus = hm.kopiereHO(src, kb, vater, pid, false, isWurzel);
      neu = hoMitStatus.getHO();
      meldung.merge(hoMitStatus.getStatus());
    }
    finally
    {
      hm.getHierarchie().beobachterAbmelden(tkaUpdate);
    }
    
    DatenModellUpdateListener.getInstanz().notifyDatenModellUpdateListener(this,
        tkaUpdate.getListeAktualisiert(), tkaUpdate.getListeGeloescht(), tkaUpdate.getListeNeu());
    
    if (neu != null)
    {
      HONavigatorElement neuEl;

      neuEl = _navigatorComposite.sucheItem(parentNe, neu, false);
      _navigatorComposite.expandNavigator(parentNe);

      if (isWurzel)
      {
        _neuesHONavigatorElement = neuEl;
      }
      if (tiefeKopie && !meldung.matches(IStatus.ERROR | IStatus.CANCEL) && neuEl != null)
      {
        IBildungsRegeln bildungsRegeln = view.getModelInfo().getBildungsRegeln();
        IHierarchie hierarchie = hm.getHierarchie();
                
        List<HierarchieObjekt> liste = new Vector<HierarchieObjekt>(hierarchie.getUntergeordneteObjekte(src));
        for (HierarchieObjekt kind: liste)
        {
          if (hm.kopierenNichtErlaubt(kind))
          {
            continue;
          }
          
          int ret = 0;
          String[] ss = null;
          IHierarchieObjektTyp hot = kind.getHierarchieObjektTyp();
          BildungsRegelnMitKB modell = new BildungsRegelnMitKB(bildungsRegeln);
          if (!hot.isSystemObjektVorhanden())
          {
            ret = BildungsRegelnTools.oeffneAssistent(modell, hot, neu, kind);
            if (ret == 0)
            {  
              ss = bildungsRegeln.getPids(hot, neu);
            }  
          }
          else
          {
            ss = new String[1];
            ss[0] = kind.getSystemObjekt(0).getPid();
          }   
          
          if (ret == 0) // ohne Dialog oder Dialog nicht abgebrochen oder !hot.isSystemObjektVorhanden()
          {
            boolean doppeltePid;
            IStatus result;
            
            do
            {
              doppeltePid = false;
              hoMitStatus = kopiereHO(kind, neuEl, modell.getKB(), neu, ss, tiefeKopie, false);
              result = hoMitStatus.getStatus();
              
              if (!hot.isSystemObjektVorhanden())
              {
                List<String> pidMeldungen = getMeldungenDoppeltePid(result);
                doppeltePid = !pidMeldungen.isEmpty();
                
                if (doppeltePid)
                {
                  zeigeMehrzeiligeMeldung("Generierte Pid bereits verwendet", pidMeldungen, result.getSeverity());
                  
                  if (BildungsRegelnTools.oeffneAssistent(modell, hot, neu, kind) == 0)
                  {
                    ss = bildungsRegeln.getPids(hot, neu);
                  } 
                  else
                  {
                    doppeltePid = false;
                    result = new Status(Status.CANCEL, Activator.PLUGIN_ID, 
                        "Kopiervorgang vom Benutzer abgebrochen");
                  }
                }
              }
            }
            while (doppeltePid);
              
            meldung.add(result);
          }
          else
          {
            meldung.add(new Status(Status.CANCEL, Activator.PLUGIN_ID, 
                "Kopiervorgang vom Benutzer abgebrochen"));
          }
          
          if (meldung.matches(IStatus.ERROR | IStatus.CANCEL))
          {
            break;
          }
        }
      }
    }
    
    return new HOMitStatus(neu, meldung);
  }
  
  private void aktionKopierenHO()
  {
    if (ausgabeWarnungBeiAenderung("Kopieren(n.O.) des Hierarchieobjekts") == SWT.CANCEL)
      return;

    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object[] selArray = selection.toArray();

    _kopierteHoNes.clear();
    for (int i = 0; i < selArray.length; i++)
    {
      // nur bei HONavigatorElement erlaubt
      if (selArray[i] instanceof HONavigatorElement)
      {
        _kopierteHoNes.add((HONavigatorElement)selArray[i]);
      }
    }

    if (_kopierteHoNes.size() > 0)
    {
      _kopierModus = KopierModus.NORMAL;
      // doZustandsUebergang(TKABasisAktion.KOPIERENHO);
    }

  }

  private void aktionKopierenHOmuO()
  {
    if (ausgabeWarnungBeiAenderung("Kopieren(m.u.O.) des Hierarchieobjekts") == SWT.CANCEL)
      return;

    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object[] selArray = selection.toArray();

    _kopierteHoNes.clear();
    for (int i = 0; i < selArray.length; i++)
    {
      // nur bei HONavigatorElement erlaubt
      if (selArray[i] instanceof HONavigatorElement)
      {
        _kopierteHoNes.add((HONavigatorElement)selArray[i]);
      }
    }

    if (_kopierteHoNes.size() > 0)
    {
      _kopierModus = KopierModus.TIEF;
      // doZustandsUebergang(TKABasisAktion.KOPIERENHOMUO);
    }

  }

  private void aktionVerschieben()
  {
    // System.out.println("Verschieben durchfhren");

    if (ausgabeWarnungBeiAenderung("Verschieben des Hierarchieobjekts") == SWT.CANCEL)
      return;

    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object[] selArray = selection.toArray();

    _kopierteHoNes.clear();
    for (int i = 0; i < selArray.length; i++)
    {
      // nur bei HONavigatorElement erlaubt
      if (selArray[i] instanceof HONavigatorElement)
      {
        _kopierteHoNes.add((HONavigatorElement)selArray[i]);
      }
    }

    if (_kopierteHoNes.size() > 0)
    {
      _kopierModus = KopierModus.VERSCHIEBEN;
      // doZustandsUebergang(TKABasisAktion.VERSCHIEBEN);
    }

  }

  private void aktionEinfuegen(Object selElement)
  {
    if (SYS_OUT)
      System.out.println("Einfgen durchfhren");
    
    _neuesHONavigatorElement = null;

    AbstractNavigatorElement parentNe = null;
    if (selElement == null)
    {
      IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
      parentNe = (AbstractNavigatorElement)selection.getFirstElement();
    }
    else
    {

      if (selElement instanceof AbstractNavigatorElement)
      {
        parentNe = (AbstractNavigatorElement)selElement;

        // erlaubte HOTs zum Objekt ermitteln
        TKABasisView view = TKABasisView.getView(_secondViewId);
        IHierarchieManager hierarchieManager = view.getModelInfo().getHierarchieManager();

        if (!pruefeEinfuegen(parentNe, hierarchieManager))
        {
          if (SYS_OUT)
          {
            System.out
                .println("Einfgen nicht erlaubt. Eventuell KB beim Kopieren nicht gleich, KB beim Verschieben gleich oder nichtaktive Objekte beim Verschieben in anderen KB.");
            System.out.println("KopierModus = " + _kopierModus + " , Anzahl kopierteHoNes = " + _kopierteHoNes.size()
                + " Einfgen in " + parentNe.getName());
          }
          return;
        }
      }
      else
        return;
    }

    // Warnung ausgeben, falls Button bernehmen aktiv
    if (ausgabeWarnungBeiAenderung("Einfgen des Hierarchieobjekts") == SWT.CANCEL)
      return;

    //Kopieren oder Verschieben durchfhren
    if (_kopierteHoNes.size() > 0 && !_kopierModus.equals(KopierModus.KEIN))
    {
      boolean abbruch = true;

      Object vater = null;
      if (parentNe instanceof HONavigatorElement)
      {
        vater = ((HONavigatorElement)parentNe).getHierarchieObjekt();
      }
      else if (parentNe instanceof KBNavigatorElement)
      {
        vater = ((KBNavigatorElement)parentNe).getKonfigurationsBereich();
      }

      for (int i = 0; i < _kopierteHoNes.size(); i++)
      {
        HONavigatorElement kopiertesHoNe = _kopierteHoNes.get(i);
        HierarchieObjekt ho = kopiertesHoNe.getHierarchieObjekt();

        if (_kopierModus.equals(KopierModus.NORMAL) || _kopierModus.equals(KopierModus.TIEF))
        {
          if (vater != null && kopierenHO(ho, parentNe, vater, _kopierModus.equals(KopierModus.TIEF)))
          {
            abbruch = false;
//            doZustandsUebergang(TKABasisAktion.EINFUEGEN);
          }
        }
        else if (_kopierModus.equals(KopierModus.VERSCHIEBEN))
        {
          if (vater != null && verschiebenHO(ho, kopiertesHoNe, parentNe, vater, true))
          {
            abbruch = false;
//            doZustandsUebergang(TKABasisAktion.EINFUEGENVER);
          }
        }
      }
      
      boolean hoNeAenderbar = true;
      if (_neuesHONavigatorElement != null)
        hoNeAenderbar = _neuesHONavigatorElement.isAenderbar();
      
      if (abbruch || !hoNeAenderbar)
      {
        doZustandsUebergang(TKABasisAktion.ABBRUCH);
        if (!hoNeAenderbar)
        {
          _kopierteHoNes.clear();
          _kopierModus = KopierModus.KEIN;          
        }
      }
      else
      {
        if (_kopierModus.equals(KopierModus.NORMAL) || _kopierModus.equals(KopierModus.TIEF))
        {
          doZustandsUebergang(TKABasisAktion.EINFUEGEN);
        }
        else if (_kopierModus.equals(KopierModus.VERSCHIEBEN))
        {
          doZustandsUebergang(TKABasisAktion.EINFUEGENVER);
        }
        
        _kopierteHoNes.clear();
        _kopierModus = KopierModus.KEIN;

        if (buttonUebernehmen.isEnabled())
          buttonUebernehmen.setFocus();
      }

    }

  }

  private boolean verschiebenHO(HierarchieObjekt ho, INavigatorElement neSrc, INavigatorElement neDest,
      Object neuerVater, boolean isWurzel)
  {
    boolean ret = false;
    TKABasisView view = TKABasisView.getView(_secondViewId);
    IHierarchieManager hm = view.getModelInfo().getHierarchieManager();
    IHierarchie hierarchie = hm.getHierarchie();
    
    ConfigurationArea srcKb = hierarchie.getKonfigurationsBereich(ho); 
    ConfigurationArea destKb = hierarchie.getKonfigurationsBereich(neuerVater);
    
    if (!ho.getHierarchieObjektTyp().isSystemObjektVorhanden() && srcKb != destKb)
    {
      MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.OK | SWT.CANCEL
          | SWT.ICON_QUESTION);
      msg.setText("Verschieben in anderen Konfigurationsbereich");
      msg.setMessage("Das Hierarchieobjekt " + ho + " (mit allen ggf. untergeordneten Objekten) wird aus dem Konfigurationsbereich " + 
          srcKb.getPid() + " gelscht und in " + destKb.getPid() + " identisch neu erzeugt.\nSoll der Vorgang fortgesetzt werden?");

      int result = msg.open();
      
      if (result == SWT.CANCEL)
        return ret;
    }
    
    MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Verschiebe Hierarchieobjekt " + ho, null);
    MeldungenSpeicher.getInstanz().setStatus(status);
    
    HierarchieObjekt neu = null;
    HOMitStatus hoMitStatus = null;
    final TKAUpdate tkaUpdate = new TKAUpdate();
    hm.getHierarchie().beobachterAnmelden(tkaUpdate);
    
    try
    {        
      hoMitStatus = hm.verschiebeHO(ho, neuerVater);
      status.merge(hoMitStatus.getStatus());
      neu = hoMitStatus.getHO();
    }
    finally
    {
      hm.getHierarchie().beobachterAbmelden(tkaUpdate);
    }
    
    DatenModellUpdateListener.getInstanz().notifyDatenModellUpdateListener(this,
        tkaUpdate.getListeAktualisiert(), tkaUpdate.getListeGeloescht(), tkaUpdate.getListeNeu());
    
    if (neu != null)
    {
      _neuesHONavigatorElement = sucheSelektiereNeuesHONavigatorElement(neDest, neu);
      
      ret = true;
    }
    
    speichernMeldung(status, true);
    ergebnisAusgabe("Verschieben", hoMitStatus.getStatus());

    return ret;
  }

  private void aktionSelektieren()
  {
    if (_ausgabeWarnungAenderung == true && _aktualisiereEditor == true)
    {
      if (ausgabeWarnungBeiAenderung("Wechsel des Hierarchieobjekts") == SWT.CANCEL)
        return;
    }

    if (_aktualisiereEditor == true)
    {

      erzeugeDatenbereich();

      if (_ausgabeWarnungAenderung == true)
      {
        doZustandsUebergang(TKABasisAktion.SELEKTIEREN);

      }
    }

  }

  /**
   * Datenbereich erzeugen
   * 
   * @return Flag, fr Datenbereich neu erzeugt
   */
  private boolean erzeugeDatenbereich()
  {
    // Datenbereich leeren wird in createTabPage gemacht

    // Datenbereich mit den Daten der selektierten Elemente neu erzeugen
    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object firstElement = selection.getFirstElement();

    if (firstElement instanceof HONavigatorElement)
    {
      HONavigatorElement na = (HONavigatorElement)firstElement;
      HierarchieObjekt ho = na.getHierarchieObjekt();

      if (ho != null)
      {
        IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
        int typIndex = 0;
        for (SystemObject so: ho.getSystemObjekte())
        {         
          DatenBereichSOKapsel kapselSo = new DatenBereichSOKapsel(so, ho);
          
          TypInfo ti = hot.getTypInfo(so.getType().getPid());         

          // Attributgruppen in der Typinformation des HOTs
          String[] atgs = ti.getAtgs().toArray(new String[0]);
          // Mengen in der Typinformation des HOTs
          String[] mengen = ti.getDynMengen().toArray(new String[0]);
          
          String asp = "asp.eigenschaften";
          
          LinkedList<DatenBereichEditorKapsel> kapseln = new LinkedList<DatenBereichEditorKapsel>();

          int atgIndex = 0;
          int mengenIndex = 0;
          
          if (atgs.length > 0)
          {
            for (int i = 0; i < atgs.length; i++)
            {
              String sAtg = atgs[i];
              
              // Prfen, ob diese ATG mit diesem Typ benutzbar ist
              AttributeGroup atg = _dataModel.getAttributeGroup(sAtg); 
              if (atg == null || !so.getType().getAttributeGroups().contains(atg))
              {
                // Es gibt diese Attributgruppe nicht im Datenmodell oder sie ist mit dem Objekttyp 
                // nicht verwendbar
                if (SYS_OUT)
                  System.out.println("Systemobjekt " + so.getPid() + " kann nicht mit ATG " + atg.getPid() +
                  " verwendet werden");
//                _debug.warning("Systemobjekt " + so.getPid() + " kann nicht mit ATG " + atg.getPid() +
//                    " verwendet werden");
                continue;
              }
              
              DatenBereichEditorKapsel kapsel = new DatenBereichEditorKapsel(kapselSo, sAtg, asp);
              kapseln.add(kapsel);
         
              if (atgIndex == 0 && typIndex == 0)
                _editor.createTabPage(kapsel, true, true);
              else if (atgIndex == 0 && typIndex > 0)
                _editor.createTabPage(kapsel, false, true);
              else
                _editor.createTabPage(kapsel, false, false);
              
              atgIndex += 1;
            }            
          }
          else if (mengen.length > 0)
          {
            List<String> mengenListe = new Vector<String>();
            for (ObjectSetUse use:so.getType().getObjectSetUses())
            {
              if (use.getObjectSetType().isMutable())
              {
                mengenListe.add(use.getObjectSetName());
              }
            }
            
            if (SYS_OUT)
              System.out.println("Mengen gefunden: " + mengen.length);
            for (int i = 0; i < mengen.length; i++)
            {
              String sMenge = mengen[i];
              
              // Prfen, ob diese Menge mit diesem Typ als dyn. Menge benutzbar ist
            
              if (!mengenListe.contains(sMenge))
              {
                // Die Menge ist mit dem Objekttyp als dyn. Menge nicht verwendbar
                if (SYS_OUT)
                  System.out.println("Systemobjekt " + so.getPid() + " kann nicht mit Menge " + sMenge +
                  " als dynamischer Menge verwendet werden");
//                _debug.warning("Systemobjekt " + so.getPid() + " kann nicht mit Menge " + sMenge +
//                    " als dynamischer Menge verwendet werden");
                continue;
              }
              
              DatenBereichEditorKapsel kapsel = new DatenBereichEditorKapsel(kapselSo, sMenge, asp, TYP.Menge);
              kapseln.add(kapsel);
       
              if (mengenIndex == 0 && typIndex == 0)
                _editor.createTabPage(kapsel, true, true);
              else if (mengenIndex == 0 && typIndex > 0)
                _editor.createTabPage(kapsel, false, true);
              else
                _editor.createTabPage(kapsel, false, false);
              
              mengenIndex +=1;
            }              
          }
          else
          {
            DatenBereichEditorKapsel kapsel = new DatenBereichEditorKapsel(kapselSo, "", asp);
            kapseln.add(kapsel);
            if (typIndex == 0)
              _editor.createTabPage(kapsel, true, true);
            else
              _editor.createTabPage(kapsel, false, true);
          }
          
          typIndex++;

        }
        _editor.setSelektionTab(0, 0);

        return true;
      }

    }
    else if (firstElement instanceof KBNavigatorElement)
    {
      _editor.loescheDarstellungsBereich();
    }
    return false;
  }

  private void aktionDarstellen()
  {
    if (ausgabeWarnungBeiAenderung(null) == SWT.CANCEL)
    {
      Zustand momentZustand = _zustandsmaschine.getMomentZustand();
      Bearbeitungsmodus momentBM = Bearbeitungsmodus.valueOf(momentZustand.toString());
      if (momentBM == Bearbeitungsmodus.BEARBEITUNG)
        setRadioButton(1);
      return;
    }

    // nderungen zurcknehmen
    if (buttonUebernehmen.isEnabled())
    {
      erzeugeDatenbereich();
    }
    doZustandsUebergang(TKABasisAktion.DARSTELLEN);
  }

  private void aktionBearbeiten()
  {
    doZustandsUebergang(TKABasisAktion.BEARBEITEN);

  }

  private void aktionDefaultParameter()
  {
    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object firstElement = selection.getFirstElement();

    if (firstElement instanceof HONavigatorElement)
    {
      HierarchieObjekt ho = ((HONavigatorElement)firstElement).getHierarchieObjekt();

      if (ho != null)
      {
        DefaultParameter dp = new DefaultParameter();
        Zustand momentZustand = _zustandsmaschine.getMomentZustand();
        Bearbeitungsmodus momentBM = Bearbeitungsmodus.valueOf(momentZustand.toString());
        boolean editable = true;
        if (momentBM == Bearbeitungsmodus.DARSTELLUNG)
          editable = false;

        boolean oeffnenOk = dp.oeffneDialog(ho, _dataModel, _secondViewId, editable);

        if (!oeffnenOk)
        {
          MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.ERROR, "ffne DefaultParameter " + firstElement,
              null);
          MeldungenSpeicher.getInstanz().setStatus(_status);
      
          MultiStatus objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.ERROR,
              "ffnen des Dialoges ist fehlgeschlagen", null);
          status.add(objektStatus);
          speichernMeldung(status, false);
        }
        
      }
    }
  }

  private void aktionPruefenJob()
  {
    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object firstElement = selection.getFirstElement();
    
    HONavigatorElement selHoNe = null;
    if (firstElement instanceof HONavigatorElement)
    {
      selHoNe = (HONavigatorElement)firstElement;
    }
    else
    {
      MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.ERROR, "Prfe Hierarchieobjekt " + firstElement,
          null);
      MeldungenSpeicher.getInstanz().setStatus(_status);
  
      MultiStatus objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.ERROR,
          "Auswahl ist kein Hierarchieobjekt", null);
      status.add(objektStatus);
      speichernMeldung(status, false);
      return;
    }
    
    if (selHoNe != null)
    {

      TKABasisView view = TKABasisView.getView(_secondViewId);
      IHierarchieManager hierarchieManager = view.getModelInfo().getHierarchieManager();
      
      pruefenHOJob(selHoNe, hierarchieManager);

    }
  }

  /**
   * Aufruf des Jobs fr die Prfung eines Hierarchieobjekts und Warten bis dieser
   * Job zu Ende ist oder abgebrochen wird. 
   * 
   * @param selHoNe
   *          selektiertes Hierarchieobjekt-NavigatorElement
   * @param hm
   *          zur View gehrender HierarchieManager
   */ 
  private void pruefenHOJob(HONavigatorElement selHoNe, IHierarchieManager hm)
  {   
    final HierarchieObjekt hierarchieObjekt = selHoNe.getHierarchieObjekt();
    final IHierarchieManager hierarchieManager = hm;

    _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Hierarchieobjekt " + hierarchieObjekt, null);
    MeldungenSpeicher.getInstanz().setStatus(_status);

    final LinkedList<SystemObjektDaten> sodListe = getSystemObjektDaten();
    
    final Job job = new Job("TKABasis: Pruefung HO") {

      protected IStatus run(IProgressMonitor monitor)
      {
        monitor.beginTask("Starte Pruefung...", 5);
        monitor.worked(1);

        try
        {

          if (hierarchieObjekt != null)
          {
            HODaten hod = new HODaten();
            int index = 0;
            for (SystemObjektDaten sod : sodListe)
            {
              //Voraussetzung ist, dass beide Listen die gleiche Reihenfolge haben
              SystemObject so = hierarchieObjekt.getSystemObjekt(index);
              hod.setSystemObjektDaten(so, sod);
              index++;
            }

            IStatus statusPruefen = hierarchieManager.pruefeDaten(hierarchieObjekt, hod);
            _status.merge(statusPruefen);
          }
          else
          {
            _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, "Kein Hierarchieobjekt vorhanden. "));
          }

          monitor.worked(1);

        }

        catch (Exception e)
        {
          _status
              .merge(new Status(Status.ERROR, Activator.PLUGIN_ID, "Pruefung Hierarchieobjekt ist fehlgeschlagen. ", e));
          return Status.CANCEL_STATUS;
        }
        finally
        {
          monitor.worked(3);
          monitor.done();
        }

        return Status.OK_STATUS;
      }
    };

    job.addJobChangeListener(new JobChangeAdapter() {

      @Override
      public void running(IJobChangeEvent event)
      {
        UIJob uiJob = new UIJob("TKABasis: Pruefung HO: aktualisiere GUI running") {

          @Override
          public IStatus runInUIThread(IProgressMonitor monitor)
          {
            // System.out.println("Running: Selektion deaktivieren?");
            aktiviereAlleAktionen(false);
            aktiviereButtons();
            return Status.OK_STATUS;
          }
        };
        uiJob.setUser(false);
        uiJob.setSystem(true);
        uiJob.schedule();
      }

      public void done(IJobChangeEvent event)
      {
        if (event.getResult().isOK())
        {

          final IStatus finalStatus = _status;

          Display.getDefault().syncExec(new Runnable() {

            @Override
            public void run()
            {
              ergebnisAusgabe("Prfen", finalStatus);
              
              boolean dirty = _editor.isDirty();
              if (finalStatus == null || finalStatus.matches(IStatus.ERROR))
              {
                doZustandsUebergang(TKABasisAktion.PRUEFEN);
                if (dirty)
                  _editor.setDirty(true);
              }
              else if (finalStatus.matches(IStatus.WARNING))
              {
                doZustandsUebergang(TKABasisAktion.PRUEFEN);
                if (dirty)
                  _editor.setDirty(true);
              }
              else if (finalStatus.isOK())
              {
                MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.OK
                    | SWT.ICON_INFORMATION);
                msg.setText("Prfen des Hierarchieobjekts");
                msg.setMessage("Prfung Ok!");
                msg.open();
                
                aktionAendernBearbeitungsmodus(null, _zustandsmaschine.getMomentZustand());
                if (dirty)
                  _editor.setDirty(true);
              }
              else
              {
                aktionAendernBearbeitungsmodus(null, _zustandsmaschine.getMomentZustand());
                if (dirty)
                  _editor.setDirty(true);
              }

            }
          });
          
          speichernMeldung(_status, true);
         
        }
        else
        {
          final IStatus finalStatus = new Status(Status.ERROR, Activator.PLUGIN_ID,
          "Pruefung Hierarchieobjekt vorzeitig beendet. Es sind Ausnahmen aufgetreten. ");
          
          _status.merge(finalStatus);
          speichernMeldung(_status, false);         
          
          Display.getDefault().syncExec(new Runnable() {

            @Override
            public void run()
            {
              ergebnisAusgabe("Prfen", finalStatus);
              aktionAendernBearbeitungsmodus(null, _zustandsmaschine.getMomentZustand());
            }
          });
        }

      }
    });

    job.setSystem(false);
    job.setUser(true);
    job.schedule();

  }

  private LinkedList<SystemObjektDaten> getSystemObjektDaten()
  {
    if (SYS_OUT)
      System.out.println("\ngetSystemObjektDaten: SODListe erstellen: ");
    
    LinkedList<DatenBereichSOKapsel> kapselnSo = _editor.getSOKapselnZumSichern();
    
    LinkedList<SystemObjektDaten> sodListe = new LinkedList<SystemObjektDaten>();
    
    for (DatenBereichSOKapsel kapSo : kapselnSo)
    {
      SystemObjektDaten sod = new SystemObjektDaten();
      
      sod.setPid(kapSo.getPidSystemObjekt());
      sod.setName(kapSo.getNameSystemObjekt());

      List<DatenBereichEditorKapsel> kapseln = kapSo.getKinderListe();
      
      int index = 0;
      
      for (DatenBereichEditorKapsel kap : kapseln)
      {
        Data data = kap.getDataSenden();
        if (kap.isTypAtg())
          sod.setAtgDaten(kap.getName(), data);
        else
          sod.setDynMengenDaten( kap.getName(), data );
                
        sod.setDatenGeaendert(kap.getName(), _editor.isDirty(kap.getId()));
        
        if (SYS_OUT)
          System.out.println("SystemObjektDaten.setDatenGeaendert(\"" + kap.getName() + "\", " + 
              sod.isDatenGeaendert(kap.getName()) + ")");
        
        index += 1;
      }
      
      if (SYS_OUT)
        System.out.println("Einfgen SOD: " + sod.getPid() + " mit Name " + sod.getName());
      
      sodListe.add(sod);
    }
    return sodListe;
  }


  private void ergebnisAusgabe(String strAktion, IStatus status)
  {
    // Ausgabe entsprechend TAnf-TKA-BPI-33 siehe Meldungsliste

    if (status.matches(IStatus.ERROR))
    {
      MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.OK | SWT.ICON_ERROR);
      msg.setText(strAktion + " des Hierarchieobjekts");
      msg.setMessage(strAktion + " ist fehlgeschagen!\n -> Siehe MeldungenKonfigAss!");
      msg.open();
    }
    else if (status.matches(IStatus.WARNING))
    {
      MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.OK
          | SWT.ICON_WARNING);
      msg.setText(strAktion + " des Hierarchieobjekts");
      msg.setMessage("Bitte Warnungen in MeldungenKonfigAss beachten!");
      msg.open();
    }
    else if (status.matches(IStatus.INFO))
    {
      MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.OK
          | SWT.ICON_INFORMATION);
      msg.setText(strAktion + " des Hierarchieobjekts");
      msg.setMessage("Informationen sind in MeldungenKonfigAss vorhanden!");
      msg.open();
    }

  }

  private void aktionKonfigAenderungInfo()
  {
    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object firstElement = selection.getFirstElement();

    Object parent = firstElement;
    while (parent instanceof HONavigatorElement)
    {
      parent = ((HONavigatorElement)parent).getParent();
    }

    if (parent instanceof KBNavigatorElement)
    {
      KBNavigatorElement kbne = (KBNavigatorElement)parent;

      ConfigurationArea kb = kbne.getKonfigurationsBereich();
      AttributeGroup atg = _dataModel.getAttributeGroup("atg.info");
      Data data = kb.getConfigurationData(atg);
      
      String info = "";
      String beschreibung = "";
      
      if (data != null){          
        info = data.getItem("kurzinfo").asTextValue().getValueText();
        beschreibung = data.getItem("beschreibung").asTextValue().getValueText();         
      }
          
      
      InfoKonfigAenderungModell2 model = new InfoKonfigAenderungModell2();
      model.setPid(kb.getPid());
      model.setName(kb.getName());
      
      model.setInfo(info);
      model.setBeschreibung(beschreibung);
      
      InfoKonfigAenderungAssistent2 wizard = new InfoKonfigAenderungAssistent2(model, false);

      wizard.init(PlatformUI.getWorkbench(), null);
      // Instantiates the wizard container with the wizard and opens it
      WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
      dialog.create();

      int ret = dialog.open();

      if (ret == 0)
      {
        short v = kb.getActiveVersion();
        Short version = new Short(v);
        version++;
        
        atg = _dataModel.getAttributeGroup("atg.konfigurationsnderungen");
        data = kb.getConfigurationData(atg);

        if (data != null)
        {

          Data copy = data.createModifiableCopy();
          Array array = copy.getItem("Konfigurationsnderung").asArray();
          int len = array.getLength();
          array.setLength(len + 1);
          array.getItem(len).getItem("Stand").asTimeValue().setMillis(model.getStand());
          array.getItem(len).getItem("Autor").asTextValue().setText(model.getAutor());
          array.getItem(len).getItem("Version").asTextValue().setText(version.toString());
          array.getItem(len).getItem("Grund").asTextValue().setText(model.getGrund());
          array.getItem(len).getItem("Text").asTextValue().setText(model.getText());

          data = copy;

        }
        else
        {
          data = AttributeBaseValueDataFactory.createAdapter(atg, AttributeHelper.getAttributesValues(atg));
          Array array = data.getItem("Konfigurationsnderung").asArray();
          array.setLength(1);
          array.getItem(0).getItem("Stand").asTimeValue().setMillis(model.getStand());
          array.getItem(0).getItem("Autor").asTextValue().setText(model.getAutor());
          array.getItem(0).getItem("Version").asTextValue().setText(version.toString());
          array.getItem(0).getItem("Grund").asTextValue().setText(model.getGrund());
          array.getItem(0).getItem("Text").asTextValue().setText(model.getText());
        }

        try
        {
          kb.setConfigurationData(atg, data);
        }
        catch (ConfigurationChangeException e)
        {
          e.printStackTrace();
        }
        
        atg = _dataModel.getAttributeGroup("atg.info");
        data = kb.getConfigurationData(atg);
        
        if (data != null)
        {
          
          Data copy = data.createModifiableCopy();
           
          copy.getItem("kurzinfo").asTextValue().setText(model.getInfo());
          copy.getItem("beschreibung").asTextValue().setText(model.getBeschreibung());
          
          data = copy;
          
        }
        else
        {
          data = AttributeBaseValueDataFactory.createAdapter(atg, AttributeHelper.getAttributesValues(atg));
          data.getItem("kurzinfo").asTextValue().setText(model.getInfo());
          data.getItem("beschreibung").asTextValue().setText(model.getBeschreibung());
        }
        
        try
        {
          kb.setConfigurationData(atg, data);
        }
        catch (ConfigurationChangeException e)
        {
          e.printStackTrace();
        }
      }
    }

  }

  private void aktionSystemObjektInfo()
  {
    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object firstElement = selection.getFirstElement();

    Object selektion = firstElement;

    if (selektion instanceof HONavigatorElement)
    {
      HONavigatorElement hoNe = (HONavigatorElement)selektion;

      SystemObject so = hoNe.getHierarchieObjekt().getSystemObjekt(0);
      ConfigurationObject co = (ConfigurationObject)so;
      AttributeGroup atg = _dataModel.getAttributeGroup("atg.info");
      Data data = co.getConfigurationData(atg);
      
      String info = "";
      String beschreibung = "";
      
      if (data != null){          
        info = data.getItem("kurzinfo").asTextValue().getValueText();
        beschreibung = data.getItem("beschreibung").asTextValue().getValueText();         
      }
               
      InfoKonfigAenderungModell2 model = new InfoKonfigAenderungModell2();
      model.setPid(so.getPid());
      model.setName(so.getName());
      
      model.setInfo(info);
      model.setBeschreibung(beschreibung);
      
      InfoSystemObjektAssistent wizard = new InfoSystemObjektAssistent(model, false);

      wizard.init(PlatformUI.getWorkbench(), null);
      // Instantiates the wizard container with the wizard and opens it
      WizardDialog dialog = new WizardDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell(), wizard);
      dialog.create();

      int ret = dialog.open();

      if (ret == 0)
      {       
        atg = _dataModel.getAttributeGroup("atg.info");
        data = co.getConfigurationData(atg);
        
//        System.out.println("Modelinfo: " + model.getInfo() + " / " + model.getBeschreibung());
        
        if (data != null)
        {
          
          Data copy = data.createModifiableCopy();
           
          copy.getItem("kurzinfo").asTextValue().setText(model.getInfo());
          copy.getItem("beschreibung").asTextValue().setText(model.getBeschreibung());
          
          data = copy;
          
        }
        else
        {
          data = AttributeBaseValueDataFactory.createAdapter(atg, AttributeHelper.getAttributesValues(atg));
          data.getItem("kurzinfo").asTextValue().setText(model.getInfo());
          data.getItem("beschreibung").asTextValue().setText(model.getBeschreibung());
        }
        
        try
        {
          co.setConfigurationData(atg, data);
        }
        catch (ConfigurationChangeException e)
        {
          e.printStackTrace();
        }
      }

    }

  }
  
  private void aktionUebernehmenJob()
  {
    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object firstElement = selection.getFirstElement();
    
    HONavigatorElement selHoNe = null;
    if (firstElement instanceof HONavigatorElement)
    {
      selHoNe = (HONavigatorElement)firstElement;
    }
    else
    {
      MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.ERROR, "Prfe Hierarchieobjekt " + firstElement,
          null);
  
      MultiStatus objektStatus = new MultiStatus(Activator.PLUGIN_ID, Status.ERROR,
          "Auswahl ist kein Hierarchieobjekt", null);
      status.add(objektStatus);
      speichernMeldung(status, false);
      return;
    }
    
    if (selHoNe != null)
    {

      TKABasisView view = TKABasisView.getView(_secondViewId);
      IHierarchieManager hierarchieManager = view.getModelInfo().getHierarchieManager();
      
      pruefenUebernehmenHOJob(selHoNe, hierarchieManager);

    }
  }
  
  /**
   * Aufruf des Jobs fr die Prfung vorm bernehmen des Hierarchie-Objekts und Warten bis dieser
   * Job zu Ende ist oder abgebrochen wird. Bei erfolgreicher Prfung bernehmen der Hierarchie-Objekts durchfhren.
   * 
   * @param selHoNe
   *          selektiertes Hierarchieobjekt-NavigatorElement
   * @param hm
   *          zur View gehrender HierarchieManager
   */ 
  private boolean pruefenUebernehmenHOJob(HONavigatorElement selHoNe, IHierarchieManager hm)
  {
    int index = 0;
    final LinkedList<SystemObjektDaten> sodListe = getSystemObjektDaten();
    final HierarchieObjekt hierarchieObjekt = selHoNe.getHierarchieObjekt();
    final HODaten hod = new HODaten();
    
    for (SystemObjektDaten sod : sodListe)
    {
      //Voraussetzung ist, dass beide Listen die gleiche Reihenfolge haben
      if ( hierarchieObjekt.getSystemObjekte().size() > index )
      {
        SystemObject so = hierarchieObjekt.getSystemObjekt(index);
        hod.setSystemObjektDaten(so, sod);
      }
      else
      {
        if (SYS_OUT)
          System.err.println("SystemObjekt " + index + " ist nicht im HierarchieObjekt vorhanden!");
      }
      index++;
    }
    
    if (!new PruefungenTools(hm.getHierarchie().getDataModel()).pruefeVorUebernehmen(hierarchieObjekt, hod))
    {
      return false;
    }
        
    final HONavigatorElement selektiertesHoNe = selHoNe;
    final IHierarchieManager hierarchieManager = hm;
    
    _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "bernehme Hierarchieobjekt " + hierarchieObjekt, null);
    MeldungenSpeicher.getInstanz().setStatus(_status);
    
    final Job job = new Job("TKABasis: Pruefung bernehmen HO") {

      protected IStatus run(IProgressMonitor monitor)
      {
        monitor.beginTask("Starte Pruefung...", 5);
        monitor.worked(1);

        try
        {

          if (hierarchieObjekt != null)
          {
            IStatus statusPruefen = hierarchieManager.pruefeDaten(hierarchieObjekt, hod);
            _status.add(statusPruefen);
          }
          else
          {
            _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, "Kein Hierarchieobjekt vorhanden. "));
          }

          monitor.worked(1);

        }

        catch (Exception e)
        {
          _status
              .merge(new Status(Status.ERROR, Activator.PLUGIN_ID, "Pruefung Hierarchieobjekt ist fehlgeschlagen. ", e));
          return Status.CANCEL_STATUS;
        }
        finally
        {
          monitor.worked(3);
          monitor.done();
        }

        return Status.OK_STATUS;
      }
    };

    job.addJobChangeListener(new JobChangeAdapter() {

      @Override
      public void running(IJobChangeEvent event)
      {
        UIJob uiJob = new UIJob("TKABasis: Pruefung bernehmen HO: aktualisiere GUI running") {

          @Override
          public IStatus runInUIThread(IProgressMonitor monitor)
          {
            aktiviereAlleAktionen(false);
            aktiviereButtons();
            return Status.OK_STATUS;
          }
        };
        uiJob.setUser(false);
        uiJob.setSystem(true);
        uiJob.schedule();
      }

      public void done(IJobChangeEvent event)
      {
        if (event.getResult().isOK())
        {
          final IStatus finalStatus = _status;
          
          Display.getDefault().syncExec(new Runnable() {

            @Override
            public void run()
            {
              List<HONavigatorElement> selektierteHoNes = new ArrayList<HONavigatorElement>();
              selektierteHoNes.add(selektiertesHoNe);
              
              if (finalStatus != null)
                aktionNachPrfung(TKABasisAktion.UEBERNEHMEN, finalStatus.getSeverity(), selektierteHoNes, hierarchieManager, sodListe);
              else
                aktionNachPrfung(TKABasisAktion.UEBERNEHMEN, Status.INFO, selektierteHoNes, hierarchieManager, sodListe);

              aktionAendernBearbeitungsmodus(null, _zustandsmaschine.getMomentZustand());
            }
          });
          speichernMeldung(_status, true);
          
        }
        else
        {
          final IStatus finalStatus = new Status(Status.ERROR, Activator.PLUGIN_ID,
          "Pruefung beim bernehmen vorzeitig beendet. Es sind Ausnahmen aufgetreten. "); 
          
          _status.merge(finalStatus);
          speichernMeldung(_status, false);

          Display.getDefault().syncExec(new Runnable() {

            @Override
            public void run()
            {
              ergebnisAusgabe("bernehmen", finalStatus);
             
              aktionAendernBearbeitungsmodus(null, _zustandsmaschine.getMomentZustand());
            }
          });
        }       

      }
    });

    job.setSystem(false);
    job.setUser(true);
    job.schedule();
    
    return _status.isOK();
    
  }

  private void aktionLoeschenJob()
  {

    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object[] selArray = selection.toArray();
    List<HONavigatorElement> selHONEs = new ArrayList<HONavigatorElement>();
    String selHONamen = "";

    for (int i = 0; i < selArray.length; i++)
    {
      // nur bei HONavigatorElement erlaubt
      if (selArray[i] instanceof HONavigatorElement)
      {
        HONavigatorElement ne = (HONavigatorElement)selArray[i];
        HierarchieObjekt ho = ne.getHierarchieObjekt();

        if (ho != null)
        {
          if (!selHONamen.isEmpty())
            selHONamen += ",\n";
          selHONamen += ho;

          selHONEs.add(ne);
        }
      }
    }

    if (selHONEs.size() > 0)
    {
//      MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.OK | SWT.CANCEL
//          | SWT.ICON_QUESTION);
//      if (selHONEs.size() == 1)
//      {
//        msg.setText("Lschen eines Hierarchieobjekts");
//        msg.setMessage("Soll das Hierarchieobjekt " + selHONamen + 
//            "\n- mit allen ggf. vorhandenen untergeordeneten Objekten -\nwirklich gelscht werden?");
//      }
//      else
//      {
//        msg.setText("Lschen von Hierarchieobjekten");
//        msg.setMessage("Sollen die Hierarchieobjekte\n" + selHONamen + 
//            "\n- mit allen ggf. vorhandenen untergeordeneten Objekten -\nwirklich gelscht werden?");
//      }
//      int result = msg.open();
//      if (result == SWT.CANCEL)
//        return;
      
      KonfigAssListDialog listDialog = new KonfigAssListDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
      if (selHONEs.size() == 1)
      {
       listDialog.setTitle("Lschen eines Hierarchieobjekts");
       listDialog.setMessage("Soll das Hierarchieobjekt " + selHONamen + 
           "\n- mit allen ggf. vorhandenen untergeordeneten Objekten -\nwirklich gelscht werden?" +
           "\n\nKonkret zu lschende Systemobjekte:");
      }
      else
      {
       listDialog.setTitle("Lschen von Hierarchieobjekten");
       listDialog.setMessage("Sollen die Hierarchieobjekte\n" + selHONamen + 
           "\n- mit allen ggf. vorhandenen untergeordeneten Objekten -\nwirklich gelscht werden?" +
           "\n\nKonkret zu lschende Systemobjekte:");
      }
           
      listDialog.setContentProvider(new ArrayContentProvider());
      listDialog.setLabelProvider(new LabelProvider());
      listDialog.setHelpContext(Activator.PLUGIN_ID + ".ho_loeschen");
      
      TKABasisView view = TKABasisView.getView(_secondViewId);
      IHierarchieManager hierarchieManager = view.getModelInfo().getHierarchieManager();
      
      List<HierarchieObjekt> selHOs = new LinkedList<HierarchieObjekt>();
      for (HONavigatorElement selObj: selHONEs)
      {
        selHOs.add(selObj.getHierarchieObjekt());
      }
      
      List<String> pids = new LinkedList<String>();
      for (SystemObject so: new LoeschenChecker(hierarchieManager).bestimmeKonkreteObjekte(selHOs))
      {
        pids.add(so.getPid());
      }

      listDialog.setInput(pids);
          
      int result = listDialog.open();
      
      if (result == Dialog.CANCEL)
      {
        return;
      }
      
      

      // Pfung auf offene Referenzen
      // if (preferences.getBoolean("selectedPruefungBeimLoeschen")) //Preferences sind im jeweiligen TKA
      if (true)
      {
        // System.out.println("Prfungen beim Lschen durchfhren");
        pruefenLoeschenHOsJob(selHONEs, hierarchieManager);
      }
      // else
      // {
      // loeschenHOs(selHONEs, hierarchieManager);
      // }

    }
  }

  private INavigatorElement loeschenHOs(List<HONavigatorElement> selHONEs, IHierarchieManager hierarchieManager)
  {
    INavigatorElement parentNe = null;

    for (int i = 0; i < selHONEs.size(); i++)
    {
      HONavigatorElement ne = (HONavigatorElement)selHONEs.get(i);
      HierarchieObjekt ho = ne.getHierarchieObjekt();
      IHierarchie hierarchie = hierarchieManager.getHierarchie();

      if (ho != null && 
          hierarchie.getUebergeordnetesObjekt(ho) != null) // Andernfalls ist das HO bereits nicht mehr in der Hierarchie enthalten
      {
        IStatus status = null;
        final TKAUpdate tkaUpdate = new TKAUpdate();
        hierarchieManager.getHierarchie().beobachterAnmelden(tkaUpdate);
        
        try
        {
          // HierarchieObjekt lschen
          status = hierarchieManager.loescheHO(ho);
        }
        finally
        {
          hierarchieManager.getHierarchie().beobachterAbmelden(tkaUpdate);
        }
        
        DatenModellUpdateListener.getInstanz().notifyDatenModellUpdateListener(this,
            tkaUpdate.getListeAktualisiert(), tkaUpdate.getListeGeloescht(), tkaUpdate.getListeNeu());
        
        if (_status == null)
          speichernMeldung(status, true);
        else
          _status.add(status);
        
        ergebnisAusgabe("Lschen", status);
        
        if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
        {
          // der Zustandbergang muss vor dem Lschen des Navigator-Elements erfolgen
          doZustandsUebergang(TKABasisAktion.LOESCHEN);
        }
        INavigatorElement parent = (INavigatorElement)ne.getParent();
        parentNe = parent;
        
      }
    }
    
    // Sysnchronisierung jetzt ber Notify der Hierarchie

    // letzten Parent zum Selektieren bergeben
    return parentNe;
  }

  /**
   * Aufruf des Jobs fr die Prfung der offenen Referenzen vorm Lschen von Hierarchie-Objekten und Warten bis dieser
   * Job zu Ende ist oder abgebrochen wird. Bei erfolgreicher Prfung Lschung der Hierarchie-Objekte durchfhren.
   * 
   * @param selHoNes
   *          Liste der selektierten Hierarchieobjekt-NavigatorElemente
   * @param hm
   *          zur View gehrender HierarchieManager
   */
 private void pruefenLoeschenHOsJob(List<HONavigatorElement> selHoNes, IHierarchieManager hm)
  {
    final List<HONavigatorElement> selektierteHoNes = selHoNes;
    final IHierarchieManager hierarchieManager = hm;
    
    _status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Lsche Hierarchieobjekte", null);
    MeldungenSpeicher.getInstanz().setStatus(_status);

    final Job job = new Job("TKABasis: Pruefung lschen HOs") {

      protected IStatus run(IProgressMonitor monitor)
      {
        monitor.beginTask("Starte Pruefung offene Referenzen...", 5);
        monitor.worked(1);

        try
        {
          if (selektierteHoNes.size() > 0)
          {
            List<HierarchieObjekt> selHos = new ArrayList<HierarchieObjekt>();
            for (HONavigatorElement hone : selektierteHoNes)
            {
              HierarchieObjekt ho = hone.getHierarchieObjekt();
              if (!ho.getHierarchieObjektTyp().isSystemObjektVorhanden())
                selHos.add(ho);
            }

            if ( selHos.size() > 0 )
            {
              LoeschenChecker loeschenChecker = new LoeschenChecker(hierarchieManager);
  
              IStatus statusPruefen = loeschenChecker.pruefe(selHos.toArray(new HierarchieObjekt[0]));
              _status.add(statusPruefen);
            }
            else
            {
              _status.merge(new Status(Status.OK, Activator.PLUGIN_ID, "Durch das Lschen werden keine konkreten Objekte gelscht. "));           
            } 
            monitor.worked(1);

          }
          else
          {
            _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, "Keine selektierten Hierarchieobjekte vorhanden. "));           
          }
        }
        catch (Exception e)
        {
          _status.merge(new Status(Status.ERROR, Activator.PLUGIN_ID, "Pruefung offene Referenzen ist fehlgeschlagen. ", e));
          return Status.CANCEL_STATUS;
        }
        finally
        {
          monitor.worked(3);
          monitor.done();
        }

        return Status.OK_STATUS;
      }
    };

    job.addJobChangeListener(new JobChangeAdapter() {

      @Override
      public void running(IJobChangeEvent event)
      {
        UIJob uiJob = new UIJob("TKABasis: Pruefung lschen HOs: aktualisiere GUI running") {

          @Override
          public IStatus runInUIThread(IProgressMonitor monitor)
          {
            aktiviereAlleAktionen(false);
            aktiviereButtons();
            return Status.OK_STATUS;
          }
        };
        uiJob.setUser(false);
        uiJob.setSystem(true);
        uiJob.schedule();
      }

      public void done(IJobChangeEvent event)
      {
        if (event.getResult().isOK())
        {

          final IStatus finalStatus = _status;

          Display.getDefault().syncExec(new Runnable() {

            @Override
            public void run()
            {
              if (finalStatus != null)
                aktionNachPrfung(TKABasisAktion.LOESCHEN, finalStatus.getSeverity(), selektierteHoNes, hierarchieManager, null);
              else
                aktionNachPrfung(TKABasisAktion.LOESCHEN, Status.INFO, selektierteHoNes, hierarchieManager, null);

              aktionAendernBearbeitungsmodus(null, _zustandsmaschine.getMomentZustand());
            }
          });
          speichernMeldung(_status, true);
          
        }
        else
        {
          final IStatus finalStatus = new Status(Status.ERROR, Activator.PLUGIN_ID,
              "Pruefung offene Referenzen vorzeitig beendet. Es sind Ausnahmen aufgetreten. ");
          _status.merge(finalStatus);
          speichernMeldung(_status, true);

          Display.getDefault().syncExec(new Runnable() {

            @Override
            public void run()
            {
              ergebnisAusgabe("Lschen", finalStatus);
              aktionAendernBearbeitungsmodus(null, _zustandsmaschine.getMomentZustand());
            }
          });
        }       

      }
    });

    job.setSystem(false);
    job.setUser(true);
    job.schedule(); // start as soon as possible

  }

  /**
   * Aktion, die je nach Status einer vorherigen Prfung ausgefhrt wird
   * 
   * @param aktion
   *          durchzufhrende Aktion
   * @param status
   *          Ergebnis der vorherigen Prfung
   * @param selHONEs
   *          Liste der selektierten Hierarchieobjekt-NavigatorElemente
   * @param hm
   *          zur View gehrender HierarchieManager 
   * @param sodListe
   *          Liste der SystemObjektDaten                  
   */
  private void aktionNachPrfung(TKABasisAktion aktion, int status, List<HONavigatorElement> selHONEs, IHierarchieManager hm, LinkedList<SystemObjektDaten> sodListe)
  {
    // Setzen der Ausgabevariablen entsprechend der Aktion
    String aktionText = "";
    String aktionMessage = "";
    switch (aktion)
    {
      case LOESCHEN:
        aktionText = "Lschen";
        aktionMessage = "gelscht";
        break;
      case UEBERNEHMEN:
        aktionText = "bernehmen";
        aktionMessage = "bernommen";
        break;
        
      default:
        break;
    }

    // Ausfhren der Aktionen entsprechend dem Status
    switch (status)
    {
      case Status.ERROR:
        ergebnisAusgabe(aktionText, new Status(status, Activator.PLUGIN_ID, ""));
        break;

      case Status.WARNING:
        //Die eigene Messagebox fr die Warnung wurde auf Grund von Protokollpunkt 25 fr die Warnung eliminiert
        //und in die 2.Messagebox inkl. der Warnung integriert.
//        ergebnisAusgabe(aktionText, new Status(status, Activator.PLUGIN_ID, ""));

        // MultiMeldung muss hier hinzugefgt werden, um die Meldungen lesen zu knnen
        // --> dann erscheint diese aber doppelt
        //speichernMeldung(_status, true);

        if (selHONEs.size() > 0)
        {
          MessageBox msgPL = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.OK
              | SWT.CANCEL | SWT.ICON_QUESTION);

          // tiefste Message zur Warnung finden
          IStatus tiefsteMessage = null;
          if (_status != null)
          {
            IStatus[] children = _status.getChildren();
            boolean gefunden = false;
            while (children.length > 0 && !gefunden)
            {
              int index = 0;
              boolean endeErreicht = true;
              for (IStatus child : children)
              {
                if (child.matches(Status.WARNING))
                {
                  IStatus[] childchildren = child.getChildren();
                  if (childchildren.length == 0)
                  {
                    tiefsteMessage = child;
                    gefunden = true;
                  }
                  else
                    children = childchildren;
                  
                  endeErreicht = false;
                  break;
                }
                index++;
              }
              if (endeErreicht)
                gefunden = true;
            }

          }

          if (selHONEs.size() == 1)
          {
            msgPL.setText(aktionText + " des Hierarchieobjekts");
            msgPL.setMessage("Bitte Warnungen (auch im MeldungenKonfigAss) beachten:\n" + tiefsteMessage.getMessage() +
            		"\n\nSoll das Hierarchieobjekt trotz Warnung wirklich " + aktionMessage + " werden?");
          }
          else
          {
            msgPL.setText(aktionText + " von Hierarchieobjekten");
            msgPL.setMessage("Bitte Warnungen beachten:\n" + _status.getMessage() +
                "\n\nSollen das Hierarchieobjekte trotz Warnung wirklich " + aktionMessage + " werden?");
          }
          int resultPL = msgPL.open();
          if (resultPL == SWT.OK)
          {
            aktionPruefungOk(aktion, selHONEs, hm, sodListe);
          }
        }

        break;

      case Status.INFO:
        ergebnisAusgabe(aktionText, new Status(status, Activator.PLUGIN_ID, ""));
        aktionPruefungOk(aktion, selHONEs, hm, sodListe);
        break;
        
      case Status.OK:
        aktionPruefungOk(aktion, selHONEs, hm, sodListe);
        break;

      default:
        break;
    }

  }

  /**
   * Aktion inklusive Refresh auf den Navigator und setzen des Items ausfhren
   * - nur aufzurufen, wenn die Prfung Ok war
   * 
   * @param aktion
   *          durchzufhrende Aktion
   * @param selHONEs
   *          Liste der selektierten Hierarchieobjekt-NavigatorElemente
   * @param hm
   *          zur View gehrender HierarchieManager
   * @param sodListe
   *          Liste der SystemObjektDaten  
   */
  private void aktionPruefungOk(TKABasisAktion aktion, List<HONavigatorElement> selHONEs, IHierarchieManager hm, LinkedList<SystemObjektDaten> sodList)
  {
    INavigatorElement parentNe = null;
    switch (aktion)
    {
      case LOESCHEN:        
        parentNe = loeschenHOs(selHONEs, hm);

        // Refresh des Navigators und setzen auf den letzten Parent
        if (parentNe != null)
        {
          _navigatorComposite.refreshNavigator();
          _ausgabeWarnungAenderung = false;
          _navigatorComposite.selektiereItem(parentNe);
          _ausgabeWarnungAenderung = true;
        }
        break;

      case UEBERNEHMEN:
        
          HONavigatorElement ne = selHONEs.get(0);
          HierarchieObjekt ho = ne.getHierarchieObjekt();

          if (ho != null)
          {
            HODaten hod = new HODaten();
            int index = 0;
            for (SystemObjektDaten sod : sodList)
            {
              //Voraussetzung ist, dass beide Listen die gleiche Reihenfolge haben
              SystemObject so = ho.getSystemObjekt(index);
              
              hod.setSystemObjektDaten(so, sod);
              index++;
            }
            
            // Bilde Systemobjekt-Liste vor der nderung
            List<SystemObject> listeHO = new Vector<SystemObject>();
            listeHO.addAll(ho.getSystemObjekte());
            
            IStatus status = hm.uebernehmeDaten(ho, hod);

            List<SystemObject> listeAktualisiert = new Vector<SystemObject>();
            List<SystemObject> listeGeloescht = new Vector<SystemObject>();
            List<SystemObject> listeNeu = new Vector<SystemObject>();
            
            for (int i = 0; i < ho.getSystemObjekte().size(); i += 1)
            {
              
              SystemObject alt = listeHO.get(i);
              SystemObject neu = ho.getSystemObjekt(i);
              
              // Falls sich Systemobjekte gendert haben
              if (alt != neu)
              {
                // Ursprngliches Objekt ist gelscht, und es kam ein neues Objekt dazu
                listeGeloescht.add(alt);
                listeNeu.add(neu);
              }
              else
              {
                listeAktualisiert.add(alt);
              }
            }

            DatenModellUpdateListener.getInstanz().notifyDatenModellUpdateListener(this,
                listeAktualisiert, listeGeloescht, listeNeu);
            
            if (_status == null)
              speichernMeldung(status, true);
            else
              _status.add(status);

            ergebnisAusgabe("bernehmen", status);
            
            Object parent = ne.getParent();
            if (parent instanceof INavigatorElement)
            {  
              _navigatorComposite.refreshNavigator((INavigatorElement)parent);
            }
            
            if (!status.matches(IStatus.ERROR | IStatus.CANCEL))
            {
              doZustandsUebergang(TKABasisAktion.UEBERNEHMEN);
              _editor.setDirty(false);
            }
          }

        break;

      default:
        break;
    }
  }

  /**
   * Selektiert den Vorgnger mit Prfung des HOT
   */
  private void aktionVorgaenger()
  {
    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    INavigatorElement firstElement = (INavigatorElement)selection.getFirstElement();

    // nur bei HONavigatorElement erlaubt
    if (firstElement instanceof HONavigatorElement)
    {
        
      // Vorgnger ermitteln ohne Beachtung des HOT
      INavigatorElement next = _navigatorComposite.getVorgaenger();
      if (next != null)
      {
        if (pruefenHOT(firstElement, next))
          _navigatorComposite.selektiereItem(next);
      }
    }
  }

  /**
   * Selektiert den Nachfolgers mit Prfung des HOT
   */
  private void aktionNachfolger()
  {
    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object firstElement = selection.getFirstElement();

    // nur bei HONavigatorElement erlaubt
    if (firstElement instanceof HONavigatorElement)
    {
      Object obj = ((HONavigatorElement)firstElement).getObjekt();
      IHierarchieObjektTyp hot = null;
      if (obj instanceof HierarchieObjekt)
        hot = ((HierarchieObjekt)obj).getHierarchieObjektTyp();

      // Vorgnger ermitteln ohne Beachtung des HOT
      INavigatorElement next = _navigatorComposite.getNachfolger();
      if (next != null)
      {
        Object nextObj = ((HONavigatorElement)next).getObjekt();
        IHierarchieObjektTyp nextHot = null;
        if (nextObj instanceof HierarchieObjekt)
          nextHot = ((HierarchieObjekt)nextObj).getHierarchieObjektTyp();

        // Selektion setzen, wenn gleiches HOT
        if ((hot.getName()).equals(nextHot.getName()))
          _navigatorComposite.selektiereItem(next);
      }
    }
  }

  /**
   * Selektiert des Elternteils
   */
  private void aktionVorgaengerEbene()
  {
    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object firstElement = selection.getFirstElement();

    // nur bei HONavigatorElement erlaubt
    if (firstElement instanceof HONavigatorElement)
    {
      HONavigatorElement hoNe = (HONavigatorElement)firstElement;
      if (hoNe.getParent() instanceof HONavigatorElement)
      {
        HONavigatorElement parentHoNe = (HONavigatorElement)hoNe.getParent();
        _navigatorComposite.selektiereItem(parentHoNe);
      }
    }
  }

  /**
   * Selektiert des 1.Kindes
   */
  private void aktionNachfolgerEbene()
  {
    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object firstElement = selection.getFirstElement();

    // nur bei HONavigatorElement erlaubt
    if (firstElement instanceof HONavigatorElement)
    {
      HONavigatorElement hoNe = (HONavigatorElement)firstElement;
      if (hoNe.hasChildren())
      {
        _navigatorComposite.expandNavigator(hoNe);
        HONavigatorElement startNe = (HONavigatorElement)hoNe.getChildren()[0];
        _navigatorComposite.selektiereItem(startNe);        
      }
    }
  }

  int ausgabeWarnungBeiAenderung(String text)
  {
    if (!Activator.getDefault().getPreferenceStore().getBoolean(DefsPara.AbfrageDatenVerwerfen))
    {
      // Abfrage ist in den Prferenzen ausgeschaltet
      // Verhlt sich so, als wrde in der MsgBox OK gedrckt
      _editor.setDirty(false);
      return SWT.OK;
    }
    
    if (_editor.isDirty())
    {
      MessageBox msg = new MessageBox(PlatformUI.getWorkbench().getDisplay().getActiveShell(), SWT.OK | SWT.CANCEL
          | SWT.ICON_WARNING);
      if (text == null)
        msg.setText("Wechsel des Bearbeitungsmodus");
      else
        msg.setText(text);
      msg.setMessage("Achtung die nderungen werden nicht bernommen!");
      int result = msg.open();

      if (result == SWT.CANCEL)
      {
        if (text != null)
        {
          // Beim Wechsel des Hierarchieobjekts, vorheriges Hierarchieobjekt wieder selektieren
          _aktualisiereEditor = false;
//          String nameHo = _editor.getSelektierteKapsel().getElternKapsel().getHierarchieObjekt().toString();
//          _navigatorComposite.sucheSelektiereItem(nameHo);
          HierarchieObjekt ho = _editor.getSelektierteKapsel().getElternKapsel().getHierarchieObjekt();
          _navigatorComposite.sucheSelektiereItem(_rootNavigatorElement, ho);
          _aktualisiereEditor = true;
        }
        return SWT.CANCEL;
      }
      else
      {
        _editor.setDirty(false);
      }
    }

    return SWT.OK;
  }

  /**
   * Meldung in der Meldungsliste "MeldungenkonfigAss" speichern
   * 
   * @param status          Multimeldung mit Status, die gespeichert werden soll   
   * @param ausgabeErwartet Flag, ob weitere Meldungen zu dieser Multimeldung erwartet und hinzugefgt werden
   */
  private void speichernMeldung(IStatus status, boolean ausgabeErwartet)
  {
    MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(status, System.currentTimeMillis(), ausgabeErwartet);
  }

  /**
   * Prfen des Zustandberganges beim Bearbeitungsmodus, ob dieser erlaubt ist
   * 
   * @param aktion
   *          auslsende TKABasisAktion
   * 
   * @return gueltig liefert, ob der Zustandsbergang erlaubt ist
   */
  private boolean pruefenZustandsUebergang(TKABasisAktion aktion)
  {
    return pruefenZustandsUebergang(aktion.getAktion().getAktionsNr());
  }

  /**
   * Prfen des Zustandberganges beim Bearbeitungsmodus, ob dieser erlaubt ist
   * 
   * @param aktionsNr
   *          Nummer der auslsenden TKABasisAktion
   * 
   * @return gueltig liefert, ob der Zustandsbergang erlaubt ist
   */
  private boolean pruefenZustandsUebergang(Integer aktionsNr)
  {
    return _zustandsmaschine.pruefeUebergang(aktionsNr);
  }

  /**
   * Durchfhren des Zustandberganges beim Bearbeitungsmodus, falls dieser erlaubt ist
   * 
   * @param aktion
   *          auslsende TKABasisAktion
   * 
   * @return gueltig liefert, ob der Zustandsbergang erlaubt ist
   */
  private boolean doZustandsUebergang(TKABasisAktion aktion)
  {
    if (pruefenZustandsUebergang(aktion))
    {
      Zustand zustandNeu = _zustandsmaschine.uebergang(aktion.getAktion().getAktionsNr());
      aktionAendernBearbeitungsmodus(aktion, zustandNeu);
      _aktionDesLetztenUebergangs = aktion;
      return true;
    }
    else
      return false;
  }

  /**
   * Aktion zum Setzen des Bearbeitungsmodus und der gltigen Steuerelemente
   * 
   * @param aktion
   *          auslsende TKABasisAktion
   * @param zustandBM
   *          Zustand zum Bearbeitungsmodus
   */
  private void aktionAendernBearbeitungsmodus(TKABasisAktion aktion, Zustand zustandBM)
  {
    Bearbeitungsmodus bm = Bearbeitungsmodus.valueOf(zustandBM.toString());
    aktionAendernBearbeitungsmodus(aktion, bm);
  }

  /**
   * Aktion zum Setzen des Bearbeitungsmodus und der gltigen Steuerelemente
   * 
   * @param aktion
   *          auslsende TKABasisAktion
   * @param bm
   *          Bearbeitungsmodus
   */
  private void aktionAendernBearbeitungsmodus(TKABasisAktion aktion, Bearbeitungsmodus bm)
  {
    aktiviereAktionen();

    // Achtung ab hier nur noch Steuerelemente (Aktionen und Buttons) deaktivieren

    switch (bm)
    {
      case DARSTELLUNG:
        setRadioButton(0);

        // System.out.println("Darstellung");
        if(_editor.getTabFolder().getItems().length > 0)
          _editor.setEditable(false);
        break;
      case BEARBEITUNG:
        setRadioButton(1);
        _editor.setEditable(true);
        
        // Aktion zum buttonUebernehmen wird im Editor gesetzt
        break;
      case NEUANLAGE:
        setRadioButton(2);
        if (aktion == null)
        {
          boolean dirty = _editor.isDirty();
          _editor.setEditable(true);
          _editor.setDirty(dirty);
        }
        else
        {
          _editor.setEditable(true);
          _editor.setDirty(false);
        }
        
        // Aktion zum buttonUebernehmen wird im Editor gesetzt
        break;
      default:
        break;
    }

    deaktiviereAktionenbzglSelektion();

    // Buttons erst hier anhand der Aktionen setzen, damit die Buttons nich flackern
    aktiviereButtons();

  }

  /**
   * Buttons entsprechend ihren Aktionen setzen; bernehmen-Button niemals auf true setzen, da dies im Editor erfolgt
   */
  private void aktiviereButtons()
  {
    boolean enable = _actionRegistry.getAction(TKABasisAktion.NEU.getAktion().getAktionsID()).isEnabled();
    buttonNeu.setEnabled(enable);
    enable = _actionRegistry.getAction(TKABasisAktion.NEUUO.getAktion().getAktionsID()).isEnabled();
    buttonNeuUO.setEnabled(enable);
    enable = _actionRegistry.getAction(TKABasisAktion.KOPIEREN.getAktion().getAktionsID()).isEnabled();
    buttonKopieren.setEnabled(enable);
    enable = _actionRegistry.getAction(TKABasisAktion.DEFAULT.getAktion().getAktionsID()).isEnabled();
    buttonDefault.setEnabled(enable);
    enable = _actionRegistry.getAction(TKABasisAktion.PRUEFEN.getAktion().getAktionsID()).isEnabled();
    buttonPruefen.setEnabled(enable);
    enable = _actionRegistry.getAction(TKABasisAktion.UEBERNEHMEN.getAktion().getAktionsID()).isEnabled();
    // nur bei False setzen, da True im Editor gesetzt wird
    if (!enable)
      buttonUebernehmen.setEnabled(false);
    enable = _actionRegistry.getAction(TKABasisAktion.LOESCHEN.getAktion().getAktionsID()).isEnabled();
    buttonLoeschen.setEnabled(enable);
  }

  /**
   * Setzen des Radio-Buttons und der Hintergrund-Farbe des Bearbeitungsmodus mit dem gegebenen Index
   * 
   * @param index
   *          Index des Radio-Buttons
   */
  private void setRadioButton(int index)
  {

    radioButton[0].setEnabled(pruefenZustandsUebergang(TKABasisAktion.DARSTELLEN));
    radioButton[1].setEnabled(pruefenZustandsUebergang(TKABasisAktion.BEARBEITEN));
    radioButton[2].setEnabled(pruefenZustandsUebergang(TKABasisAktion.NEU));

    radioButton[0].setSelection(false);
    radioButton[1].setSelection(false);
    radioButton[2].setSelection(false);

    radioButton[index].setEnabled(true);
    radioButton[index].setSelection(true);

    Color farbe = BMFARBEN[index];

    composite4.setBackground(farbe);
    radioButton[0].setBackground(farbe);
    radioButton[1].setBackground(farbe);
    radioButton[2].setBackground(farbe);
  }

  /**
   * Alle Aktionen zu den Steuerelementen de-/aktivieren entsprechend ihrer Gltigkeit beim momentanen Zustand.
   * 
   */
  private void aktiviereAktionen()
  {
    Iterator<IAction> actions = _actionRegistry.getActions();

    while (actions.hasNext())
    {
      IAction iAction = (IAction)actions.next();
      Integer aktionNr = TKABasisAktion.getAktionsNr(iAction.getId());
      if (aktionNr != null)
        iAction.setEnabled(pruefenZustandsUebergang(aktionNr));
    }
    
    // Die Buttons werden erst anhand der aktiven Aktionen in aktionAendernBearbeitungsmodus gesetzt, damit die Buttons nich
    // flackern

  }

  /**
   * Alle Aktionen zu den Steuerelementen de-/aktivieren, die nicht permanent aktiviert sind. Alle noch nicht
   * realisierten Steuerelemente deaktivieren.
   * 
   * @param enable
   *          aktivieren/deaktivieren
   */
  private void aktiviereAlleAktionen(boolean enable)
  {
    Iterator<IAction> actions = _actionRegistry.getActions();
    while (actions.hasNext())
    {
      IAction iAction = (IAction)actions.next();
      iAction.setEnabled(enable);
    }

    // Die Buttons werden erst anhand der aktiven Aktionen in aktionAendernBearbeitungsmodus gesetzt, damit die Buttons nich
    // flackern

  }

  /**
   * Aktionen zu den Steuerelementen deaktivieren, falls das erste selektierte Element oder dessen Hierarchie diese
   * nicht erlaubt.
   * 
   */
  private void deaktiviereAktionenbzglSelektion()
  {
    if (SYS_OUT)
      System.out.println("Bestimmte Steuerelemente entsprechend dem ersten selektierten Element deaktivieren!");

    // Steuerelemente in Abhngigkeit des ersten selektierten Elementes und der Hierarchie deaktivieren
    IStructuredSelection selection = (IStructuredSelection)_navigatorComposite.getNavigatorSelection();
    Object firstElement = selection.getFirstElement();

    if (firstElement == null)
    {
      // ohne Selektion keine Aktionen erlaubt
      radioButton[1].setEnabled(false);
      radioButton[2].setEnabled(false);
      aktiviereAlleAktionen(false);
      _editor.setEditable(false);
      return;
    }

    AbstractNavigatorElement ne = (AbstractNavigatorElement)firstElement;

    if (!ne.isVerschiebbar())
    {
      // keinerlei Aktionen erlaubt, da nicht Konfigurationsverantwortlicher
      radioButton[1].setEnabled(false);
      radioButton[2].setEnabled(false);
      aktiviereAlleAktionen(false);
      _editor.setEditable(false);

      // Ausnahme an dieser Stelle der Button Default Parameter, der hier bei HONavigatorElementen
      // immer aktiv bleiben soll, falls Default Parameter vorhanden sind
      if (ne instanceof HONavigatorElement)
      {
        HierarchieObjekt ho = ((HONavigatorElement)ne).getHierarchieObjekt();
        if (ho != null && DefaultParameter.hatDefaultParameter(ho, _dataModel))
          _actionRegistry.getAction(TKABasisAktion.DEFAULT.getAktion().getAktionsID()).setEnabled(true);
      }
      return;
    }     

    // Selektion ist vorhanden und nderungen sind erlaubt, da Konfigurationsverantwortlicher

    // erlaubte HOTs zum Objekt ermitteln
    TKABasisView view = TKABasisView.getView(_secondViewId);
    IHierarchieManager hierarchieManager = view.getModelInfo().getHierarchieManager();
    IHierarchieObjektTyp[] uHots = hierarchieManager.getUntergeordneteHOTs(ne.getObjekt());
    
    boolean systemObjektNichtAenderbar = false;
    if ( ne instanceof HONavigatorElement)
    {
      HierarchieObjekt ho = ((HONavigatorElement)ne).getHierarchieObjekt();
      systemObjektNichtAenderbar = ho.getHierarchieObjektTyp().isSystemObjektNichtAenderbar();
    }
    
    if (uHots == null || uHots.length == 0 || systemObjektNichtAenderbar)
    {
      // Keine untergeordneten Objekte
      // System.out.println("Neu(u.O.)-Button deaktivieren da keine untergeordneten Objekttypen vorhanden");
      _actionRegistry.getAction(TKABasisAktion.NEUUO.getAktion().getAktionsID()).setEnabled(false);
    }

    if (_editor == null || _editor.getTabList().length == 0)
    {
      // System.out.println("bernehmen-Button deaktivieren da kein Editor");
      _actionRegistry.getAction(TKABasisAktion.UEBERNEHMEN.getAktion().getAktionsID()).setEnabled(false);
    }

    if (!pruefeEinfuegen(ne, hierarchieManager))
    {
      _actionRegistry.getAction(TKABasisAktion.EINFUEGEN.getAktion().getAktionsID()).setEnabled(false);
    }

    if (ne instanceof KBNavigatorElement)
    {
      // System.out.println("Steuerelemente deaktivieren da KBNavigatorElement");
      radioButton[1].setEnabled(false);
      if (!radioButton[2].getSelection())
        radioButton[2].setEnabled(false);

      _actionRegistry.getAction(TKABasisAktion.NEU.getAktion().getAktionsID()).setEnabled(false);
      _actionRegistry.getAction(TKABasisAktion.BEARBEITEN.getAktion().getAktionsID()).setEnabled(false);
      _actionRegistry.getAction(TKABasisAktion.KOPIEREN.getAktion().getAktionsID()).setEnabled(false);
      _actionRegistry.getAction(TKABasisAktion.KOPIERENHO.getAktion().getAktionsID()).setEnabled(false);
      _actionRegistry.getAction(TKABasisAktion.KOPIERENHOMUO.getAktion().getAktionsID()).setEnabled(false);
      _actionRegistry.getAction(TKABasisAktion.VERSCHIEBEN.getAktion().getAktionsID()).setEnabled(false);
      _actionRegistry.getAction(TKABasisAktion.LOESCHEN.getAktion().getAktionsID()).setEnabled(false);
      _actionRegistry.getAction(TKABasisAktion.DEFAULT.getAktion().getAktionsID()).setEnabled(false);
      _actionRegistry.getAction(TKABasisAktion.PRUEFEN.getAktion().getAktionsID()).setEnabled(false);
      _actionRegistry.getAction(TKABasisAktion.KONFIGAENDERUNGINFO.getAktion().getAktionsID()).setEnabled(true);
      _actionRegistry.getAction(TKABasisAktion.SOINFO.getAktion().getAktionsID()).setEnabled(false);
      _actionRegistry.getAction(TKABasisAktion.VORGAENGER.getAktion().getAktionsID()).setEnabled(false);
      _actionRegistry.getAction(TKABasisAktion.NACHFOLGER.getAktion().getAktionsID()).setEnabled(false);
      _actionRegistry.getAction(TKABasisAktion.VORGAENGEREBENE.getAktion().getAktionsID()).setEnabled(false);
      _actionRegistry.getAction(TKABasisAktion.NACHFOLGEREBENE.getAktion().getAktionsID()).setEnabled(false);

      // Die Buttons werden erst anhand der aktiven Aktionen in aktionAendernBearbeitungsmodus gesetzt, damit die Buttons nich
      // flackern

    }
    else
    // HONavigatorElement
    {
      _actionRegistry.getAction(TKABasisAktion.KONFIGAENDERUNGINFO.getAktion().getAktionsID()).setEnabled(false);
      
      IAction actionVor = _actionRegistry.getAction(TKABasisAktion.VORGAENGER.getAktion().getAktionsID());
      if (actionVor.isEnabled())
      {
        boolean hatVorgnger = pruefenHOT(ne, _navigatorComposite.getVorgaenger());
        if (!hatVorgnger)
          actionVor.setEnabled(false);
      }
      IAction actionNach = _actionRegistry.getAction(TKABasisAktion.NACHFOLGER.getAktion().getAktionsID());
      if (actionNach.isEnabled())
      {
        boolean hatNachfolger = pruefenHOT(ne, _navigatorComposite.getNachfolger());
        if (!hatNachfolger)
          actionNach.setEnabled(false);
      }

      if (ne.getParent() instanceof KBNavigatorElement)
        _actionRegistry.getAction(TKABasisAktion.VORGAENGEREBENE.getAktion().getAktionsID()).setEnabled(false);
      if (!ne.hasChildren())
        _actionRegistry.getAction(TKABasisAktion.NACHFOLGEREBENE.getAktion().getAktionsID()).setEnabled(false);

      // keine DefaultParameter vorhanden?
      HierarchieObjekt ho = ((HONavigatorElement)ne).getHierarchieObjekt();
      if (ho != null && !DefaultParameter.hatDefaultParameter(ho, _dataModel))
        _actionRegistry.getAction(TKABasisAktion.DEFAULT.getAktion().getAktionsID()).setEnabled(false);

      Object ueObj = hierarchieManager.getHierarchie().getUebergeordnetesObjekt(ho);
      boolean ueObjNichtAenderbar = false;
      if ( ueObj instanceof HierarchieObjekt)
        ueObjNichtAenderbar = ((HierarchieObjekt)ueObj).getHierarchieObjektTyp().isSystemObjektNichtAenderbar();

      IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();

      if (hierarchieManager.pruefeNeuanlageVerboten(ueObj, hot) || ueObjNichtAenderbar)
      {
        _actionRegistry.getAction(TKABasisAktion.NEU.getAktion().getAktionsID()).setEnabled(false);
      }

      if (SYS_OUT)
        System.out.println("HOT " + hot.getName() + " hat vorhandene SO: " + hot.isSystemObjektVorhanden());

      SystemObject so = (SystemObject)ho.getSystemObjekt(0);
     
      if (hot.isSystemObjektVorhanden())
      {
        _actionRegistry.getAction(TKABasisAktion.KOPIEREN.getAktion().getAktionsID()).setEnabled(false);
        _actionRegistry.getAction(TKABasisAktion.KOPIERENHO.getAktion().getAktionsID()).setEnabled(false);
        _actionRegistry.getAction(TKABasisAktion.KOPIERENHOMUO.getAktion().getAktionsID()).setEnabled(false);
        _actionRegistry.getAction(TKABasisAktion.VERSCHIEBEN.getAktion().getAktionsID()).setEnabled(false);       
      }
      
      if (hot.isSystemObjektNichtAenderbar() || !HierarchieObjekt.isSystemObjektAenderbar(so))
      {
        _actionRegistry.getAction(TKABasisAktion.NEUUO.getAktion().getAktionsID()).setEnabled(false);
        _actionRegistry.getAction(TKABasisAktion.PRUEFEN.getAktion().getAktionsID()).setEnabled(false);
        _actionRegistry.getAction(TKABasisAktion.BEARBEITEN.getAktion().getAktionsID()).setEnabled(false);
        _actionRegistry.getAction(TKABasisAktion.EINFUEGEN.getAktion().getAktionsID()).setEnabled(false);
        radioButton[1].setEnabled(false);
      }

    }
    
    List<IHierarchieObjektTyp> ausschlussListe = new ArrayList<IHierarchieObjektTyp>();
    
    if (uHots != null)
    {
      for (IHierarchieObjektTyp hot : uHots)
      {
        if (hierarchieManager.pruefeNeuanlageVerboten(ne.getObjekt(), hot))
          ausschlussListe.add(hot);      
      }
      if (ausschlussListe.size() >= uHots.length)
        _actionRegistry.getAction(TKABasisAktion.NEUUO.getAktion().getAktionsID()).setEnabled(false);      
    }

  }

  /**
   * Prfen, ob das Einfgen erlaubt ist
   * 
   * @param ne
   * @param hierarchieManager
   * @return
   */
  private boolean pruefeEinfuegen(AbstractNavigatorElement ne, IHierarchieManager hierarchieManager)
  {
    boolean einfuegenErlaubt = false;

    if (_kopierteHoNes.size() == 0 || _kopierModus.equals(KopierModus.KEIN))
      return false;

    // erlaubte HOTs zum Objekt ermitteln
    IHierarchie hierarchie = hierarchieManager.getHierarchie();
    IHierarchieObjektTyp[] uHots = hierarchieManager.getUntergeordneteHOTs(ne.getObjekt());   
    
    if (uHots == null || uHots.length == 0)
      return false;

    einfuegenErlaubt = true;
    for (int i = 0; i < _kopierteHoNes.size(); i++)
    {
      HONavigatorElement kopiertesHoNe = _kopierteHoNes.get(i);
      HierarchieObjekt ho = kopiertesHoNe.getHierarchieObjekt();

      // Verschiebung ins gleiche Element verboten
      if (kopiertesHoNe.getParent() == ne && _kopierModus.equals(KopierModus.VERSCHIEBEN))
      {
        einfuegenErlaubt = false;
        break;
      }
      
      if (Arrays.asList(uHots).contains(ho.getHierarchieObjektTyp()))
      {
        ConfigurationObject ko = (ConfigurationObject)ho.getSystemObjekte().get(0);
        if (ko != null)
        {
          // zugehriges KBNavigatorElement ermitteln
          Object parent = ne;
          while (parent instanceof HONavigatorElement)
          {
            parent = ((HONavigatorElement)parent).getParent();
          }
          if (parent instanceof KBNavigatorElement)
          {
            ConfigurationArea kb = ((KBNavigatorElement)parent).getKonfigurationsBereich();
//            if (ko.getConfigurationArea().equals(kb)) // KBs sind gleich => Einfgen ist immer mglich
            //Konfigurationsbereich des Hierarchieobjektes statt des Systemobjektes zum Vergleich heranziehen
            if (hierarchie != null && hierarchie.getKonfigurationsBereich(ho) != null && hierarchie.getKonfigurationsBereich(ho).equals(kb)) // KBs sind gleich => Einfgen ist immer mglich
            {
              // Einfgen erlaubt, wenn dies fr alle kopierten HoNes gilt
              if (SYS_OUT) 
                System.out.println("Einfgen erlaubt, da gleicher KB");
            }
            else if (_kopierModus.equals(KopierModus.VERSCHIEBEN)/* && hierarchie.alleObjekteAktiviert(ho, true)*/)
            // KBs sind unterschiedlich => Einfgen nur nach Verschieben mglich, nur fr aktivierte Objekte
            {
              // Einfgen erlaubt, wenn dies fr alle kopierten HoNes gilt
//              if (_debug)
//                System.out.println("Einfgen erlaubt, nach Verschieben und fr aktivierte Objekte");
            }
            else
            {
//              if (_debug)
//                System.out.println("Einfgen nicht erlaubt, wegen ko = " + ko.getConfigurationArea().getPid() +
//                 " kb = "
//                 + kb.getPid() + " alleObjekteAktiviert = " + hierarchie.alleObjekteAktiviert(ho, true));
//              einfuegenErlaubt = false;
              break;
            }
          }
          else
          {
            einfuegenErlaubt = false;
            break;
          }
        }
        else
        {
          einfuegenErlaubt = false;
          break;
        }
      }
      else
      {
        einfuegenErlaubt = false;
        break;
      }
    }// for

    return einfuegenErlaubt;
  }

  /**
   * Prfung des HOT
   */
  private boolean pruefenHOT(INavigatorElement first, INavigatorElement next)
  {
    if (first == null || next == null)
      return false;

    Object obj = ((HONavigatorElement)first).getObjekt();
    IHierarchieObjektTyp hot = null;
    if (obj instanceof HierarchieObjekt)
      hot = ((HierarchieObjekt)obj).getHierarchieObjektTyp();
    else
      return false;

    Object nextObj = ((HONavigatorElement)next).getObjekt();
    IHierarchieObjektTyp nextHot = null;
    if (nextObj instanceof HierarchieObjekt)
      nextHot = ((HierarchieObjekt)nextObj).getHierarchieObjektTyp();
    else
      return false;

    // Prfung auf gleiches HOT
    if ((hot.getName()).equals(nextHot.getName()))
      return true;
    else
      return false;

  }

  public void setConfigDataModel(ConfigDataModel dataModel, String pfad)
  {
    _dataModel = dataModel;

    _kopierteHoNes.clear();
    _kopierModus = KopierModus.KEIN;

    if (dataModel != null)
    {
//      System.out.println("Data nicht leer");
      _editor.setDataModel(_dataModel);

      if (pfad != null && !label1.isDisposed())
      {
        String text = label1.getText();

        if (text.contains("'"))
        {
          String sub = text.substring(0, text.lastIndexOf("'") + 1);
          sub += " " + pfad;
          label1.setText(sub);
          group4.layout(true);
        }
      }

      if (_navigatorComposite != null)
      {
//        System.out.println("Data leer und _navigatorComposite nicht leer");
        TKABasisView view = TKABasisView.getView(_secondViewId);

        IHierarchieManager hierarchieManager = view.getModelInfo().getHierarchieManager();
        
        KBHierarchie hierarchie = new KBHierarchie(dataModel, hierarchieManager);

        if (hierarchie != null)
        {
          try
          {
            for (ConfigurationArea kb: dataModel.getAllConfigurationAreas().values())
            {
              hierarchie.kbHinzufuegen(kb);
            }
          }
          catch (RuntimeException e)
          {
            setConfigDataModel(null, "");
            
            MultiStatus status = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Zuweisen des Datenmodells zu TKA-" + _secondViewId, null);
            status.add(new Status(Status.ERROR, Activator.PLUGIN_ID, "Beim Erzeugen der Hierarchie ist eine Ausnahme aufgetreten", e));
            MeldungenSpeicher.getInstanz().hinzufuegenMultimeldung(status, System.currentTimeMillis(), false);
            
            return;
          }
          
          IBildungsRegeln bildungsRegeln = view.getModelInfo().getBildungsRegeln();
          if (bildungsRegeln != null)
          {
            bildungsRegeln.setHierarchie(hierarchie);
          }

          IPruefungen pruefungen = view.getModelInfo().getPruefungen();
          if (pruefungen != null)
          {
            pruefungen.setHierarchie(hierarchie);
          }

          // System.out.println("setConfigDataModel: DataModel: " + dataModel + " Pfad: " + pfad);

          _rootNavigatorElement = new NavigatorElement(null, null, null, null);
          _navigatorComposite.setHierarchie(hierarchie);
          _navigatorComposite.initialisiereBaumStruktur(_rootNavigatorElement, null);
          _navigatorComposite.setNavigatorInput(_rootNavigatorElement);

          // Bearbeitungsmodus wieder auf Darstellung setzen
          Zustand momentZustand = _zustandsmaschine.getMomentZustand();
          Bearbeitungsmodus momentBM = Bearbeitungsmodus.valueOf(momentZustand.toString());
          if (momentBM != Bearbeitungsmodus.DARSTELLUNG)
            doZustandsUebergang(TKABasisAktion.SELEKTIEREN);
        }
      }
    }
    else
    {
      if (pfad != null && !label1.isDisposed())
      {
        if (pfad != null)
        {
          String sub = label1.getText().substring(0, label1.getText().lastIndexOf("'") + 1);
          label1.setText(sub);
        }
        group4.layout(true);
      }

      // System.out.println("DataModel: null Pfad: " + pfad);

      if (_navigatorComposite != null)
      {
//        System.out.println("Data leer und _navigatorComposite nicht leer");
        _navigatorComposite.setHierarchie(null);
        INavigatorElement root = new NavigatorElement(null, null, null, null);
        _navigatorComposite.setNavigatorInput(root);

        _editor.loescheDarstellungsBereich();

        // Bearbeitungsmodus wieder auf Darstellung setzen
        Zustand momentZustand = _zustandsmaschine.getMomentZustand();
        Bearbeitungsmodus momentBM = Bearbeitungsmodus.valueOf(momentZustand.toString());
        if (momentBM != Bearbeitungsmodus.DARSTELLUNG)
          doZustandsUebergang(TKABasisAktion.SELEKTIEREN);
      }
      else
      {
//        System.out.println("Data leer und _navigatorComposite leer");
        doZustandsUebergang(TKABasisAktion.SELEKTIEREN);        
      }
    }
  }

  public ActionRegistry getActionRegistry()
  {
    return _actionRegistry;
  }

  public Action getAction(String id)
  {
    return (Action)_actionRegistry.getAction(id);
  }

  public ConfigDataModel getDataModel()
  {
    return _dataModel;
  }

  public void setDataModel(ConfigDataModel dataModel)
  {
    _dataModel = dataModel;

    _editor.setDataModel(_dataModel);

  }

  public EndlicherAutomat getZustandsmaschine()
  {
    return _zustandsmaschine;
  }

  public void setZustandsmaschine(EndlicherAutomat zustandsmaschine)
  {
    _zustandsmaschine = zustandsmaschine;
  }

  /**
   * 
   * @return die Aktion, die den bergang in den bergang in den aktuellen Zustand bewirkt hat
   */
  public TKABasisAktion getAktionDesLetztenUebergangs()
  {
    return _aktionDesLetztenUebergangs;
  }
  
  TabbedDatenBereichEditorComposite getEditor()
  {
    return _editor;
  }
  
  NavigatorComposite getNavigatorComposite()
  {
    return _navigatorComposite;
  }
  
}
