/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.datenbereich.datenbasis;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Stellt eine Listendarstellung der Daten des GTM als {@link Composite} bereit.
 * 
 * @author HofmannJ
 * 
 */
public class GtmListeCompositeOffline extends Composite
{
	/**
	 * Darstellung von SystemObjekten
	 */
	public static final int _darstlungSystemObjekte = 0;

	/**
	 * Darstellung von SystemObjekten und ObjektTypen
	 */
	public static final int _darstlungSystemObjekteObjektTypen = 1;

	/**
	 * Darstellung von SystemObjekten, ObjektTypen und AttributGruppen
	 */
	public static final int _darstlungSystemObjekteObjektTypenAttributGruppen = 2;

	/**
	 * Darstellung von SystemObjekten, ObjektTypen, AttributGruppen und Aspekten
	 */
	public static final int _darstlungSystemObjekteObjektTypenAttributGruppenAspekte = 3;

	/**
	 * Darstellung von SystemObjekten, ObjektTypen, AttributGruppen, Aspekten und
	 * der Simulationsvariante
	 */
	public static final int _darstlungSystemObjekteObjektTypenAttributGruppenAspekteSimVar = 4;

	private int _darstellungsArt = 3;

	/**
	 * Verbindung zum DaV
	 */
	private GtmDatenBasisOffline _gtmDatenBasis = null;
	private SashForm _formBasis;
	private Composite _compositeLinks;
	private Composite _compositeRechts;
	private Table _tableSystemObjekte = null;
	private TableViewer _tvSystemObjekte = null;
	private ISelectionChangedListener _listenerTvSystemObjekte = null;
	private Table _tableObjektTypen = null;
	private TableViewer _tvObjektTypen = null;
	private ISelectionChangedListener _listenerTvObjektTypen = null;
	private Table _tableAttributGruppen = null;
	private TableViewer _tvAttributGruppen = null;
	private ISelectionChangedListener _listenerTvAttributGruppen = null;
	private Table _tableAspekte = null;
	private TableViewer _tvAspekte = null;
	private ISelectionChangedListener _listenerTvAspekte = null;

	private GtmListeSelektionsKopf _selektionObjektListe = null;
	private FilterBox _filterBoxObjekte = null;
	
	private GtmListeSelektionsKopf _selektionObjektTypen = null;
	private GtmListeSelektionsKopf _selektionAttributGruppen = null;
	private GtmListeSelektionsKopf _selektionAspekte = null;
	private int _simulationsVariante = -1;

	/**
	 * Konstruktor zur Darstellung eines Composites zur Datenidentifikation.
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}, auf dessen Basis die Darstellung
	 *          erfolgen soll.
	 * @param stil
	 *          {@link SWT}-Stil dieses {@link Composite}s.
	 * @param gtmDatenBasis
	 *          {@link GtmDatenBasis}, auf dessen Basis die Darstellung erfolgen
	 *          soll.
	 * @param darstellungsArt
	 *          Die Art der Darstellung ist whlbar ber
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekte},
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypen},
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypenAttributGruppen},
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypenAttributGruppenAspekte}
	 *          sowie
	 *          {@link GtmListeCompositeOffline#_darstlungSystemObjekteObjektTypenAttributGruppenAspekteSimVar}.
	 */
	public GtmListeCompositeOffline(Composite parent, int stil,
			GtmDatenBasisOffline gtmDatenBasis, int darstellungsArt)
	{
		super(parent, stil);

		this._darstellungsArt = darstellungsArt;
		this._gtmDatenBasis = gtmDatenBasis;

		initialisierung();
	}

	private void initialisierung()
	{
		setLayout(new FillLayout());
		_formBasis = new SashForm(this, SWT.HORIZONTAL);

		erzeugeCompositeLinks(_formBasis);
		erzeugeCompositeRechts(_formBasis);

		if (_darstellungsArt > 0)
		{
			_formBasis.setWeights(new int[] { 50, 50 });
		}
		else
		{
			_formBasis.setWeights(new int[] { 0, 100 });
		}

		fuelleAlleListenSelektionsKoepfe();
	}

	/**
	 * Erzeugt das {@link Composite} auf der linken Seite
	 * 
	 * @param parent
	 *          Eltern-{@link Composite}
	 */
	private void erzeugeCompositeLinks(Composite parent)
	{
		_compositeLinks = new Composite(parent, SWT.BORDER);
		_compositeLinks.setLayout(new FillLayout());

		SashForm formLinks = new SashForm(_compositeLinks, SWT.VERTICAL);

		Composite objTyp = null;
		Composite atg = null;
		Composite asp = null;
		Composite simvar = null;

		FillLayout fillLayout = new FillLayout();

		if (_darstellungsArt >= 1)
		{
			objTyp = new Composite(formLinks, SWT.NONE);
			objTyp.setLayout(fillLayout);
			erzeugeCompositeObjektTypen(objTyp);
		}
		if (_darstellungsArt >= 2)
		{
			atg = new Composite(formLinks, SWT.NONE);
			atg.setLayout(fillLayout);
			erzeugeCompositeAttributGruppen(atg);
		}
		if (_darstellungsArt >= 3)
		{
			asp = new Composite(formLinks, SWT.NONE);
			asp.setLayout(fillLayout);
			erzeugeCompositeAspekte(asp);
		}
		if (_darstellungsArt >= 4)
		{
			simvar = new Composite(formLinks, SWT.NONE);
			simvar.setLayout(fillLayout);
			erzeugeCompositeSimulation(simvar);
		}

		switch (_darstellungsArt)
		{
		case 0:
			// formLinks.setWeights(new int[] { 100, 0, 0, 0 });
			break;
		case 1:
			formLinks.setWeights(new int[] { 100 });
			break;
		case 2:
			formLinks.setWeights(new int[] { 50, 50 });
			break;
		case 3:
			formLinks.setWeights(new int[] { 33, 33, 33 });
			break;
		case 4:
			formLinks.setWeights(new int[] { 30, 30, 30, 10 });
			break;

		}

	}

	/**
	 * Erzeugt das {@link Composite} auf der rechten Seite.
	 * 
	 * @param parent
	 *          Eltern {@link Composite}
	 */
	private void erzeugeCompositeRechts(Composite parent)
	{
		_compositeRechts = new Composite(parent, SWT.BORDER);
		_compositeRechts.setLayout(new FillLayout());

		erzeugeCompositeObjektListe(_compositeRechts);
	}

	/**
	 * Erzeugt das Composite zur Darstellung der Objekt-Liste
	 * 
	 * @param parent
	 *          Eltern-Composite
	 */
	private void erzeugeCompositeObjektListe(Composite parent)
	{
		Composite objektListe = new Composite(parent, SWT.NONE);

		Runnable run = new Runnable()
		{

			@Override
			public void run()
			{
				if (_tvSystemObjekte != null)
				{
					_tvSystemObjekte.setSelection(null, true);
				}
			}

		};

		_selektionObjektListe = new GtmListeSelektionsKopf(objektListe, SWT.BORDER,
				"Objekte:", run);

		_filterBoxObjekte = new FilterBox(objektListe, SWT.BORDER);
		
		_tableSystemObjekte = new Table(objektListe, SWT.BORDER | SWT.MULTI
				| SWT.FULL_SELECTION | SWT.V_SCROLL | SWT.H_SCROLL);
		_tableSystemObjekte.setHeaderVisible(true);
		_tableSystemObjekte.setLinesVisible(true);
		TableColumn tc = new TableColumn(_tableSystemObjekte, SWT.LEAD);
		tc.setText("SystemObjekte:");
		tc.setWidth(300);
		_tvSystemObjekte = new TableViewer(_tableSystemObjekte);
		_tvSystemObjekte.setContentProvider(new CPAllgemeinList());
		_tvSystemObjekte.setLabelProvider(new LabelProvider());
		_tvSystemObjekte.setSorter(new ViewerSorter());

		_listenerTvSystemObjekte = new ISelectionChangedListener()
		{

			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				fuelleAlleListenSelektionsKoepfe();
			}

		};
		_tvSystemObjekte.addSelectionChangedListener(_listenerTvSystemObjekte);
		
	
		inputSystemObjekte();

		GridLayout gridLayoutCompositeObjektListe = new GridLayout(1, false);
		objektListe.setLayout(gridLayoutCompositeObjektListe);

		GridData gdSelektion = new GridData();
		gdSelektion.grabExcessHorizontalSpace = true;
		gdSelektion.horizontalAlignment = GridData.FILL;
		_selektionObjektListe.setLayoutData(gdSelektion);

    GridData gdFilter = new GridData();
    gdFilter.grabExcessHorizontalSpace = true;
    gdFilter.horizontalAlignment = GridData.FILL;
    _filterBoxObjekte.setLayoutData(gdFilter);
		
		GridData gdListSystemObjekte = new GridData();
		gdListSystemObjekte.grabExcessHorizontalSpace = true;
		gdListSystemObjekte.horizontalAlignment = GridData.FILL;
		gdListSystemObjekte.grabExcessVerticalSpace = true;
		gdListSystemObjekte.verticalAlignment = GridData.FILL;
		// _listSystemObjekte.setLayoutData(gdListSystemObjekte);
		_tableSystemObjekte.setLayoutData(gdListSystemObjekte);
	}

	public TableViewer getTvSystemObjekte()
  {
    return _tvSystemObjekte;
  }

  public void setTvSystemObjekte(TableViewer tvSystemObjekte)
  {
    _tvSystemObjekte = tvSystemObjekte;
  }

  /**
	 * Erzeugt das {@link Composite} zur Darstellung von ObjektTypen
	 * 
	 * @param parent
	 *          Eltern {@link Composite}
	 */
	private void erzeugeCompositeObjektTypen(Composite parent)
	{
		Composite objektTypen = new Composite(parent, SWT.NONE);

		Runnable run = new Runnable()
		{

			@Override
			public void run()
			{
				if (_tvObjektTypen != null)
				{
					// removeAllListeners();
					_tvObjektTypen.setSelection(null, true);
					// inputSystemObjekte();
					// inputAttributGruppen();
					// inputAspekte();
					// addAllListeners();
				}
			}

		};

		_selektionObjektTypen = new GtmListeSelektionsKopf(objektTypen, SWT.BORDER,
				"Objekttyp:", run);

		_tableObjektTypen = new Table(objektTypen, SWT.BORDER | SWT.MULTI
				| SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
		_tableObjektTypen.setHeaderVisible(true);
		_tableObjektTypen.setLinesVisible(true);
		TableColumn tc = new TableColumn(_tableObjektTypen, SWT.LEAD);
		tc.setText("ObjektTypen:");
		tc.setWidth(300);
		_tvObjektTypen = new TableViewer(_tableObjektTypen);
		_tvObjektTypen.setContentProvider(new CPAllgemeinList());
		_tvObjektTypen.setLabelProvider(new LabelProvider());
		_tvObjektTypen.setSorter(new ViewerSorter());

		_listenerTvObjektTypen = new ISelectionChangedListener()
		{

			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection)
				{
					inputAttributGruppen();
					inputAspekte();
					filtere();
				}
			}
		};
		_tvObjektTypen.addSelectionChangedListener(_listenerTvObjektTypen);

		inputObjektTypen();

		GridLayout glObjektTypen = new GridLayout(1, false);
		objektTypen.setLayout(glObjektTypen);

		GridData gdSelektion = new GridData();
		gdSelektion.grabExcessHorizontalSpace = true;
		gdSelektion.horizontalAlignment = GridData.FILL;
		_selektionObjektTypen.setLayoutData(gdSelektion);

		GridData gdListObjektTypen = new GridData();
		gdListObjektTypen.grabExcessHorizontalSpace = true;
		gdListObjektTypen.horizontalAlignment = GridData.FILL;
		gdListObjektTypen.grabExcessVerticalSpace = true;
		gdListObjektTypen.verticalAlignment = GridData.FILL;
		_tableObjektTypen.setLayoutData(gdListObjektTypen);
	}

	/**
	 * Erzeugt das {@link Composite} zur Darstellung von Attributgruppen.
	 * 
	 * @param parent
	 *          Eltern {@link Composite}
	 */
	private void erzeugeCompositeAttributGruppen(Composite parent)
	{
		Composite attributGruppen = new Composite(parent, SWT.NONE);

		Runnable run = new Runnable()
		{

			@Override
			public void run()
			{
				if (_tvAttributGruppen != null)
				{
					_tvAttributGruppen.setSelection(null, true);
				}
			}

		};

		_selektionAttributGruppen = new GtmListeSelektionsKopf(attributGruppen,
				SWT.BORDER, "Attributgruppe:", run);

		_tableAttributGruppen = new Table(attributGruppen, SWT.BORDER
				| SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		_tableAttributGruppen.setHeaderVisible(true);
		_tableAttributGruppen.setLinesVisible(true);
		TableColumn tc = new TableColumn(_tableAttributGruppen, SWT.LEAD);
		tc.setWidth(300);
		tc.setText("Attributgruppen:");
		_tvAttributGruppen = new TableViewer(_tableAttributGruppen);
		_tvAttributGruppen.setContentProvider(new CPAllgemeinList());
		_tvAttributGruppen.setLabelProvider(new LabelProvider());
		_tvAttributGruppen.setSorter(new ViewerSorter());

		_listenerTvAttributGruppen = new ISelectionChangedListener()
		{
			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection)
				{
					inputAspekte();
					filtere();
				}

			}

		};
		_tvAttributGruppen.addSelectionChangedListener(_listenerTvAttributGruppen);

		inputAttributGruppen();

		GridLayout glAttributGruppen = new GridLayout(1, false);
		attributGruppen.setLayout(glAttributGruppen);

		GridData gdSelektion = new GridData();
		gdSelektion.grabExcessHorizontalSpace = true;
		gdSelektion.horizontalAlignment = GridData.FILL;
		_selektionAttributGruppen.setLayoutData(gdSelektion);

		GridData gdListAttributGruppen = new GridData();
		gdListAttributGruppen.grabExcessHorizontalSpace = true;
		gdListAttributGruppen.horizontalAlignment = GridData.FILL;
		gdListAttributGruppen.grabExcessVerticalSpace = true;
		gdListAttributGruppen.verticalAlignment = GridData.FILL;
		_tableAttributGruppen.setLayoutData(gdListAttributGruppen);
	}

	/**
	 * Erzeugt das {@link Composite} zur Darstellung von Aspekten.
	 * 
	 * @param parent
	 *          Eltern {@link Composite}
	 */
	private void erzeugeCompositeAspekte(Composite parent)
	{
		Composite compAspekte = new Composite(parent, SWT.NONE);

		Runnable run = new Runnable()
		{

			@Override
			public void run()
			{
				if (_tvAspekte != null)
				{
					_tvAspekte.setSelection(null, true);
				}
			}

		};

		_selektionAspekte = new GtmListeSelektionsKopf(compAspekte, SWT.BORDER,
				"Aspekt:", run);

		_tableAspekte = new Table(compAspekte, SWT.BORDER | SWT.MULTI
				| SWT.FULL_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
		_tableAspekte.setLinesVisible(true);
		_tableAspekte.setHeaderVisible(true);
		TableColumn tc = new TableColumn(_tableAspekte, SWT.LEAD);
		tc.setWidth(300);
		tc.setText("Aspekte:");
		_tvAspekte = new TableViewer(_tableAspekte);
		_tvAspekte.setContentProvider(new CPAllgemeinList());
		_tvAspekte.setLabelProvider(new LabelProvider());
		_tvAspekte.setSorter(new ViewerSorter());

		_listenerTvAspekte = new ISelectionChangedListener()
		{

			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				ISelection selection = event.getSelection();
				if (selection instanceof StructuredSelection)
				{
					StructuredSelection ss = (StructuredSelection) selection;
					if (ss.size() > 0)
					{
						filtere();
					}
				}

			}

		};
		_tvAspekte.addSelectionChangedListener(_listenerTvAspekte);

		inputAspekte();

		GridLayout glCompAspekte = new GridLayout(1, false);
		compAspekte.setLayout(glCompAspekte);

		GridData gdSelektion = new GridData();
		gdSelektion.grabExcessHorizontalSpace = true;
		gdSelektion.horizontalAlignment = GridData.FILL;
		_selektionAspekte.setLayoutData(gdSelektion);

		GridData gdListAspekte = new GridData();
		gdListAspekte.grabExcessHorizontalSpace = true;
		gdListAspekte.horizontalAlignment = GridData.FILL;
		gdListAspekte.grabExcessVerticalSpace = true;
		gdListAspekte.verticalAlignment = GridData.FILL;
		_tableAspekte.setLayoutData(gdListAspekte);
	}

	private void erzeugeCompositeSimulation(Composite parent)
	{
		Composite simulation = new Composite(parent, SWT.NONE);

		Label labelSim = new Label(simulation, SWT.NONE);
		labelSim.setText("Simulationsvariante:");

		final Spinner spinnerSim = new Spinner(simulation, SWT.NONE);
		spinnerSim.setIncrement(1);
		spinnerSim.setMinimum(0);
		spinnerSim.setMaximum(999);

		spinnerSim.addModifyListener(new ModifyListener()
		{
			@Override
			public void modifyText(ModifyEvent e)
			{
				int spinnerWert = spinnerSim.getSelection();
				_simulationsVariante = spinnerWert;
			}
		});

		this._simulationsVariante = spinnerSim.getSelection();

		/*
		 * Layout
		 */
		GridLayout glSimulation = new GridLayout(2, false);
		simulation.setLayout(glSimulation);

		// GridData gdSimulation = new GridData();
		// gdSimulation.grabExcessHorizontalSpace = true;
		// gdSimulation.horizontalAlignment = GridData.CENTER;
		// simulation.setLayoutData(gdSimulation);
	}

	protected void removeAllListeners()
	{
		if ((_listenerTvSystemObjekte != null) && (_tvSystemObjekte != null))
		{
			_tvSystemObjekte.removeSelectionChangedListener(_listenerTvSystemObjekte);
		}

		if ((_listenerTvObjektTypen != null) && (_tvObjektTypen != null))
		{
			_tvObjektTypen.removeSelectionChangedListener(_listenerTvObjektTypen);
		}

		if ((_listenerTvAttributGruppen != null) && (_tvAttributGruppen != null))
		{
			_tvAttributGruppen
					.removeSelectionChangedListener(_listenerTvAttributGruppen);
		}

		if ((_listenerTvAspekte != null) && (_tvAspekte != null))
		{
			_tvAspekte.removeSelectionChangedListener(_listenerTvAspekte);
		}
	}

	protected void addAllListeners()
	{
		if ((_listenerTvSystemObjekte != null) && (_tvSystemObjekte != null))
		{
			_tvSystemObjekte.addSelectionChangedListener(_listenerTvSystemObjekte);
		}

		if ((_listenerTvObjektTypen != null) && (_tvObjektTypen != null))
		{
			_tvObjektTypen.addSelectionChangedListener(_listenerTvObjektTypen);
		}

		if ((_listenerTvAttributGruppen != null) && (_tvAttributGruppen != null))
		{
			_tvAttributGruppen
					.addSelectionChangedListener(_listenerTvAttributGruppen);
		}

		if ((_listenerTvAspekte != null) && (_tvAspekte != null))
		{
			_tvAspekte.addSelectionChangedListener(_listenerTvAspekte);
		}
	}

	/**
	 * Stellt alles {@link SystemObject}s aus der {@link GtmDatenBasis} dar.
	 */
	private void inputSystemObjekte()
	{
		if (_darstellungsArt >= 0)
		{
			if (_gtmDatenBasis != null)
			{
				if (_tvSystemObjekte != null)
				{
					_tableSystemObjekte.setVisible(false);
					_tvSystemObjekte.setInput(null);
					_tvSystemObjekte.setInput(_gtmDatenBasis.getSystemObjects());
					_tableSystemObjekte.setVisible(true);
				}
			}
		}
	}

	/**
	 * Stellt bergebene {@link SystemObject}s dar.
	 * 
	 * @param systemObjects
	 *          Darzustellende {@link SystemObject}s
	 */
	private void inputSystemObjekte(
			java.util.List<GtmListenElementObjekt> systemObjects)
	{		
		if (_darstellungsArt >= 0)
		{
			if (_tvSystemObjekte != null)
			{
				_tableSystemObjekte.setVisible(false);
				_tvSystemObjekte.setInput(null);
				_tvSystemObjekte.setInput(systemObjects);
				_tableSystemObjekte.setVisible(true);
			}
		}
	}

	/**
	 * Stellt die in der {@link GtmDatenBasis} hinterlegten SytemObjectTypen dar.
	 */
	private void inputObjektTypen()
	{
		if (_darstellungsArt >= 1)
		{
			if (_gtmDatenBasis != null)
			{
				if (_tvObjektTypen != null)
				{
					_tableObjektTypen.setVisible(false);
					_tvObjektTypen.setInput(null);
					_tvObjektTypen.setInput(_gtmDatenBasis.getSystemObjectTypes());
					_tableObjektTypen.setVisible(true);
				}
			}
		}
	}

	/**
	 * Stellt an Hand der selektierten Objekttypen die zugehrigen Attributgruppen
	 * dar. Sind keine Objekttypen selektiert, werden alle Attributgruppen
	 * dargestellt.
	 */
	private void inputAttributGruppen()
	{
		if (_darstellungsArt >= 2)
		{
			if (_gtmDatenBasis != null)
			{
				if (_tvAttributGruppen != null)
				{

					LinkedList<SystemObjectType> sot = getSelektierteObjektTypenOderAlle();

					java.util.List<AttributeGroup> atgs = _gtmDatenBasis.getAtg(sot);

					if (sot.size() > 0)
					{
						_tableAttributGruppen.setVisible(false);
						_tvAttributGruppen.setInput(null);
						_tvAttributGruppen.setInput(atgs);
						_tableAttributGruppen.setVisible(true);
					}

					// if(_listAttributGruppen != null)
					// {
					// _listAttributGruppen.selectAll();
					// }

				}
			}
		}
	}

	/**
	 * Stellt an Hand der selektierten Attributgruppen die zugehrigen Aspekte
	 * dar. Sind keine Attributgruppen selektiert, werden alle Aspekte
	 * dargestellt.
	 */
	private void inputAspekte()
	{
		if (_darstellungsArt >= 3)
		{
			if (_gtmDatenBasis != null)
			{
				if (_tvAspekte != null)
				{
					java.util.List<Aspect> asps = null;
					/*
					 * Wenn Attributgruppen selektiert sind
					 */
					LinkedList<AttributeGroup> selektierteAttributGruppen = getSelektierteAttributGruppen();
					if (selektierteAttributGruppen.size() > 0)
					{
						asps = _gtmDatenBasis.getAsp(selektierteAttributGruppen);
					}
					else
					{
						asps = _gtmDatenBasis.getAsp(_gtmDatenBasis
								.getAtg(getSelektierteObjektTypenOderAlle()));
					}
					if (asps.size() > 0)
					{
						_tableAspekte.setVisible(false);
						_tvAspekte.setInput(null);
						_tvAspekte.setInput(asps);
						_tableAspekte.setVisible(true);
					}
				}
			}
		}
	}

	/**
	 * Fhrt die Filterung der SystemObjekte an Hand der in den Listen gewhlten
	 * Objekten durch.
	 */
	private void filtere()
	{
//		long start = new Date().getTime();

		LinkedList<SystemObjectType> selObjTyp = null;
		LinkedList<AttributeGroup> selAtg = null;
		LinkedList<Aspect> selAsp = null;

		LinkedList<SystemObjectType> selektierteObjektTypen = getSelektierteObjektTypenOderAlle();
		if (selektierteObjektTypen.size() > 0)
		{
			selObjTyp = selektierteObjektTypen;
		}

		LinkedList<AttributeGroup> selektierteAttributGruppen = getSelektierteAttributGruppen();
		if (selektierteAttributGruppen.size() > 0)
		{
			selAtg = selektierteAttributGruppen;
		}

		LinkedList<Aspect> selektierteAspekte = getSelektierteAspekte();
		if (selektierteAspekte.size() > 0)
		{
			selAsp = selektierteAspekte;
		}

		if (_gtmDatenBasis != null)
		{
		  
		  List<de.bsvrz.buv.plugin.tkabasis.datenbereich.datenbasis.GtmListenElementObjekt> systemObjects = _gtmDatenBasis.getSystemObjects();
		  
			LinkedList<GtmListenElementObjekt> leos = _gtmDatenBasis.filtere(systemObjects, selObjTyp, selAtg, selAsp);

			inputSystemObjekte(leos);
		}

		fuelleAlleListenSelektionsKoepfe();

	}

	/**
	 * Fllt alle vorhandenen {@link GtmListeSelektionsKpfe}
	 */
	private void fuelleAlleListenSelektionsKoepfe()
	{
		if (_selektionObjektListe != null)
		{
			// flleListenSelektionsKopf(_selektionObjektListe, _listSystemObjekte);
			flleListenSelektionsKopf(_selektionObjektListe, _tableSystemObjekte);
		}

		if (_selektionObjektTypen != null)
		{
			flleListenSelektionsKopf(_selektionObjektTypen, _tableObjektTypen);
		}

		if (_selektionAttributGruppen != null)
		{
			flleListenSelektionsKopf(_selektionAttributGruppen,
					_tableAttributGruppen);
		}

		if (_selektionAspekte != null)
		{
			flleListenSelektionsKopf(_selektionAspekte, _tableAspekte);
		}
	}

	private void flleListenSelektionsKopf(GtmListeSelektionsKopf header,
			Table liste)
	{
		header.setAnzahl(liste.getItemCount(), liste.getSelectionCount());
	}

	/**
	 * Liefert eine Liste der selektierten {@link SystemObjectType}s. Ist kein
	 * {@link SystemObjectType} selektiert, werden alle {@link SystemObjectType}
	 * der {@link GtmDatenBasis} zurckgeliefert.
	 * 
	 * @return Liste der selektierten {@link SystemObjectType}s
	 */
	private LinkedList<SystemObjectType> getSelektierteObjektTypenOderAlle()
	{
		LinkedList<SystemObjectType> sot = new LinkedList<SystemObjectType>();
		if (_tvObjektTypen != null)
		{
			ISelection selection = _tvObjektTypen.getSelection();
			if (!selection.isEmpty())
			{
				StructuredSelection ss = (StructuredSelection) selection;
//				@SuppressWarnings("unchecked")
//				Iterator it = ss.iterator();
//				while (it.hasNext())
//				{
//					sot.add((SystemObjectType) it.next());
//				}
				@SuppressWarnings("unchecked")
				Iterator it = ss.iterator();
				while (it.hasNext())
				{
				  bestimmeUnterTypen((SystemObjectType) it.next(), sot);
				}
			}
			else if (_gtmDatenBasis != null)
			{
//				Iterator<SystemObjectType> it = _gtmDatenBasis.getSystemObjectTypes()
//						.iterator();
//				while (it.hasNext())
//				{
//					sot.add(it.next());
//				}
  				Iterator<SystemObjectType> it = _gtmDatenBasis.getSystemObjectTypes()
  						.iterator();
  				while (it.hasNext())
  				{
  				  bestimmeUnterTypen(it.next(), sot);
  				}
			}
		}
		else if (_gtmDatenBasis != null)
		{
//			Iterator<SystemObjectType> it = _gtmDatenBasis.getSystemObjectTypes()
//					.iterator();
//			while (it.hasNext())
//			{
//				sot.add(it.next());
//			}
			Iterator<SystemObjectType> it = _gtmDatenBasis.getSystemObjectTypes()
					.iterator();
			while (it.hasNext())
			{
			  bestimmeUnterTypen(it.next(), sot);
			}
		}

		return sot;
	}
	
	
	 private void bestimmeUnterTypen(SystemObjectType typ, List<SystemObjectType> list)
	 {
	   list.add(typ);
	   List<SystemObjectType> subTypes = typ.getSubTypes();
	   
	   for (SystemObjectType unterTyp:  subTypes)
     {
       bestimmeUnterTypen(unterTyp, list);
     }
	   
//	    List<SystemObjectType> subTypes = typ.getSubTypes();
//
//	    if (subTypes.size() > 0){
//	      for (SystemObjectType unterTyp:  subTypes)
//	      {
////	        System.out.println("--> : " + unterTyp.getPid());	      	              
//	        list.add(unterTyp);
//	        bestimmeUnterTypen(unterTyp, list);
//	      }	      
//	    }else{
////	      System.out.println("--> : " + typ.getPid());       
//	      list.add(typ);
//	    }
	   
	  }

	/**
	 * Liefert eine Liste der selektierten {@link AttributeGroup}s. Ist keine
	 * Attributgruppe selektiert, werden alle in der Liste dargestellten
	 * Attributgruppen zurckgeliefert.
	 * 
	 * @return Liste der selektierten {@link AttributeGroup}s
	 */
	private LinkedList<AttributeGroup> getSelektierteAttributGruppen()
	{
		LinkedList<AttributeGroup> atgs = new LinkedList<AttributeGroup>();

		if (_tvAttributGruppen != null)
		{
			ISelection selection = _tvAttributGruppen.getSelection();

			if (!selection.isEmpty())
			{
				StructuredSelection ss = (StructuredSelection) selection;
				@SuppressWarnings("unchecked")
				Iterator it = ss.iterator();
				while (it.hasNext())
				{
					atgs.add((AttributeGroup) it.next());
				}
			}
		}

		return atgs;
	}

	/**
	 * Liefert eine Liste der selektierten {@link Aspect}s zurck. Wurde kein
	 * {@link Aspect} selektiert, werden alle in der Liste dargestellten
	 * {@link Aspect}s zurckgeliefert.
	 * 
	 * @return Liste der selektierten {@link Aspect}s
	 */
	private LinkedList<Aspect> getSelektierteAspekte()
	{
		LinkedList<Aspect> asps = new LinkedList<Aspect>();

		if (_tvAspekte != null)
		{
			ISelection selection = _tvAspekte.getSelection();
			if (!selection.isEmpty())
			{
				StructuredSelection ss = (StructuredSelection) selection;
				@SuppressWarnings("unchecked")
				Iterator it = ss.iterator();
				while (it.hasNext())
				{
					asps.add((Aspect) it.next());
				}
			}
		}
		else if (_gtmDatenBasis != null)
		{
			if (_tableAspekte != null)
			{
				for (int i = 0; i < _tableAspekte.getItemCount(); i++)
				{
					asps.add((Aspect) _tvAspekte.getElementAt(i));
				}
			}
		}

		return asps;
	}

	/**
	 * Liefert eine Liste der selektierten SystemObjekte zurck.
	 * 
	 * @return
	 */
	public LinkedList<SystemObject> getSelektierteSystemObjekte()
	{
		LinkedList<SystemObject> re = new LinkedList<SystemObject>();

		if (_tvSystemObjekte != null)
		{
			ISelection selection = _tvSystemObjekte.getSelection();
			if (!selection.isEmpty())
			{
				StructuredSelection ss = (StructuredSelection) selection;
				@SuppressWarnings("unchecked")
				Iterator it = ss.iterator();
				while (it.hasNext())
				{
					re.add(((GtmListenElementObjekt) it.next()).getSystemObjekt());
				}
			}
		}

		return re;
	}

	/**
	 * Liefert die mit Hilfe des Spinners gewhlte Simulationsvariante zurck.
	 * 
	 * @return Simulationsvariante oder -1, falls es keinen Whler der
	 *         Simulationsvariante in der Darstellung gab.
	 */
	public int getSimulationsVariante()
	{
		return _simulationsVariante;
	}

	/**
	 * ContentProvider fr die Typen des Datenverteilers
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class CPAllgemeinList implements IStructuredContentProvider
	{

		@SuppressWarnings("unchecked")
		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;
			if (inputElement instanceof java.util.List)
			{
				java.util.List elemente = (java.util.List) inputElement;
				re = elemente.toArray();
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	private class LabelProvider implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
		  if (element instanceof SystemObjectType)
		  {
		    String result = ((SystemObjectType)element).getPid();
		    if (result != null && !result.isEmpty())
		    {
		      String name = ((SystemObjectType)element).getName();
		      if (name != null && !name.isEmpty())
		      {
		        result +=" (";
		        result += name;
		        result +=")";
		      }
		      
		      return result;
		    }
		  }
			return element.toString();
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	/**
	 * Klasse zur Darstellung der ListenKpfe
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class GtmListeSelektionsKopf extends Composite
	{
		private String _titel = null;
		private int _anzMax = -1;
		private int _anzAkt = -1;
		private Runnable _buttonRun = null;
		private Label _labelTitle;
		private Text _textAnzahl;

		public GtmListeSelektionsKopf(Composite parent, int style, String titel,
				Runnable buttonRun)
		{
			super(parent, style);

			this._titel = titel;
			this._buttonRun = buttonRun;

			initialize();
		}

		private void initialize()
		{
			GridLayout glHeader = new GridLayout(3, false);
			setLayout(glHeader);

			_labelTitle = new Label(this, SWT.NONE);
			_labelTitle.setText(_titel);

			_textAnzahl = new Text(this, SWT.BORDER);
			_textAnzahl.setEditable(false);
			setAnzahl(_anzMax, _anzAkt);

			Button buttonReset = new Button(this, SWT.PUSH);
			buttonReset.setText("X");
			buttonReset.addSelectionListener(new SelectionAdapter()
			{
				public void widgetSelected(org.eclipse.swt.events.SelectionEvent e)
				{
					if (_buttonRun != null)
					{
						_buttonRun.run();
					}
				}
			});

			GridData gdTitel = new GridData();
			gdTitel.grabExcessHorizontalSpace = true;
			gdTitel.horizontalAlignment = GridData.FILL;
			_labelTitle.setLayoutData(gdTitel);
		}

		public void setAnzahl(int anzahlMax, int anzahlAktuell)
		{
			this._anzMax = anzahlMax;
			this._anzAkt = anzahlAktuell;

			_textAnzahl.setText(_anzAkt + " / " + anzahlMax);
			this.layout(true);
		}
	}
		
	private class FilterBox extends Composite
	{
	  private Label _labelFilter;
	  private Text _textFilter;
	  
	  public FilterBox(Composite parent, int style)
    {
      super(parent, style);
      initialize();
    } 
	  
	  private void initialize()
	  {
	    GridLayout gl = new GridLayout(2, false);
      setLayout(gl);

      _labelFilter = new Label(this, SWT.NONE);
      _labelFilter.setText("Filter:");

      _textFilter = new Text(this, SWT.BORDER);
      
      _textFilter.addModifyListener(new ModifyListener() {
        public void modifyText(ModifyEvent e)
        {
          String regAusdruck = _textFilter.getText();
          aktionFilterObjekte( regAusdruck );
        }
      });
      
      GridData gdText = new GridData();
      gdText.grabExcessHorizontalSpace = true;
      gdText.horizontalAlignment = GridData.FILL;
      _textFilter.setLayoutData(gdText);
	  }
	}
	
	/**
	 * Filtern nach regulren Ausdrcken
	 * 
	 * @author Sans
	 *
	 */
	private class RegExViewerFilterObjekte extends ViewerFilter
	{
	  private Pattern _patFilter;
	  
	  RegExViewerFilterObjekte(Pattern patFilter)
	  {
	    _patFilter = patFilter;
	  }
	  
    public Object[] filter(Viewer viewer, Object parent, Object[] elements) 
    {
      Object[] result = super.filter(viewer, parent, elements);
      int size = result.length;
      _tvSystemObjekte.setSelection(null, true);
      _selektionObjektListe.setAnzahl(size, 0);
      return result;
    }

	  public boolean select(Viewer viewer, Object parentElement, Object element)
    {
      if (element instanceof GtmListenElementObjekt)
      {
        GtmListenElementObjekt listenElement = (GtmListenElementObjekt)element;
        
        // String als Matcher fr den Vergleich setzen und Ergebnis abfragen
        String name = listenElement.getName();
        Matcher mat = _patFilter.matcher(name);
        if (/*name.contains(_patFilter.pattern()) || */mat.find())
        {  
          return true;
        }  
      }
      
      return false;
    }
	}
	
	/**
   * Setzt den Filter fr die Tabelle der Systemobjekte auf den angegebenen regulren Ausdruck
   * - wobei aus Performance-Grnden die contains-Methode mit verwendet wird 
   * 
   * @param regAusdruck   regulrer Ausdruck
   */
  private void aktionFilterObjekte(String regAusdruck)
  {
    Pattern patFilter = null;
    try
    {
      patFilter = Pattern.compile(regAusdruck);
    }
    catch (PatternSyntaxException pse)
    {
      // falscher regulrer Ausdruck - ignorieren
      return;
    }
    
    _tvSystemObjekte.getTable().setRedraw(false);
    Cursor defaultCursor = getShell().getCursor();
    Cursor busyCursor = new Cursor(getDisplay(), SWT.CURSOR_WAIT);
    getShell().setCursor(busyCursor);
    
    try
    {
      ArrayList<ViewerFilter> filterListe = new ArrayList<ViewerFilter>();
      
      for (ViewerFilter filter: _tvSystemObjekte.getFilters())
      {
        if (!(filter instanceof RegExViewerFilterObjekte))
        {
          filterListe.add(filter);
        }
      }
     
      if (!regAusdruck.isEmpty())
      {
        // ViewFilter setzen
        ViewerFilter textFilter = new RegExViewerFilterObjekte(patFilter);
        filterListe.add(textFilter);
        _tvSystemObjekte.setFilters(filterListe.toArray(new ViewerFilter[filterListe.size()]));
      }
      else
      {
        // Kein Filter (mehr) gesetzt
        _tvSystemObjekte.setFilters(filterListe.toArray(new ViewerFilter[filterListe.size()]));
        _tvSystemObjekte.setSelection(null, true);
        _selektionObjektListe.setAnzahl(_tableSystemObjekte.getItemCount(), 0);
      }
    }
    finally
    {  
      getShell().setCursor(defaultCursor);
      busyCursor.dispose();
      _tvSystemObjekte.getTable().setRedraw(true);
    }  
  } 
   
}
