/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.Collections;
import java.util.List;

/**
 * Es werden Informationen ber ein Attribut modelliert, das von einem Hierarchieobjekttypen 
 * spezifiziert ist
 * 
 * @author Sans
 *
 */
public class AttributInfo implements IAttributInfo
{
  /**
   * Erzeugt eine neue Instanz vom Typ AttributInfo
   * 
   * @param name Name des Attributs
   * @param isPflichtFeld ist dieses Attribut Pflichtfeld?
   * @param isEindeutig sind die Werte dieses Attributs eindeutig?
   * @param isAutoGeneriert werden die Werte dieses Attributs automatisch generiert?
   * @param isAenderbarNeu sind die Werte dieses Attributs nderbar im Zustand 'Neu'?
   * @param isAenderbarBearb sind die Werte dieses Attributs nderbar im Zustand 'Bearbeiten'?
   * @param isVorbelegt sind die Werte dieses Attributs vorbelegt)
   */
  public AttributInfo(String name, boolean isPflichtFeld, boolean isEindeutig, 
      boolean isAutoGeneriert, boolean isAenderbarNeu, boolean isAenderbarBearb, 
      boolean isVorbelegt)
  {
    _name = name;
    _isPflichtFeld = isPflichtFeld;
    _isEindeutig = isEindeutig; 
    _isAutoGeneriert = isAutoGeneriert; 
    _isAenderbarNeu = isAenderbarNeu; 
    _isAenderbarBearb = isAenderbarBearb;
  }
  
  /**
   * Name des Attributs
   */
  private String _name;
  
  /**
   * Ist dieses Attribut Pflichtfeld?
   */
  private boolean _isPflichtFeld;
  
  /**
   * Sind die Werte dieses Attributs eindeutig?
   */
  private boolean _isEindeutig;
  
  /**
   * Werden die Werte dieses Attributs automatisch generiert?
   */
  private boolean _isAutoGeneriert;
  
  /**
   * Sind die Werte dieses Attributs nderbar im Zustand 'Neu'?
   */
  private boolean _isAenderbarNeu;
  
  /**
   * Sind die Werte dieses Attributs nderbar im Zustand 'Bearbeiten'?
   */
  private boolean _isAenderbarBearb;
  
  /**
   * Sind die Werte dieses Attributs vorbelegt)
   */
  private boolean _isVorbelegt;
  
  public String getName()
  {
    return _name;
  }
 
  public boolean isPflichtFeld()
  {
    return _isPflichtFeld;
  }
 
  public boolean isEindeutig()
  {
    return _isEindeutig;
  }
 
  public boolean isAutoGeneriert()
  {
    return _isAutoGeneriert;
  }
  
  public boolean isAenderbarNeu()
  {
    return _isAenderbarNeu;
  }
  
  public boolean isAenderbarBearb()
  {
    return _isAenderbarBearb;
  }
  
  public boolean isVorbelegt()
  {
    return _isVorbelegt;
  }
  
  public List<IAttributInfo> getInnereAttribute()
  {
    return Collections.emptyList();
  }
  
  public boolean hasInnereAttribute()
  {
    return false;
  }

  public boolean isArray()
  {
    return false;
  }
}
