/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.hierarchie;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Vector;

/**
 * Es werden Informationen ber eine Attributliste modelliert, die von einem Hierarchieobjekttypen 
 * spezifiziert ist
 * 
 * @author Sans
 *
 */
public class AttributListeInfo implements IAttributInfo
{
  
  /**
   * Erzeugt eine neue Instanz vom Typ AttributListeInfo<br>
     * 
   * @param name Name der Attributliste
   * @param innereAttribute Liste der Beschreibungen der inneren Attribute
   */
  public AttributListeInfo(String name, IAttributInfo[] innereAttribute)
  {
    _name = name;
    if (innereAttribute != null)
    {
      _innereAttribute = new Vector<IAttributInfo>();
      _innereAttribute.addAll(Arrays.asList(innereAttribute)); 
    }
  }
  
  /**
   * Name des Attributs
   */
  private String _name;
    
  /**
   * Liste der inneren Attribute 
   */
  private List<IAttributInfo> _innereAttribute = Collections.emptyList();
    
  
  public List<IAttributInfo> getInnereAttribute()
  {
    return  Collections.unmodifiableList(_innereAttribute);
  }
  
  public boolean hasInnereAttribute()
  {
    return !_innereAttribute.isEmpty();
  }
  
  public String getName()
  {
    return _name;
  }
 
  public boolean isPflichtFeld()
  {
    return false;
  }
 
  public boolean isEindeutig()
  {
    return false;
  }
 
  public boolean isAutoGeneriert()
  {
    return false;
  }
  
  public boolean isAenderbarNeu()
  {
    return false;
  }
  
  public boolean isAenderbarBearb()
  {
    return false;
  }
  
  public boolean isVorbelegt()
  {
    return false;
  }
  
  public boolean isArray()
  {
    return false;
  }
  
}  
  