/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.navigator;

import java.util.List;

import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IColorProvider;
import org.eclipse.jface.viewers.IFontProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.HONavigatorElement;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.INavigatorElement;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.ImageFactory;
import de.bsvrz.buv.plugin.tkabasis.navigator.navigatorModell.KBNavigatorElement;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Erzeugt die Label der Elemente des TreeViewers, bestehend aus Grafik und Text.
 * 
 * @author Krause
 * 
 */
public class NavigatorLabelProvider extends CellLabelProvider implements ILabelProvider, IColorProvider, IFontProvider, IStyledLabelProvider
{
  FontRegistry registry = new FontRegistry();
  
  /**
   * Liefert das Bild zum Label
   */
  public Image getImage(Object obj)
  {
    if (obj instanceof INavigatorElement)
    {
      String imageFilePath = ((INavigatorElement)obj).getImagePath();

      ImageFactory images = ImageFactory.getInstanz();
      
      Image image = images.getImage(imageFilePath);
      if (image != null)
        return image;
    }
    return PlatformUI.getWorkbench().getSharedImages().getImage(ISharedImages.IMG_OBJS_ERROR_TSK);

  }

  // ist in getStyledText des Decorators enthalten
  /**
   * Liefert den Text zum Label
   */
  public String getText(Object obj)
  {
    if (obj instanceof INavigatorElement)
    {
      return ((INavigatorElement)obj).getName();
    }
    return "";

  }

  @Override
  public Color getBackground(Object arg0)
  {
    return null;
  }

  @Override
  public Color getForeground(Object arg0)
  {
    
    if (arg0 instanceof KBNavigatorElement)
    {
      KBNavigatorElement kbne = (KBNavigatorElement)arg0;
      if (!kbne.getKonfigurationsBereich().isValid())
        return Display.getCurrent().getSystemColor(SWT.COLOR_BLUE);

    }
    else if (arg0 instanceof HONavigatorElement)
    {
      HONavigatorElement hone = (HONavigatorElement)arg0;
      HierarchieObjekt ho = hone.getHierarchieObjekt();
      List<SystemObject> sos = ho.getSystemObjekte();

      boolean gueltig = true;
      
      for (SystemObject so : sos)
      {
        if (!so.isValid())
          gueltig = false;
      }

      if (!gueltig)
        return Display.getCurrent().getSystemColor(SWT.COLOR_BLUE);
    }

    return Display.getCurrent().getSystemColor(SWT.COLOR_BLACK);
  }

  @Override
  public Font getFont(Object obj)
  {
    
    if (obj instanceof KBNavigatorElement)
    {
      KBNavigatorElement kbne = (KBNavigatorElement)obj;
      if (!kbne.getKonfigurationsBereich().isValid())
        return registry.getItalic(Display.getCurrent().getSystemFont().getFontData()[0].getName());
      
    }
    else if (obj instanceof HONavigatorElement)
    {
      HONavigatorElement hone = (HONavigatorElement)obj;
      HierarchieObjekt ho = hone.getHierarchieObjekt();
      List<SystemObject> sos = ho.getSystemObjekte();

      boolean gueltig = true;

      for (SystemObject so : sos)
      {
        if (!so.isValid())
          gueltig = false;
      }

      if (!gueltig)
        return registry.getItalic(Display.getCurrent().getSystemFont().getFontData()[0].getName());

    }
    return null;
  }

  // falls eigene Tooltips angezeigt werden sollen, mit extends CellLabelProvider
  public String getToolTipText(Object obj)
  {
    if (obj instanceof INavigatorElement)
    {
      return ((INavigatorElement)obj).getToolTip();
    }
    return null;
  }

  public Point getToolTipShift(Object obj)
  {     
    return new Point(0,10);
  }
  
//  public int getToolTipDisplayDelayTime(Object object)
//  {
//    return 2000;
//  }
  
  public int getToolTipTimeDisplayed(Object obj)
  {
    return 10000;
  }

  @Override
  public void update(ViewerCell cell)
  {
    cell.setImage(getImage(cell.getElement()));
    cell.setText(getText(cell.getElement()));
    cell.setBackground(getBackground(cell.getElement())); 
    cell.setForeground(getForeground(cell.getElement()));
    cell.setFont(getFont(cell.getElement()));
//    cell.setStyleRanges(null);
  }

  @Override
  public StyledString getStyledText(Object obj)
  {
    String text = "";
    if (obj instanceof INavigatorElement)
    {
      text = ((INavigatorElement)obj).getName();
    }
    StyledString styledString = new StyledString(text);
    
    return styledString;
  }

}