/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabasis.zustaende;

import de.bsvrz.buv.plugin.tkabasis.aktionen.AktionFactory.TKABasisAktion;

/**
 * Die Klasse implementiert die Zustandsfabrik fr die TKABasis
 * 
 * @author Krause
 *
 */
public class ZustandsFactory
{
//  /**
//   * Singleton-Instanz
//   */
//  private static ZustandsFactory _instanz = null;
  
  /**
   * Mgliche Bearbeitungsmodi fr dieses Composite
   * 
   * @author Krause
   *
   */
  public static enum Bearbeitungsmodus{
  DARSTELLUNG,
  BEARBEITUNG,
  NEUANLAGE
  };
   
  /**
   * Anzahl der erzeugten Zustnde
   */
  private static final int _anzahlZustaende = 3;
  
  /**
   * Die Zustandsmaschine
   */
  private EndlicherAutomat _endlicherAutomat;

//  /**
//   * Liefert die Singletoninstanz.
//   * 
//   * @return Instanz dieser Klasse.
//   */
//  public static ZustandsFactory getInstanz()
//  {
//    if (_instanz == null)
//    {
//      _instanz = new ZustandsFactory();
//    }
//    return _instanz;
//  }
  
  public ZustandsFactory()
  {
    Bearbeitungsmodus initModus = Bearbeitungsmodus.DARSTELLUNG;
    Zustand initZustand = null;
   
    Zustand darstellung = new Zustand(Bearbeitungsmodus.DARSTELLUNG.toString());
    Zustand bearbeitung = new Zustand(Bearbeitungsmodus.BEARBEITUNG.toString());
    Zustand neuanlage = new Zustand(Bearbeitungsmodus.NEUANLAGE.toString());  
    
    Zustand[] zustaende = new Zustand[_anzahlZustaende];
    zustaende[0] = darstellung;
    zustaende[1] = bearbeitung;
    zustaende[2] = neuanlage;
    
    switch (initModus)
    {
      case DARSTELLUNG:
        initZustand = darstellung;
        break;
      case BEARBEITUNG:
        initZustand = bearbeitung;
        break;
      case NEUANLAGE:
        initZustand = neuanlage;
        break;
      default:
        break;
    }
 
    _endlicherAutomat = new EndlicherAutomat(_anzahlZustaende, initZustand);
    _endlicherAutomat.addZustaende(zustaende);
    
    //bergnge definieren
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.DARSTELLEN.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.BEARBEITEN.getAktion().getAktionsNr(), bearbeitung); 
    
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.NEU.getAktion().getAktionsNr(), neuanlage);
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.NEUUO.getAktion().getAktionsNr(), neuanlage);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.NEU.getAktion().getAktionsNr(), neuanlage);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.NEUUO.getAktion().getAktionsNr(), neuanlage);
//    _endlicherAutomat.addUebergang(neuanlage,   TKABasisAktion.NEU.getAktion().getAktionsNr(), neuanlage);
//    _endlicherAutomat.addUebergang(neuanlage,   TKABasisAktion.NEUUO.getAktion().getAktionsNr(), neuanlage);
    
    _endlicherAutomat.addUebergang(neuanlage,   TKABasisAktion.UEBERNEHMEN.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.UEBERNEHMEN.getAktion().getAktionsNr(), bearbeitung);

//    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.ABBRUCH.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.ABBRUCH.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(neuanlage,   TKABasisAktion.ABBRUCH.getAktion().getAktionsNr(), darstellung);
    
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.SELEKTIEREN.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.SELEKTIEREN.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(neuanlage,   TKABasisAktion.SELEKTIEREN.getAktion().getAktionsNr(), darstellung);

    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.LOESCHEN.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.LOESCHEN.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(neuanlage,   TKABasisAktion.LOESCHEN.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.VERSCHIEBEN.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.VERSCHIEBEN.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.KOPIEREN.getAktion().getAktionsNr(), neuanlage);
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.KOPIERENHO.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.KOPIERENHOMUO.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.KOPIEREN.getAktion().getAktionsNr(), neuanlage);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.KOPIERENHO.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.KOPIERENHOMUO.getAktion().getAktionsNr(), darstellung);

    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.PRUEFEN.getAktion().getAktionsNr(), bearbeitung);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.PRUEFEN.getAktion().getAktionsNr(), bearbeitung);
    _endlicherAutomat.addUebergang(neuanlage,   TKABasisAktion.PRUEFEN.getAktion().getAktionsNr(), neuanlage);
    
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.EINFUEGEN.getAktion().getAktionsNr(), neuanlage);
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.EINFUEGENVER.getAktion().getAktionsNr(), bearbeitung);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.EINFUEGEN.getAktion().getAktionsNr(), neuanlage);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.EINFUEGENVER.getAktion().getAktionsNr(), bearbeitung);
    
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.NACHFOLGER.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.VORGAENGER.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.NACHFOLGEREBENE.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.VORGAENGEREBENE.getAktion().getAktionsNr(), darstellung);

    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.KONFIGAENDERUNGINFO.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.SOINFO.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.SOINFO.getAktion().getAktionsNr(), bearbeitung);
    _endlicherAutomat.addUebergang(neuanlage, TKABasisAktion.SOINFO.getAktion().getAktionsNr(), neuanlage);    
    _endlicherAutomat.addUebergang(darstellung, TKABasisAktion.DEFAULT.getAktion().getAktionsNr(), darstellung);
    _endlicherAutomat.addUebergang(bearbeitung, TKABasisAktion.DEFAULT.getAktion().getAktionsNr(), bearbeitung);
    _endlicherAutomat.addUebergang(neuanlage,   TKABasisAktion.DEFAULT.getAktion().getAktionsNr(), neuanlage);
    
  }
  
  /**
   * Liefert die Zustandsmaschine
   * 
   * @return  Zustandsmaschine
   */
  public EndlicherAutomat getZustandsmaschine()
  {
    return _endlicherAutomat;
  }
  
}
