/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkabast;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.EingabeVorbelegung;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeBaseValueDataFactory;
import de.bsvrz.dav.daf.communication.dataRepresentation.AttributeHelper;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

public class BildungsRegelnBWBASt extends BildungsRegelnAllgemeinBASt
{
  protected Map<String, EingabeVorbelegung> _vorbelegungen = new HashMap<String, EingabeVorbelegung>();
   
  protected void initVorbelegungen()
  {
    _vorbelegungen.put("Bezeichnung", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("Strae", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("Knotennummer", new EingabeVorbelegung(null, false));
    _vorbelegungen.put("Fahrtrichtung",  new EingabeVorbelegung(new String[] {"N", "O", "S", "W"}));
  }
  
  @Override
  protected void initPraefixe()
  {
    _praefixe.put("typ.auswerteQuerschnitt", "awq.DZ");
    _praefixe.put("typ.langZeitZhlstelle", "dz");
  }
  
  @Override
  protected void initFelder()
  {
    _felder.put(HOT_BASt.AuswerteQuerschnitt, new String[] { "Bezeichnung", "Strae", "Knotennummer", "Fahrtrichtung"});
    _felder.put(HOT_BASt.LangZeitZaehlStelle, new String[] { "Bezeichnung", "Strae", "Knotennummer" });
  }
  
  
  public BildungsRegelnBWBASt()
  {
    super();

  	initVorbelegungen();
  }
  
  @Override
  public EingabeVorbelegung getEingabeVorbelegung(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld, 
      HierarchieObjekt verbundenesHO)
  {
    if (hotNeuesObjekt.equals(HOT_BASt.AuswerteQuerschnitt) || hotNeuesObjekt.equals(HOT_BASt.LangZeitZaehlStelle))
    {
      if (Arrays.asList(_felder.get(hotNeuesObjekt)).contains(eingabeFeld))
      {
        return _vorbelegungen.get(eingabeFeld);
      }
    }
    
    return super.getEingabeVorbelegung(hotNeuesObjekt, eingabeFeld, verbundenesHO);
  }
  
  @Override
  public String getPidRegelText(IHierarchieObjektTyp hot)
  {
  	String result = BildungsRegelnTools.LEERER_WERT;
  	
  	if (hot.equals(HOT_BASt.AuswerteQuerschnitt))
  	{
  		result = getPraefix(hot.getTypen().get(0))
  				+ ".Strae"
  				+ ".Knotennummer"
  				+ ".Fahrtrichtung";
  	}
  	else if (hot.equals(HOT_BASt.LangZeitZaehlStelle))
  	{
  		result = getPraefix(hot.getTypen().get(0))
			+ ".Strae"
			+ ".Knotennummer";
		}
  	
  	return result;
  }
  
  @Override
  public String getNameRegelText(IHierarchieObjektTyp hot)
  {
  	String result = "";
  	
  	if (hot.equals(HOT_BASt.AuswerteQuerschnitt))
  	{
  		result = "AWQ"
  				+ " Bezeichnung"
  				+ " Fahrtrichtung";
  	}
  	else if (hot.equals(HOT_BASt.LangZeitZaehlStelle))
  	{
  		result = "DZ"
  			+ " Bezeichnung";
		}
  	
  	return result;
  }
  
  @Override
  public String[] getPids(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO)
  {
    String text = "";
    if (hotNeuesObjekt.equals(HOT_BASt.AuswerteQuerschnitt))
    {  
      text += getWert(hotNeuesObjekt, "Strae");
      text += ".";
      text += getWert(hotNeuesObjekt, "Knotennummer");
      text += ".";
      text += getWert(hotNeuesObjekt, "Fahrtrichtung");
    }
    else if (hotNeuesObjekt.equals(HOT_BASt.LangZeitZaehlStelle))
    {
      text += getWert(hotNeuesObjekt, "Strae");
      text += ".";
      text += getWert(hotNeuesObjekt, "Knotennummer");
    }
    
    String[] result = new String[1]; 
    
    String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
   
    StringBuffer buffer = new StringBuffer();
    
    buffer.append(getPraefix(pidTypNeuesObjekt));
    buffer.append('.');
    
    buffer.append(BildungsRegelnTools.getPidText(text, true, true));
    
    String pid = buffer.toString(); 
    Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
    if (werte != null && !werte.isEmpty())
    {
      _historie.put(pid, new HashMap<String, String>(werte));
    }
    
    result[0] = pid;
    
    return result; 
  }
  
  public String getName(SystemObject objekt)
  {
  	String result = "";
  	
  	String pid = objekt.getPid();
  	
  	Map<String, String> werte = _historie.get(pid);
  	
  	if (werte != null)
  	{
  		SystemObjectType typ = objekt.getType();
  		StringBuffer buffer = new StringBuffer();
      
      if (typ.getPid().equals("typ.auswerteQuerschnitt"))
      {
        buffer.append("AWQ");
        buffer.append(' ');
        buffer.append(werte.get("Bezeichnung"));
        buffer.append(' ');
        buffer.append(werte.get("Fahrtrichtung"));
        
        result = buffer.toString();
      }
      else if (typ.getPid().equals("typ.langZeitZhlstelle"))
      {
        buffer.append("DZ");
        buffer.append(' ');
        buffer.append(werte.get("Bezeichnung"));
        
        result = buffer.toString();
      }
  	}
  	
  	return result;
  }
  		
  public Data getDatensatz(HierarchieObjekt ho, SystemObject objekt, AttributeGroup atg, Data daten,
  		boolean nurPflichtFelder)
  {
  	if (objekt.getType().getPid().equals("typ.langZeitZhlstelle"))
  	{	
	  	if (!ho.getSystemObjekte().contains(objekt))
	  	{
	  		throw new IllegalArgumentException("Systemobjekt " + objekt.getPid() + 
	  				" ist nicht im Hierarchieobjet enthalten");
	  	}
	  	
	  	if (!objekt.getType().getAttributeGroups().contains(atg))
	  	{
	  		throw new IllegalArgumentException("Attributgruppe " + atg.getName() +
	  				" kann nicht mit Systemobjekt " + objekt.getPid() + "verwendet werden "); 
	  	}
	  	
	  	Data data;
	  	
	  	if (daten == null)
	  	{
		    data = DataFactory.createData(atg);
		    // Default und undefiniert Werte setzen
		    data.setToDefault();
	  	}
	  	else
	  	{
	  		ObjectLookup lookUp = LookupFactory.createLookupForModifiableVersion(getHierarchie().getDataModel());
	  		data = DataFactory.createModifiableCopy(daten, lookUp);
	  	}
  	
	  	if (atg.getPid().equals("atg.langZeitZhlstelle"))
  		{
				data.getItem("ZhlstellenName").asTextValue().setText("");
				
  			// ZhlstellenNummer ist nicht vorbelegt
				// data.getItem("ZhlstellenNummer").asUnscaledValue().set(0);
				
				data.getItem("LandesNummer").asTextValue().setText("Baden-Wrttemberg");
				
  			// StraenKlasse ist nicht vorbelegt
				// data.getItem("StraenKlasse").asTextValue().setText("Autobahn");
				
				data.getItem("ZhlstellenArt").asTextValue().setText("Einfachzhlstelle");
				
  			// Strae ist nicht vorbelegt
  			//data.getItem("Strae").asReferenceValue().setSystemObject(null);
				
  			// vonNK ist nicht vorbelegt
				// data.getItem("vonNK").asUnscaledValue().set(0);
				
				// nachNK ist nicht vorbelegt
				// data.getItem("nachNK").asUnscaledValue().set(0);
				
				// Station ist nicht vorbelegt
				// data.getItem("Station").asUnscaledValue().set(0);
				
				// Beginn ist nicht vorbelegt
				// data.getItem("Beginn").asTimeValue().setSeconds(0);
				
				// Ende ist nicht vorbelegt
				// data.getItem("Ende").asTimeValue().setSeconds(0);
				
				data.getItem("KlassifizierungsgruppeBAST").asTextValue().setText("8 plus 1");
				
				data.getItem("TelefonNummer").asTextValue().setText("");
				
				
				Data inRichtung = data.getItem("InRichtung");

				// NK ist nicht vorbelegt
				// inRichtung.getItem("NK").asUnscaledValue().set(0);
				
				inRichtung.getItem("Nahziel").asTextValue().setText("");
				
				inRichtung.getItem("Fernziel").asTextValue().setText("");
				
				inRichtung.getItem("Bezeichnung").asTextValue().setText("");
				
				// Auswertequerschnitt ist nicht vorbelegt
  			//inRichtung.getItem("Auswertequerschnitt").asReferenceValue().setSystemObject(null);
				
				
				Data gegenRichtung = data.getItem("GegenRichtung");

				// NK ist nicht vorbelegt
				// gegenRichtung.getItem("NK").asUnscaledValue().set(0);
				
				gegenRichtung.getItem("Nahziel").asTextValue().setText("");
				
				gegenRichtung.getItem("Fernziel").asTextValue().setText("");
				
				gegenRichtung.getItem("Bezeichnung").asTextValue().setText("");
				
				// Auswertequerschnitt ist nicht vorbelegt
  			//gegenRichtung.getItem("Auswertequerschnitt").asReferenceValue().setSystemObject(null);
	 		}
	  	
	  	return data;
  	}
  	else
  	{
  		return super.getDatensatz(ho, objekt, atg, daten, nurPflichtFelder);
  	}	
  }
      
}
