/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkamq.preferences;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.help.IWorkbenchHelpSystem;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkamq.Activator;
import de.bsvrz.buv.plugin.tkamq.HOT_MQ;
import de.bsvrz.buv.plugin.tkamq.preferences.tree.AtgInfoNode;
import de.bsvrz.buv.plugin.tkamq.preferences.tree.ITreeNode;
import de.bsvrz.buv.plugin.tkamq.preferences.tree.RootNode;
import de.bsvrz.buv.plugin.tkamq.preferences.tree.TreeContentProvider;
import de.bsvrz.buv.plugin.tkamq.preferences.tree.TreeLabelProvider;

/**
 * Preferencepage fr die Einstellungen des TKA Verkehr MQ
 * 
 * @author Sans
 * 
 */
public class PreferencePage extends org.eclipse.jface.preference.PreferencePage
		implements IWorkbenchPreferencePage {
	/**
	 * ID der PreferencePage
	 */
	public static final String ID = "de.bsvrz.buv.plugin.tkamq.preferencePage";

	private Composite _top;

	private Group _grpBR;
	private Text _textBR;
	private Label _labelBR;

	private Group _grpP;
	private Text _textP;
	private Label _labelP;

	private Group _grpPL;
	private Button _checkPL;
	
	private Group _grpOptional;
	private CheckboxTreeViewer _treeVOptional;

	/**
	 * @wbp.parser.constructor
	 */
	public PreferencePage() {
		setPreferenceStore(Activator.getDefault().getPreferenceStore());
		// setPreferenceStore(PlatformUI.getPreferenceStore());

		String msg = "";
		msg = msg
				+ "Mit Hilfe dieser Seite knnen Einstellungen bzgl. des Plugins\n";
		msg = msg
				+ "Teilkonfigurationsassistent Verkehr MQ vorgenommen werden\n";

		setDescription(msg);
	}

	public PreferencePage(String title) {
		super(title);
	}

	public PreferencePage(String title, ImageDescriptor image) {
		super(title, image);
	}

	@Override
	protected Control createContents(Composite parent) {
		_top = new Composite(parent, SWT.NONE);

		_grpBR = new Group(_top, SWT.NONE);
		_grpBR.setText("Klassen-Instanz fr Bildungsregeln");

		_labelBR = new Label(_grpBR, SWT.NONE);
		_labelBR.setText("Klassenname:");

		_textBR = new Text(_grpBR, SWT.BORDER);

		_grpP = new Group(_top, SWT.NONE);
		_grpP.setText("Klassen-Instanz fr Prfungen");

		_labelP = new Label(_grpP, SWT.NONE);
		_labelP.setText("Klassenname:");

		_textP = new Text(_grpP, SWT.BORDER);

		//Prfung beim Lschen
//		_grpPL = new Group(_top, SWT.NONE);
//		_grpPL.setText("Prfung beim Lschen von Hierarchieobjekten");
//
//		_checkPL = new Button(_grpPL, SWT.CHECK);	
//		_checkPL.setText("Prfung durchfhren");
		
		//optionale Attributgruppen
		_grpOptional = new Group(_top, SWT.NONE);
		_grpOptional.setText("optionale Attributgruppen");

		_treeVOptional = new CheckboxTreeViewer(_grpOptional, SWT.BORDER);
		_treeVOptional.setContentProvider(new TreeContentProvider());
		_treeVOptional.setLabelProvider(new TreeLabelProvider());
		_treeVOptional.addCheckStateListener(new ICheckStateListener() {

			@Override
			public void checkStateChanged(CheckStateChangedEvent event) {
				boolean checked = event.getChecked();
				ITreeNode element = (ITreeNode) event.getElement();
				if (element.hasChildren()) {
					// ... check/uncheck all its children
					_treeVOptional.setSubtreeChecked(element, checked);
				}
			}
		});
		if (HOT_MQ.values().length > 0) {

			HOT_MQ[] hotMqs = HOT_MQ.values();

			List<HOT_MQ> list = new ArrayList<HOT_MQ>();

			for (int i = 0; i < hotMqs.length; i++) {
				HOT_MQ hotMQ = hotMqs[i];

				boolean istAtgMandatory = true;

				List<String> typen = hotMQ.getTypen();
				for (String string : typen) {

					TypInfo ti = hotMQ.getTypInfo(string);
					List<String> atgs = ti.getAtgs();

					for (String atg : atgs) {
						AtgInfo ai = ti.getAtgInfo(atg);
						istAtgMandatory = ai.isMandatory();
					}
				}

				if (!istAtgMandatory)
					list.add(hotMQ);

			}

			RootNode rootNode = new RootNode(list.toArray(new HOT_MQ[list
					.size()]));
			// RootNode rootNode = new RootNode(HOT_MQ.values());
			_treeVOptional.setInput(rootNode);
			_treeVOptional.expandAll();
			_treeVOptional.refresh();
		}

		/*
		 * Layout
		 */
		GridData gdTop = new GridData();
		gdTop.grabExcessHorizontalSpace = true;
		gdTop.horizontalAlignment = GridData.FILL;
		_top.setLayoutData(gdTop);

		GridLayout glTop = new GridLayout(1, false);
		_top.setLayout(glTop);

		GridData gdGrpBR = new GridData();
		gdGrpBR.grabExcessHorizontalSpace = true;
		gdGrpBR.horizontalAlignment = GridData.FILL;
		_grpBR.setLayoutData(gdGrpBR);

		GridLayout glGrpBR = new GridLayout(4, false);
		_grpBR.setLayout(glGrpBR);

		GridData gdTextBR = new GridData();
		gdTextBR.grabExcessHorizontalSpace = true;
		gdTextBR.horizontalAlignment = GridData.FILL;
		_textBR.setLayoutData(gdTextBR);

		GridData gdGrpP = new GridData();
		gdGrpP.grabExcessHorizontalSpace = true;
		gdGrpP.horizontalAlignment = GridData.FILL;
		_grpP.setLayoutData(gdGrpP);

		GridLayout glGrpP = new GridLayout(4, false);
		_grpP.setLayout(glGrpP);

		GridData gdTextP = new GridData();
		gdTextP.grabExcessHorizontalSpace = true;
		gdTextP.horizontalAlignment = GridData.FILL;
		_textP.setLayoutData(gdTextP);

		//Prfung beim Lschen
//		_grpPL.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
//		
//		GridLayout glGrpPL = new GridLayout(4, false);
//		_grpPL.setLayout(glGrpPL);
		
		_grpOptional.setLayoutData(new GridData(GridData.FILL_BOTH));
		_grpOptional.setLayout(new GridLayout());

		/*
		 * Defaultwerte setzen
		 */
		actionLoadFromPreferenceStore();

		return null;
	}

	/**
	 * Stellt die Default-Einstellungen aus den Prferenzen wieder her.
	 */
	private void actionLoadDefaultsFromPreferenceStore() {
		// Preferences preferences =
		// Activator.getDefault().getPluginPreferences();
		// IPreferenceStore preferences = PlatformUI.getPreferenceStore();

		IPreferenceStore preferences = Activator.getDefault()
				.getPreferenceStore();
		// IPreferenceStore preferences = getPreferenceStore();

		String klassenNameBR = preferences
				.getDefaultString(DefsPara.KlasseBildungsregeln);

		if (_textBR != null) {
			_textBR.setText(klassenNameBR);
		}

		String klassenNameP = preferences
				.getDefaultString(DefsPara.KlassePruefungen);

		if (_textP != null) {
			_textP.setText(klassenNameP);
		}

		//Prfung beim Lschen
//		boolean selectedPL = true;
//		if (preferences.contains(DefsPara.PruefenLoechen))	
//				selectedPL = preferences.getDefaultBoolean(DefsPara.PruefenLoechen);
//		
//		if (_checkPL != null) {
//			_checkPL.setSelection(selectedPL);
//		}

		// DefaultWerte aus den Prferenzen oder den AtgInfo-Objekten holen
		if (_treeVOptional != null) {
			Object input = _treeVOptional.getInput();
			if (input instanceof ITreeNode) {
				ITreeNode root = (ITreeNode) input;
				setDefaultCheck(root);
			}
		}

	}

	/**
	 * Rekursives Setzen der Checkboxen im Baum auf Checked entsprechend den
	 * Informationen aus aus den Prferenzen oder AtgInfo
	 * 
	 * @param node
	 *            Knoten-Element des Baumes
	 */
	private void setDefaultCheck(ITreeNode node) {
		IPreferenceStore preferences = Activator.getDefault()
				.getPreferenceStore();

		if (node.hasChildren()) {
			List<Object> children = node.getChildren();
			for (Object obj : children) {
				ITreeNode child = (ITreeNode) obj;
				if (child.hasChildren())
					setDefaultCheck(child);
				else if (child instanceof AtgInfoNode) {
					AtgInfo atgInfo = ((AtgInfoNode) child).getAtgInfo();
					String typ = ((AtgInfoNode) child).getTyp();

					if (preferences.contains(typ + "," + atgInfo.getAtg())) {

						boolean isOptional = preferences.getDefaultBoolean(typ
								+ "," + atgInfo.getAtg());						 

						_treeVOptional.setChecked(child, isOptional);
					}
					else if (atgInfo.isOptional()) {
						_treeVOptional.setChecked(child, true);
					}
				}
			}
		}

	}

	/**
	 * Rekursives Setzen der Checkboxen im Baum auf Checked entsprechend den
	 * Informationen aus den Prferenzen oder AtgInfo
	 * 
	 * @param node
	 *            Knoten-Element des Baumes
	 */
	private void setCheck(ITreeNode node) {
		IPreferenceStore preferences = Activator.getDefault()
				.getPreferenceStore();

		if (node.hasChildren()) {
			List<Object> children = node.getChildren();
			for (Object obj : children) {
				ITreeNode child = (ITreeNode) obj;
				if (child.hasChildren())
					setCheck(child);
				else if (child instanceof AtgInfoNode) {
					AtgInfo atgInfo = ((AtgInfoNode) child).getAtgInfo();
					String typ = ((AtgInfoNode) child).getTyp();

					if (preferences.contains(typ + "," + atgInfo.getAtg())) {
						boolean isOptional = preferences.getBoolean(typ + ","
								+ atgInfo.getAtg());


						_treeVOptional.setChecked(child, isOptional);
					}
					else if (atgInfo.isOptional()){
						_treeVOptional.setChecked(child, true);
					}				
				}
			}
		}

	}

	/**
	 * Stellt die Einstellungen aus den Prferenzen wieder her. Wenn keine
	 * gesetzt wurden, werden die Defaulteinstellungen wiederhergestellt.
	 */
	private void actionLoadFromPreferenceStore() {

		IPreferenceStore preferences = Activator.getDefault()
				.getPreferenceStore();
		// IPreferenceStore preferences = getPreferenceStore();

		String klassenNameBR = preferences
				.getString(DefsPara.KlasseBildungsregeln);

		if (_textBR != null) {
			_textBR.setText(klassenNameBR);
		}

		String klassenNameP = preferences.getString(DefsPara.KlassePruefungen);

		if (_textP != null) {
			_textP.setText(klassenNameP);
		}
		
		// Einstellungen fr den Tree aus den Prferenzen holen
		if (_treeVOptional != null) {
			Object input = _treeVOptional.getInput();
			if (input instanceof ITreeNode) {
				ITreeNode root = (ITreeNode) input;
				setCheck(root);
			}

		}
	}

	/**
	 * Speichert die i.d. PreferencePage gemachten Einstellungen in den
	 * Prferenzen
	 */
	private void actionSaveToPreferenceStore() {
		boolean allValuesAvailable = true;
		String klassenNameBR = "";
		String klassenNameP = "";
		Boolean selectedPL = false;
		
		if (_textBR != null && _textP != null) {
			klassenNameBR = _textBR.getText();
			klassenNameP = _textP.getText();
		} else {
			allValuesAvailable = false;
		}
		// optionale Attributgruppen in den Prferenzen und in den
		// AtgInfo-Objekten speichern
		if (_treeVOptional != null) {
			Object input = _treeVOptional.getInput();
			if (input instanceof ITreeNode) {
				ITreeNode root = (ITreeNode) input;
				saveOptional(root);
			}
		}

		if (allValuesAvailable) {
			IPreferenceStore preferences = Activator.getDefault()
					.getPreferenceStore();
			// IPreferenceStore preferences = getPreferenceStore();

			preferences.setValue(DefsPara.KlasseBildungsregeln, klassenNameBR);
			preferences.setValue(DefsPara.KlassePruefungen, klassenNameP);
			preferences.setValue(DefsPara.PruefenLoechen, selectedPL);
		}

	}

	/**
	 * Rekursives Sichern der Checkboxen des Baumes in den Prferenzen und den AtgInfos
	 * 
	 * @param node
	 *            Knoten-Element des Baumes
	 */
	private void saveOptional(ITreeNode node) {

		IPreferenceStore preferences = Activator.getDefault()
				.getPreferenceStore();

		if (node.hasChildren()) {
			List<Object> children = node.getChildren();
			for (Object obj : children) {
				ITreeNode child = (ITreeNode) obj;
				if (child.hasChildren())
					saveOptional(child);
				else if (child instanceof AtgInfoNode) {
					AtgInfo atgInfo = ((AtgInfoNode) child).getAtgInfo();
					String typ = ((AtgInfoNode) child).getTyp();
					if (!atgInfo.isMandatory()) {
						atgInfo.setOptional(_treeVOptional.getChecked(child));

						preferences.setValue(typ + "," + atgInfo.getAtg(),
								atgInfo.isOptional());
					}
				}
			}
		}

	}

	@Override
	protected void performApply() {
		actionSaveToPreferenceStore();
	}

	@Override
	public boolean performCancel() {
		return super.performCancel();
	}

	@Override
	protected void performDefaults() {
		actionLoadDefaultsFromPreferenceStore();
		super.performDefaults();
	}

	@Override
	public boolean performOk() {
		performApply();
		return super.performOk();
	}

	@Override
	public void init(IWorkbench workbench) {

	}

	@Override
	public void performHelp() {
		String helpContextId = Activator.PLUGIN_ID + ".preferencepage";
		IWorkbenchHelpSystem helpSystem = PlatformUI.getWorkbench()
				.getHelpSystem();
		// helpSystem.displayHelp(helpContextId);
		// helpSystem.displayHelp(helpContextId);
	}

}
