/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkanetz.preferences;

import java.util.List;

import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.jface.preference.IPreferenceStore;

import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.TypInfo;
import de.bsvrz.buv.plugin.tkanetz.Activator;
import de.bsvrz.buv.plugin.tkanetz.HOT_Netz;

/**
 * Initialisierung der Einstellungen fr TKA Netz Netzbestandteile
 * 
 * @author Krause
 * 
 */
public class PreferenceInitializer extends AbstractPreferenceInitializer
{
	@Override
	public void initializeDefaultPreferences()
	{
		IPreferenceStore preferences = Activator.getDefault().getPreferenceStore();
		
		/**
		 * Default-Klasse fr Bildungsregeln
		 */
		preferences.setDefault(DefsPara.KlasseBildungsregeln, "de.bsvrz.buv.plugin.tkanetz.BildungsRegelnAllgemeinNetz" );
		preferences.setDefault(DefsPara.KlassePruefungen, "de.bsvrz.buv.plugin.tkanetz.PruefungenNetz" );
		
		/**
		 * Default-Wert fr die Prfung beim Lschen
		 */	
		preferences.setDefault(DefsPara.PruefenLoechen, true );
		
		preferences.setDefault(DefsPara.VorbelegungVerwaltung, "" );
		
		/**
		 * Default-Werte fr die optionalen Attributgruppen
		 */		
		HOT_Netz[] hots = HOT_Netz.values();

		for (int i = 0; i < hots.length; i++) {
			HOT_Netz hot = hots[i];

			boolean istAtgMandatory = true;
			 
			List<String> typen = hot.getTypen();
			for (String typ : typen) {

				TypInfo ti = hot.getTypInfo(typ);
				List<String> atgs = ti.getAtgs();

				for (String atg : atgs) {
					AtgInfo ai = ti.getAtgInfo(atg);
					istAtgMandatory = ai.isMandatory();					
					if (!istAtgMandatory){						
						preferences.setDefault(typ + "," + atg, ai.isOptional());
					}
				}
			}
		}
	}
}
