/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkatls;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import de.bsvrz.buv.plugin.tkabasis.DataFactory;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.regeln.BildungsRegelnTools;
import de.bsvrz.buv.plugin.tkabasis.regeln.EingabeVorbelegung;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;

public class BildungsRegelnBWTLS extends BildungsRegelnAllgemeinTLS
{
	 protected Map<String, HierarchieObjekt> _uebergeordneteHO = new HashMap<String, HierarchieObjekt>();
	 
	 protected final EingabeVorbelegung _belegungPortnummer = new EingabeVorbelegung(0, 9223372036854775807l);
	 protected final EingabeVorbelegung _belegungKnotennummer = new EingabeVorbelegung(0, 16777215);
	 protected final EingabeVorbelegung _belegungOSI2Adresse = new EingabeVorbelegung(0, 255);
	 
	 protected Map<IHierarchieObjektTyp, EingabeVorbelegung> _belegungFgKuerzel = new HashMap<IHierarchieObjektTyp, EingabeVorbelegung>();
	
	 @Override
	  public void setHierarchie(IHierarchie hierarchie)
		{
	  	IHierarchie alteHierarchie = getHierarchie();
	  	
	  	super.setHierarchie(hierarchie);
	  	
	  	if (hierarchie != alteHierarchie)
	  	{	
	  		_belegungFgKuerzel.clear();
	  		DeTypenManager deTypenManager = DatenSpeicher.getInstanz().getDeTypenManager();
			  
				for (DeUnterTyp unterTyp: deTypenManager.getDeEbene())
				{
					_belegungFgKuerzel.put(unterTyp, new EingabeVorbelegung(bestimmeFgKuerzel(unterTyp.getFg()), null, false, false));
				}
	  	}	
	  	
	  	_uebergeordneteHO.clear();
	  	_vorherigeEakOSI2Adresse = null;
		}
	 
	 public BildungsRegelnBWTLS()
	 {
		 super();
		  
//		 DeTypenManager deTypenManager = DatenSpeicher.getInstanz().getDeTypenManager();
//		  
//		 for (DeUnterTyp unterTyp: deTypenManager.getDeEbene())
//		 {
//		 	_belegungFgKuerzel.put(unterTyp, new EingabeVorbelegung(bestimmeFgKuerzel(unterTyp.getFg()), null, false, false));
//		 }
	 }
	 
	 @Override
	 protected void initPraefixe()
	  {
	    _praefixe.put("typ.viz", "viz");
	    _praefixe.put("typ.vrz", "vrz");
	    _praefixe.put("typ.uz", "uz");
	    _praefixe.put("typ.kri", "");
	    _praefixe.put("typ.steuerModul", "SM");
	    _praefixe.put("typ.eak", "EAK");
	    
	    _praefixe.put("typ.anschlussPunkt", "anschlussPunkt");
	    _praefixe.put("typ.anschlussPunktKommunikationsPartner", "anschlussPunktKommunikationsPartner");
	    
	    DeTypenManager deTypenManager = DatenSpeicher.getInstanz().getDeTypenManager();
	    for (String typ: deTypenManager.getSystemObjektTypen())
	    {	
	      _praefixe.put(typ, "De");
	    }
	  }
	  
	  @Override
	  protected void initFelder()
	  {
	    _felder.put(HOT_TLS.VIZ, new String[] { "Bezeichnung" });
	    _felder.put(HOT_TLS.VRZ, new String[] { "Bezeichnung" });
	    _felder.put(HOT_TLS.UZ, new String[] { "Bezeichnung" });
	    _felder.put(HOT_TLS.KRI, new String[] { "Bezeichnung", "KriPID" });
	    _felder.put(HOT_TLS.Inselbus, new String[] { "ZustzlicheBezeichnung" });
	    _felder.put(HOT_TLS.SM, new String[] { "BezeichnungSM", "Portnummer", "Knotennummer" });
	    _felder.put(HOT_TLS.EAK, new String[] { "OSI2Adresse (EAK)" });
	    
	    DeTypenManager deTypenManager = DatenSpeicher.getInstanz().getDeTypenManager();
	    for (IHierarchieObjektTyp hot: deTypenManager.getDeEbene())
	    {	
	    	_felder.put(hot, new String[] { "BezeichnungDE", "DEKanal", "FGKrzel" });
	    }	
	  }
	  
	  @Override
	  public EingabeVorbelegung getEingabeVorbelegung(IHierarchieObjektTyp hotNeuesObjekt, String eingabeFeld, HierarchieObjekt verbundenesHO)
	  {
	  	if (HOT_TLS.SM.equals(hotNeuesObjekt))
  		{
	  		if ("Portnummer".equals(eingabeFeld))
	  		{	
	  			return _belegungPortnummer;
	  		}
	  		if ("Knotennummer".equals(eingabeFeld))
	  		{	
	  			return _belegungKnotennummer;
	  		}
  		}
	  	if (HOT_TLS.EAK.equals(hotNeuesObjekt))
	  	{
	  		if ("OSI2Adresse (EAK)".equals(eingabeFeld))
	  		{	
	  			return _belegungOSI2Adresse;
	  		}
	  	}
	  	if (hotNeuesObjekt instanceof DeUnterTyp)
	  	{
	  		if ("FGKrzel".equals(eingabeFeld))
	  		{	
	  			return _belegungFgKuerzel.get(hotNeuesObjekt);
	  		}
	  	}
	  	
	  	return super.getEingabeVorbelegung(hotNeuesObjekt, eingabeFeld, verbundenesHO);
	  }
	  
	  public String[] getPids(IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO)
	  {
	  	String[] result = BildungsRegelnTools.LEERE_TEXT_LISTE;
	  	
	  	if (hotNeuesObjekt.equals(HOT_TLS.VIZ))
	  	{
	  		result = new String[1];
	  		StringBuffer buffer = new StringBuffer();
	  		String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
	  		buffer.append(getPraefix(pidTypNeuesObjekt));
	  		String bezeichnung = getWert(hotNeuesObjekt, "Bezeichnung");
	      buffer.append(BildungsRegelnTools.getPidText(bezeichnung, true, true));
	      result[0] = buffer.toString();
	  	}
	  	else if (hotNeuesObjekt.equals(HOT_TLS.VRZ) || hotNeuesObjekt.equals(HOT_TLS.UZ))
	  	{
	  		if (!(uebergeordnetesHO instanceof HierarchieObjekt))
	  		{
	  			result = new String[1];
		  		StringBuffer buffer = new StringBuffer();
		  		String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
		  		buffer.append(getPraefix(pidTypNeuesObjekt));
		  		String bezeichnung = getWert(hotNeuesObjekt, "Bezeichnung");
		      buffer.append(BildungsRegelnTools.getPidText(bezeichnung, true, true));
		      result[0] = buffer.toString();
	  		}
	  		else
	  		{
	  			result = new String[3];
	  			StringBuffer buffer = new StringBuffer();
		  		String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
		  		buffer.append(getPraefix(pidTypNeuesObjekt));
		  		String bezeichnung = getWert(hotNeuesObjekt, "Bezeichnung");
		      buffer.append(BildungsRegelnTools.getPidText(bezeichnung, true, true));
		      
		      result[0] = buffer.toString();

//	        // Fr ap und apkp wird die PID des KP vorrrangig aus dem vorlageHO geholt
//	        HierarchieObjekt vorlageHO = getVorlageHO();
//	        String kpPid = vorlageHO != null ? vorlageHO.getSystemObjekt(0).getPid() : result[0];
	        
	        // Fr ap und apkp wird die PID des KP vorrrangig aus _hoVRZUZ geholt
	        String kpPid = _hoVRZUZ != null ? _hoVRZUZ.getSystemObjekt(0).getPid() : result[0];
	        
	        // ap
	        buffer = new StringBuffer();
	        buffer.append(getPraefix("typ.anschlussPunkt"));
	        buffer.append('.');
	        buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
	        buffer.append('.');
	        buffer.append(kpPid);
	        result[1] = buffer.toString();
	        
	        // apkp
	        buffer = new StringBuffer();
	        buffer.append(getPraefix("typ.anschlussPunktKommunikationsPartner"));
	        buffer.append('.');
	        buffer.append(kpPid);
	        buffer.append('.');
	        buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
	        result[2] = buffer.toString();
	  		}
	  	}
	  	else if (hotNeuesObjekt.equals(HOT_TLS.KRI))
			{
  			result = new String[3];
  			StringBuffer buffer = new StringBuffer();
	  		String bezeichnung = getWert(hotNeuesObjekt, "KriPID");
	      buffer.append(BildungsRegelnTools.getPidText(bezeichnung, true, true));
	      
	      result[0] = buffer.toString();

//        // Fr ap und apkp wird die PID des KP vorrrangig aus dem vorlageHO geholt
//        HierarchieObjekt vorlageHO = getVorlageHO();
//        String kpPid = vorlageHO != null ? vorlageHO.getSystemObjekt(0).getPid() : result[0];
        
        // Fr ap und apkp wird die PID des KP vorrrangig aus _hoVRZUZ geholt
        String kpPid = _hoVRZUZ != null ? _hoVRZUZ.getSystemObjekt(0).getPid() : result[0];
        
        // ap
        buffer = new StringBuffer();
        buffer.append(getPraefix("typ.anschlussPunkt"));
        buffer.append('.');
        buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
        buffer.append('.');
        buffer.append(kpPid);
        result[1] = buffer.toString();
        
        // apkp
        buffer = new StringBuffer();
        buffer.append(getPraefix("typ.anschlussPunktKommunikationsPartner"));
        buffer.append('.');
        buffer.append(kpPid);
        buffer.append('.');
        buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
        result[2] = buffer.toString();

			}
	  	else if (hotNeuesObjekt.equals(HOT_TLS.Inselbus))
			{
	  		result = new String[1];
	  			  		
	  		StringBuffer buffer = new StringBuffer();
        buffer.append(getPraefix("typ.anschlussPunkt"));
        buffer.append('.');
        buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
        buffer.append('.');
      	String zusatzBez = getWert(hotNeuesObjekt, "ZustzlicheBezeichnung");
	      buffer.append(BildungsRegelnTools.getPidText(zusatzBez, true, true));
        result[0] = buffer.toString();
			}
	  	else if (hotNeuesObjekt.equals(HOT_TLS.SM))
			{
	  		result = new String[2];

	  		HierarchieObjekt vaterHO = (HierarchieObjekt)getHierarchie().getUebergeordnetesObjekt(uebergeordnetesHO);
	  		StringBuffer buffer = new StringBuffer();
	  		buffer.append(vaterHO.getSystemObjekt(0).getPid());
	  		buffer.append('.');
        buffer.append(getPraefix("typ.steuerModul"));
        String portNr = getWert(hotNeuesObjekt, "Portnummer"); 
        buffer.append(BildungsRegelnTools.getPidText(portNr, true, true));
        buffer.append('_');
        String knotenNr = getWert(hotNeuesObjekt, "Knotennummer"); 
        buffer.append(BildungsRegelnTools.getPidText(knotenNr, true, true));
        result[0] = buffer.toString();
        
	  		buffer = new StringBuffer();
        buffer.append(getPraefix("typ.anschlussPunktKommunikationsPartner"));
        buffer.append('.');
        buffer.append(result[0]);
        buffer.append('.');
        buffer.append(vaterHO.getSystemObjekt(0).getPid());
        result[1] = buffer.toString();
			}
	  	else if (hotNeuesObjekt.equals(HOT_TLS.EAK))
			{
	  		result = new String[1];

	  		StringBuffer buffer = new StringBuffer();
	  		buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
	  		buffer.append('.');
        buffer.append(getPraefix("typ.eak"));
        String adresse = getWert(hotNeuesObjekt, "OSI2Adresse (EAK)");
        buffer.append(BildungsRegelnTools.getPidText(adresse, true, true));
        result[0] = buffer.toString();
			}
	  	else if (hotNeuesObjekt instanceof DeUnterTyp)
	  	{
	  		result = new String[1];
	  		
	  		StringBuffer buffer = new StringBuffer();
	  		buffer.append(uebergeordnetesHO.getSystemObjekt(0).getPid());
	  		String pidTypNeuesObjekt = hotNeuesObjekt.getTypen().get(0);
	  		buffer.append('.');
	  		buffer.append(getPraefix(pidTypNeuesObjekt));
//	  		String fgText = getWert(hotNeuesObjekt, "Funktionsgruppe");
	  		String fgKuerzel = bestimmeFgKuerzel(((DeUnterTyp)hotNeuesObjekt).getFg());
	  		buffer.append(fgKuerzel);
	      String deKanal = getWert(hotNeuesObjekt, "DEKanal");
	      buffer.append(BildungsRegelnTools.getPidText(deKanal, false, false));
	      result[0] = buffer.toString();
	  	}
	  	
	  	if (result.length > 0)
	  	{
		  	Map<String, String> werte = _feldWerte.get(hotNeuesObjekt);
//		    if (werte != null && !werte.isEmpty())
//		    {
//		      _historie.put(result[0], new HashMap<String, String>(werte));
//		    }
		    
		    for (int i = 0; i < result.length; i += 1)
		    {
		    	if (werte != null && !werte.isEmpty())
			    {
			      _historie.put(result[i], new HashMap<String, String>(werte));
			    }
		    	if (uebergeordnetesHO != null)
		    	{	
		    		_uebergeordneteHO.put(result[i], uebergeordnetesHO);
		    	}	
		    }	
	  	}
	  
	  	return result;
	  }	
	  
	  protected String bestimmeFgKuerzel(int fg)
	  {
	  	String result = BildungsRegelnTools.LEERER_WERT;
	  		  	
	  	switch (fg)
	  	{
	  		case 1:
	  			result = "Lve";
	  			break;
	  			
	  		case 2:
	  			result = "Axl";
	  			break;
	  			
	  		case 3:
	  			result = "Ufd";
	  			break;
	  			
	  		case 4:
	  			result = "Wzg";
	  			break;
	  			
	  		case 5:
	  			result = "Www";
	  			break;
	  			
	  		case 6:
	  			result = "Vlt";
	  			break;
	  			
	  		case 254:
	  			result = "Sys";
	  			break;
	  		
	  		default:
	  			result = String.format("%03d.", fg);
	  			break;
	  	}
	  	
	  	return result;
	  }
	  
	  @Override
	  public String getName(SystemObject objekt)
	  {
	  	String pid = objekt.getPid();
	  	
	  	Map<String, String> werte = _historie.get(pid);
	  	
	  	if (werte != null)
	  	{
	  		SystemObjectType typ = objekt.getType();
	  		StringBuffer buffer = new StringBuffer();
	      
	      if (typ.getPid().equals("typ.viz"))
	      {
	        buffer.append("VIZ ");
	        buffer.append(werte.get("Bezeichnung"));
	        return buffer.toString();
	      }
	      else if (typ.getPid().equals("typ.vrz"))
	      {
	        buffer.append("VRZ ");
	        buffer.append(werte.get("Bezeichnung"));
	        return buffer.toString();
	      }
	      else if (typ.getPid().equals("typ.uz"))
	      {
	        buffer.append("UZ ");
	        buffer.append(werte.get("Bezeichnung"));
	        return buffer.toString();
	      }
	      else if (typ.getPid().equals("typ.kri"))
	      {
	        buffer.append(werte.get("Bezeichnung"));
	        return buffer.toString();
	      }
	      else if (typ.getPid().equals("typ.steuerModul"))
	      {
	        buffer.append(werte.get("BezeichnungSM"));
	        buffer.append(" SM");
	        buffer.append(werte.get("Portnummer"));
	        buffer.append('_');
	        buffer.append(werte.get("Knotennummer"));
	        return buffer.toString();
	      }  
	      else if (typ.getPid().equals("typ.eak"))
	      {
	      	HierarchieObjekt smHO = _uebergeordneteHO.get(objekt.getPid());
	      	String bezSM = bestimmeBezeichnungSM(smHO);
	      	buffer.append(bezSM);
	      	buffer.append(" EAK");
	      	buffer.append(werte.get("OSI2Adresse (EAK)"));
	      	return buffer.toString();
	      }
	      else if (typ.getPid().equals("typ.anschlussPunkt"))
	      {
	      	buffer.append("Verbindung ");
	      	
	      	HierarchieObjekt vaterHO = _uebergeordneteHO.get(objekt.getPid());
	      	buffer.append(vaterHO.getSystemObjekt(0).getPid());
	      	
	      	buffer.append("-");
	      	
	      	HierarchieObjekt kindHO = bestimmeHOMitAP(vaterHO, objekt);
	      	if (HOT_TLS.VRZ.equals(kindHO.getHierarchieObjektTyp()) ||
		  				HOT_TLS.UZ.equals(kindHO.getHierarchieObjektTyp()) ||
		  				HOT_TLS.KRI.equals(kindHO.getHierarchieObjektTyp()))
					{
	      		buffer.append(kindHO.getSystemObjekt(0).getPid());
					}
		  		else if (HOT_TLS.Inselbus.equals(kindHO.getHierarchieObjektTyp()))
		  		{
		  			buffer.append(werte.get("ZustzlicheBezeichnung"));
		  		}
	      	
	      	return buffer.toString();
	      }
	      else if (typ.getPid().equals("typ.anschlussPunktKommunikationsPartner"))
	      {
	      	buffer.append("Verbindung ");
	      	
	      	SystemObject kind = bestimmeAPKP(objekt);
	      	buffer.append(kind.getPid());
	      	buffer.append("-");
	      	
	      	HierarchieObjekt vaterHO = _uebergeordneteHO.get(objekt.getPid());
	      	
	      	if (kind.isOfType("typ.vrz") || kind.isOfType("typ.uz") || kind.isOfType("typ.kri"))
					{
	      		buffer.append(vaterHO.getSystemObjekt(0).getPid());
					}
		  		else if (kind.isOfType("typ.steuerModul"))
		  		{
		  			vaterHO = (HierarchieObjekt)getHierarchie().getUebergeordnetesObjekt(vaterHO);
		  			buffer.append(vaterHO.getSystemObjekt(0).getPid());
		  		}
	      	
	      	return buffer.toString();
	      }
	      else
	      {
	      	ConfigDataModel dataModel = getHierarchie().getDataModel();
	      	SystemObjectType deTyp = dataModel.getType("typ.de");
	      	
	      	if (typ.inheritsFrom(deTyp))
	      	{
	      		HierarchieObjekt eakHO = _uebergeordneteHO.get(objekt.getPid());
	      		HierarchieObjekt smHO = (HierarchieObjekt)getHierarchie().getUebergeordnetesObjekt(eakHO);
	      		String bezSM = bestimmeBezeichnungSM(smHO);
		      	buffer.append(bezSM);
		      	buffer.append(" / ");
		      	buffer.append(werte.get("BezeichnungDE"));
		      	return buffer.toString();
		      }	
	      }	
	  	}  
	        	  	
	  	//return BildungsRegelnTools.LEERER_WERT;
	  	return super.getName(objekt);
	  }
	  
	  SystemObject bestimmeAPKP(SystemObject apkp)
	  {
	    SystemObject result = null;
	  	
	  	ConfigDataModel dataModel = getHierarchie().getDataModel();
	  	ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
	  	AttributeGroup atg = dataModel.getAttributeGroup( "atg.anschlussPunktKommunikationsPartner");
	  	Data data = ((ConfigSystemObject)apkp).getConfigurationData(atg, lookup);
	  	if (data != null)
	  	{
	  		result = data.getItem("KommunikationsPartner").asReferenceValue().getSystemObject();
	  	}
	  	
	  	return result;
	  }
	  
	  protected HierarchieObjekt bestimmeHOMitAP(HierarchieObjekt vater, SystemObject ap)
	  {
	  	for (HierarchieObjekt kind: getHierarchie().getUntergeordneteObjekte(vater))
	  	{
	  		if (HOT_TLS.VRZ.equals(kind.getHierarchieObjektTyp()) ||
	  				HOT_TLS.UZ.equals(kind.getHierarchieObjektTyp()) ||
	  				HOT_TLS.KRI.equals(kind.getHierarchieObjektTyp()))
				{
					if (kind.getSystemObjekt(1) == ap)
					{
						return kind;
					}
				}
	  		else if (HOT_TLS.Inselbus.equals(kind.getHierarchieObjektTyp()))
	  		{
	  			if (kind.getSystemObjekt(0) == ap)
					{
						return kind;
					}
	  		}
	  	}
	  	
	  	return null;
	  }
	  
	  protected String bestimmeBezeichnungSM(HierarchieObjekt smHO)
	  {
	  	String result = null;
	  	
	  	SystemObject sm = smHO.getSystemObjekt(0);
	  	ConfigDataModel dataModel = getHierarchie().getDataModel();
	  	ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
	  	AttributeGroup atg = dataModel.getAttributeGroup( "atg.gert");
	  	Data data = ((ConfigSystemObject)sm).getConfigurationData(atg, lookup);
	  	if (data != null)
	  	{
	  		result = data.getItem("Bezeichnung").asTextValue().getText();
	  	}
	  	
	  	return result;
	  }
	  
	  @Override
	  public Data getDatensatz(HierarchieObjekt ho, SystemObject objekt, AttributeGroup atg, Data daten,
	  		boolean nurPflichtFelder)
	  {
	  	if (!ho.getSystemObjekte().contains(objekt))
	  	{
	  		throw new IllegalArgumentException("Systemobjekt " + objekt.getPid() + 
	  				" ist nicht im Hierarchieobjet enthalten");
	  	}
	  	
	  	if (!objekt.getType().getAttributeGroups().contains(atg))
	  	{
	  		throw new IllegalArgumentException("Attributgruppe " + atg.getName() +
	  				" kann nicht mit Systemobjekt " + objekt.getPid() + "verwendet werden "); 
	  	}
	  	
	  	//IHierarchie hierarchie = getHierarchie();
	  	//ConfigDataModel dataModel = hierarchie.getDataModel();
	  	
	  	Data data;
	  	
	  	if (daten == null)
	  	{
		    data = DataFactory.createData(atg);
		    // Default und undefiniert Werte setzen
		    data.setToDefault();
	  	}
	  	else
	  	{
	  		ObjectLookup lookUp = LookupFactory.createLookupForModifiableVersion(getHierarchie().getDataModel());
	  		data = DataFactory.createModifiableCopy(daten, lookUp);
	  	}
	  	
	  	Map<String, String> werte = _historie.get(objekt.getPid());
	  	if (werte == null)
	  	{
	  		werte = Collections.emptyMap();
	  	}
	  	
	  	IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
	  	
	  	if (hot.equals(HOT_TLS.VIZ) ||
	  			hot.equals(HOT_TLS.VRZ) ||
	  			hot.equals(HOT_TLS.UZ) ||
	  			hot.equals(HOT_TLS.KRI))
	  	{
	  		if (objekt.isOfType("typ.viz") || objekt.isOfType("typ.vrz") || objekt.isOfType("typ.uz") || objekt.isOfType("typ.kri"))
	  		{
	  			return super.getDatensatz(ho, objekt, atg, daten, nurPflichtFelder);
	  		}
	  		else if (objekt.isOfType("typ.anschlussPunkt"))
	  		{
	  			if (atg.getPid().equals("atg.anschlussPunkt"))
	  			{
//	  				if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//	    			{	
//	    				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//	    			}
	  				
	  				// Bezeichnung soll Pflichtfeld sein
	  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
	  				
	  				if (!nurPflichtFelder) // ProtokollTyp soll kein Pflichtfeld sein
	      		{	
	  					data.getItem("ProtokollTyp").asTextValue().setText("de.bsvrz.kex.tls.osi2osi3.osi2.wancom.Client");
	      		}
	  				
	  				long portNummer = bestimmePortNummerAP(ho);
	  				data.getItem("PortNummer").asTextValue().setText("" + portNummer);
	  			}
	  		}
	  		else if (objekt.isOfType("typ.anschlussPunktKommunikationsPartner"))
	  		{
	  			if (atg.getPid().equals("atg.anschlussPunktKommunikationsPartner"))
	  			{
//	  				if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//	    			{	
//	    				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//	    			}
	  				
	  				// Bezeichnung soll Pflichtfeld sein
	  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
	  				
	  				if (!nurPflichtFelder) // ProtokollTyp soll kein Pflichtfeld sein
	      		{	
	  					data.getItem("ProtokollTyp").asTextValue().setText("");
	      		}	
	  				
	  				// KommunikationsPartner wird beim Bilden der Hierarchie gesetzt
	  				
	  				long portNummer = bestimmePortNummerAPKP(ho);
	  				data.getItem("PortNummer").asTextValue().setText("" + portNummer);
	  			}
	  		}
	  	}
	  	else if (hot.equals(HOT_TLS.Inselbus))
	 		{
	  		if (objekt.isOfType("typ.anschlussPunkt"))
	  		{
	  			if (atg.getPid().equals("atg.anschlussPunkt"))
	  			{
//	  				if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//	    			{	
//	    				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//	    			}
	  				
	  				// Bezeichnung soll Pflichtfeld sein
	  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
	  				
	  				if (!nurPflichtFelder) // ProtokollTyp soll kein Pflichtfeld sein
	      		{	
	  					data.getItem("ProtokollTyp").asTextValue().setText("");
	      		}	
	  				
	  				// Eigentlich keine Vorbelegung fr Portnummer bei BW
//	  				long portNummer = bestimmePortNummerAP(ho);
//	  				data.getItem("PortNummer").asTextValue().setText("" + portNummer);
	  			}
	  		}
	 		}
	  	else if (hot.equals(HOT_TLS.SM))
			{
	  		if (objekt.isOfType("typ.steuerModul"))
	  		{
		  		if (atg.getPid().equals("atg.gert"))
					{
//						if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//		  			{
//							String bez = werte.get("BezeichnungSM");
//							if (bez != null)
//							{	
//								data.getItem("Bezeichnung").asTextValue().setText(bez);
//							}
////							else
////							{
////								// keine Vorbelegung
////							}
//		  			}	
						
		  			
	  				// Bezeichnung soll Pflichtfeld sein
		  			String bez = werte.get("BezeichnungSM");
						if (bez != null)
						{	
							data.getItem("Bezeichnung").asTextValue().setText(bez);
						}	
		  			
						String textKnotenNr = werte.get("Knotennummer");
						long knotenNummer;
						try
						{
							knotenNummer = Long.parseLong(textKnotenNr);
							data.getItem("KnotenNummer").asTextValue().setText("" + knotenNummer);
						}
						catch (NumberFormatException e)
						{
							// Keine Vorbelegung
						}
								
						if (!nurPflichtFelder) // UmsetzungsModul  ist kein Pflichtfeld
		  			{	
		  				data.getItem("UmsetzungsModul").asTextValue().setText("");
		  			}
					}
		  		else if (atg.getPid().equals("atg.gertZustndigkeit"))
					{
						if (!nurPflichtFelder) // Gert ist kein Pflichtfeld
		  			{	
		  				data.getItem("Gert").asArray().setLength(0);
		  			}	
					}
		  		else if (atg.getPid().equals("atg.punkKoordinaten"))
					{
						// Feld x nicht vorbelegt
						// Feld y nicht vorbelegt
					}
				}
				else if (objekt.isOfType("typ.anschlussPunktKommunikationsPartner"))
				{
					if (atg.getPid().equals("atg.anschlussPunktKommunikationsPartner"))
					{
//						if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//		  			{	
//		  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//		  			}
						
	  				// Bezeichnung soll Pflichtfeld sein
	  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
						
	  				if (!nurPflichtFelder) // ProtokollTyp soll kein Pflichtfeld sein
	      		{	
	  					data.getItem("ProtokollTyp").asTextValue().setText("");
	      		}
	  				
						// KommunikationsPartner wird beim Bilden der Hierarchie gesetzt
						
						String textPortNr = werte.get("Portnummer");
						long portNummer;
						try
						{
							portNummer = Long.parseLong(textPortNr);
							data.getItem("PortNummer").asTextValue().setText("" + portNummer);
						}
						catch (NumberFormatException e)
						{
							// Keine Vorbelegung
						}
					}
				}
			}
	  	else if (hot.equals(HOT_TLS.EAK))
			{
	  		if (objekt.isOfType("typ.eak"))
	  		{
		  		if (atg.getPid().equals("atg.eak"))
					{
//						if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//		  			{	
//		  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//		  			}	
		  			
	  				// Bezeichnung soll Pflichtfeld sein
	  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
						
						String textOSI2Adr = werte.get("OSI2Adresse (EAK)");
						long osi2Adresse;
						try
						{
							osi2Adresse = Long.parseLong(textOSI2Adr);
							data.getItem("OSI2Adresse").asTextValue().setText("" + osi2Adresse);
						}
						catch (NumberFormatException e)
						{
							// Keine Vorbelegung
						}
						
						if (!nurPflichtFelder) // UmsetzungsModul  ist kein Pflichtfeld
		  			{	
		  				data.getItem("UmsetzungsModul").asTextValue().setText("");
		  			}
					}
		  		else if (atg.getPid().equals("atg.punkKoordinaten"))
					{
						// Feld x nicht vorbelegt
						// Feld y nicht vorbelegt
					}
				}
			}
	  	else if (hot instanceof DeUnterTyp)
	  	{
	  		if (objekt.isOfType("typ.de"))
	  		{
		  		if (atg.getPid().equals("atg.de"))
					{
//						if (!nurPflichtFelder) // Bezeichnung ist kein Pflichtfeld
//		  			{
//							String bez = werte.get("BezeichnungDE");
//							
//							if (bez != null)
//							{	
////								data.getItem("Bezeichnung").asTextValue().setText(bez);
//								data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
//							}
////							else
////							{
////								// keine Vorbelegung
////							}
//		  			}	
							
	  				// Bezeichnung soll Pflichtfeld sein
	  				data.getItem("Bezeichnung").asTextValue().setText(getName(objekt));
						
						data.getItem("Cluster").asTextValue().setText("Nein");
						
//						if (!nurPflichtFelder) // EAKanal und DEKanal werden wie Nicht-Pflichtfelder behandelt
//						{	
							// Ea-Kanal ist fr BW eigentlich nicht vorbelegt
//							String praefix = bestimmeUmsetzungsModulPraefix(fg);
//							int eaKanal = bestimmeEAKanal(ho, praefix);
//							data.getItem("EAKanal").asTextValue().setText("" + eaKanal);
							
							String textDeKanal = werte.get("DEKanal");
							int deKanal;
							try
							{
								deKanal = Integer.parseInt(textDeKanal);
								data.getItem("DEKanal").asTextValue().setText("" + deKanal);
							}
							catch (NumberFormatException e)
							{
								// Keine Vorbelegung
							}
//  					}
											
						if (!nurPflichtFelder) // UmsetzungsModul  ist kein Pflichtfeld
		  			{	
							String text;
							
							if ("typ.deUfdMicKS".equals(objekt.getType().getPid()))
							{
								text = "de.bsvrz.kex.tls.osi7.conversion.mv.Fg003Typ132MvMicKS";
							}
							else
							{
								text = bestimmeUmsetzungsModul(((DeUnterTyp)hot).getFg());
							}
								
		  				data.getItem("UmsetzungsModul").asTextValue().setText(text);
		  			}
					}
		  		else if (atg.getPid().equals("atg.punkKoordinaten"))
					{
						// Feld x nicht vorbelegt
						// Feld y nicht vorbelegt
					}
				}
	  	}
	  	    
	    return data;
	  }
	  
	  public String getPidRegelText(IHierarchieObjektTyp hot)
	  {
	   	String felder[] = _felder.get(hot);
	  	if (felder != null)
	  	{
	  		String result = BildungsRegelnTools.LEERER_WERT;;
	  		
	  		if (hot.equals(HOT_TLS.VIZ) || hot.equals(HOT_TLS.VRZ)  || hot.equals(HOT_TLS.UZ))
	  		{
	  			String typPid = hot.getTypen().get(0);
	    		result = getPraefix(typPid) + "Bezeichnung";
	  		}
	  		else if (hot.equals(HOT_TLS.KRI))
	  		{
	  			result = "KriPid";
	  		}
	  		else if (hot.equals(HOT_TLS.Inselbus))
	  		{
	  			String typPid = hot.getTypen().get(0);
	    		result = getPraefix(typPid) + ".<PID bergeordnetes Systemobjekt>.ZustzlicheBezeichnung";
	  		}
	  		else if (hot.equals(HOT_TLS.SM))
	  		{	
	  			String typPid = hot.getTypen().get(0);
	  			result = "<PID bergeordnetes Systemobjekt>." + getPraefix(typPid) + "Portnummer_Knotennummer";
	  		}
	  		else if (hot.equals(HOT_TLS.EAK))
	  		{	
	  			String typPid = hot.getTypen().get(0);
	  			result = "<PID bergeordnetes Systemobjekt>." + getPraefix(typPid) + "OSI2Adresse (EAK)";
	  		}
	  		else if (DatenSpeicher.getInstanz().getDeTypenManager().hasHOT(hot))
	  		{
	  			String typPid = hot.getTypen().get(0);
	  			result = "<PID EAK>." + getPraefix(typPid) + "<FgKrzel><DEKanal>";
	  		}	
	  			
	  		return result;
	  	}
	  	return BildungsRegelnTools.LEERER_WERT;
	  }
	  
	  public String getNameRegelText(IHierarchieObjektTyp hot)
	  {
	  	String felder[] = _felder.get(hot);
	  	if (felder != null)
	  	{
	  		String result = BildungsRegelnTools.LEERER_WERT;;
	  		
	  		if (hot.equals(HOT_TLS.VIZ))
	  		{
	  			result = "VIZ Bezeichnung";
	  		}
	  		if (hot.equals(HOT_TLS.VRZ))
	  		{
	  			result = "VRZ Bezeichnung";
	  		}
	  		if (hot.equals(HOT_TLS.UZ))
	  		{
	  			result = "UZ Bezeichnung";
	  		}
	  		else if (hot.equals(HOT_TLS.KRI))
	  		{
	  			result = "Bezeichnung";
	  		}
	  		else if (hot.equals(HOT_TLS.Inselbus))
	  		{
	  			result = "Verbindung PidDesObjekts-ZustzlicheBezeichnung";
	  		}
	  		else if (hot.equals(HOT_TLS.SM))
	  		{	
	  			result = "BezeichnungSM SMPortnummer_Knotennummer";
	  		}
	  		else if (hot.equals(HOT_TLS.EAK))
	  		{	
	  			result = "BezeichnungSM EAKOSI2Adresse";
	  		}
	  		else if (DatenSpeicher.getInstanz().getDeTypenManager().hasHOT(hot))
	  		{
	  			result = "BezeichnungSM / BezeichnungDE";
	  		}	
	  			
	  		return result;
	  	}
	  	return BildungsRegelnTools.LEERER_WERT;
	  	
	  }
	  
	  @Override
	  public boolean setWerte (IHierarchieObjektTyp hotNeuesObjekt, HierarchieObjekt uebergeordnetesHO, 
	      HierarchieObjekt vorlageHO)
	  {
	  	boolean result = false;
	    
	    if (HOT_TLS.VIZ.equals(hotNeuesObjekt) ||
	    		HOT_TLS.VRZ.equals(hotNeuesObjekt) ||
	    		HOT_TLS.UZ.equals(hotNeuesObjekt) ||
	    		HOT_TLS.KRI.equals(hotNeuesObjekt))
	    		
	    {
	      if (uebergeordnetesHO != null)
	      {
	        EingabeVorbelegung belegung = getEingabeVorbelegung(hotNeuesObjekt, "Bezeichnung", uebergeordnetesHO);
	        String wert = belegung.getWert();
	      
	        Map<IHierarchieObjektTyp, Integer> typMap = new HashMap<IHierarchieObjektTyp, Integer>();
	            
	        for (HierarchieObjekt ho: getHierarchie().getUntergeordneteObjekte(uebergeordnetesHO))
	        {
	        	bestimmeTypMapRekursiv(ho, typMap);
	        }
	        Integer suffix = typMap.get(hotNeuesObjekt);
//	        if (suffix != null)
//	        {
//	          wert += (suffix.intValue() + 1);
//	        }
//	        
//	        setWert(hotNeuesObjekt, "Bezeichnung", wert);
//
//	        if (suffix == null)
//	        {
//	          result = true;
//	        }
	       
	        if (suffix == null)
	        {
	          setWert(hotNeuesObjekt, "Bezeichnung", wert);
	          // KRI: Bezeichnung wird vorbelegt, KriPid gefrag
	          result = !HOT_TLS.KRI.equals(hotNeuesObjekt);
	        }
	      } 
	    }
	    // else
	    // HOT_TLS.Inselbus wird immer gefragt
    	// HOT_TLS.EAK wird immer gefragt
	    // HOT_TLS.DE wird immer gefragt
	    
//	    if (uebergeordnetesHO == null)
//	    {	
//	    	setVorlageHO(vorlageHO);
//	    }	
	    
	    return result;
	  }  
	  
	  private String _vorherigeEakOSI2Adresse = null;
	  
	  /**
		 * Setzen von Zusatzinformationen fr einen bestimmten HOT
		 *    
		 * @param hot der Hierarchieobjekttyp, fr den die Zusatzinfos gelten
		 * @param info - Zusatzinfo:<br>
		 * Fr die HOTs VRZ und UZ wird das jeweilige HO vom HO-Typ VRZ oder UZ bergeben<br>
		 * Fr den HOT EAK wird das Feld OSI2Adresse korrekt gesetzt, info bleibt unbercksichtigt<br>
		 * Fr DE-Untertypen wird der Wert des Feldes DE-Kanal mit der String-Reprsentation von info gesetzt
		 */
		public void setZusatzInfo(IHierarchieObjektTyp hot, Object info)
		{
			if (HOT_TLS.EAK.equals(hot))
			{
				_vorherigeEakOSI2Adresse = getWert(hot, "OSI2Adresse (EAK)");
				setWert(hot, "OSI2Adresse (EAK)", info.toString());
			}
			else
			{
				super.setZusatzInfo(hot, info);
			}
			
		}
		
		/**
		 * Setzt die Zusatzinformationen fr einen bestimmeten HOT auf den Wert vor
		 * dem letzten <code>setZusatzInfo</code>-Aufruf zurck
		 * 
		 * @param hot der Hierarchieobjekttyp, fr den die Zusatzinfos zurckgestellt werden
		 */
		public void resetZusatzInfo(IHierarchieObjektTyp hot)
		{
			if (HOT_TLS.EAK.equals(hot))
			{
				setWert(hot, "OSI2Adresse (EAK)", _vorherigeEakOSI2Adresse);
			}
			else
			{
				super.resetZusatzInfo(hot);
			}
			
		}
	  
}
