/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkatls;

import java.util.Collection;
import java.util.HashSet;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.tkabasis.Activator;
import de.bsvrz.buv.plugin.tkabasis.LookupFactory;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.AtgInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.HierarchieObjekt;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IAttributInfo;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchie;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieObjektTyp;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.PruefungenTools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.ObjectLookup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.puk.config.configFile.datamodel.ConfigSystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * BW-spezifische Prfungen fr den TKA TLS
 * 
 * @author Sans
 *
 */
public class PruefungenBWTLS extends PruefungenTLS
{
	/**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();

	
	@Override
	public IStatus pruefeDaten(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten)
	{
		MultiStatus result = new MultiStatus(Activator.PLUGIN_ID,	Status.OK, 
  			"Prfe Daten der ATG " + atgInfo.getAtg(), null);
		 
		 IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
		 
		 if (HOT_TLS.VRZ.equals(hot) || HOT_TLS.UZ.equals(hot) || HOT_TLS.KRI.equals(hot))
		 {	 
		   // VRZ/UZ/KRI BW-spezifisch
			 if (so.isOfType("typ.anschlussPunktKommunikationsPartner"))
			 {
				 if ("atg.anschlussPunktKommunikationsPartner".equals(atgInfo.getAtg()))
				 {
					 pruefeDatenAtgAPKPBeiVRZUZKRI(ho, so, atgInfo, daten, result);
					 return result;
				 }
			 }
		 }
		 if (HOT_TLS.Inselbus.equals(hot))
		 {
			 // Inselbus BW-spezifisch
		 	 if (so.isOfType("typ.anschlussPunkt"))
			 {
				 if ("atg.anschlussPunkt".equals(atgInfo.getAtg()))
				 {
					 pruefeDatenAtgAPBeiInselbus(ho, so, atgInfo, daten, result);
					 return result;
				 }
			 }
		 }
		 if (HOT_TLS.SM.equals(hot))
		 {	 
			 // SM BW-spezifisch
			 if (so.isOfType("typ.anschlussPunktKommunikationsPartner"))
			 {
				 if ("atg.anschlussPunktKommunikationsPartner".equals(atgInfo.getAtg()))
				 {
					 pruefeDatenAtgAPKPBeiSM(ho, so, atgInfo, daten, result);
					 return result;
				 }
			 }
		 }
		 
		 return super.pruefeDaten(ho, so, atgInfo, daten);
	}
	
	/**
	 * Prfen - BW-spezifisch - der Daten fr die ATG atg.anschlussPunktKommunikationsPartner 
	 * bei HOT SM
	 * 
	 * @param ho das Hierarchieobhekt, zu dem das bergebene Systemobjekt gehrt
	 * @param so	so das Systemobjekt, zu dem ATG-Daten bernommen werden sollen
   * @param atgInfo Info ber die ATG der zu bernehmenden Daten
   * @param daten Datensatz der Daten, die geprft werden sollen
	 * @param status Multimeldung, die mit Meldungen ergnzt wird
	 */
	private void pruefeDatenAtgAPKPBeiSM(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten, MultiStatus status)
  {
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
//  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten));
  		boolean pflichtFeldFlag = "ProtokollTyp".equals(attInfo.getName()); // ProtokollTyp in BW KEIN Pflichtfeld
  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten, pflichtFeldFlag)); 
  		
  		if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
  			if ("PortNummer".equals(attribut))
    		{
  				Assert.isTrue(HOT_TLS.SM.equals(ho.getHierarchieObjektTyp()));
  				HierarchieObjekt hoVater = (HierarchieObjekt)getHierarchie().getUebergeordnetesObjekt(ho);
  				HierarchieObjekt hoGeraet = (HierarchieObjekt)getHierarchie().getUebergeordnetesObjekt(hoVater);
  				
    			Collection<String> vorhandeneKombinationen = new HashSet<String>();
    			IHierarchie hierarchie = getHierarchie();
    			ConfigDataModel dataModel = hierarchie.getDataModel();
    			AttributeGroup atgAP = dataModel.getAttributeGroup("atg.anschlussPunkt");
    			AttributeGroup atgAPKP = dataModel.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
    			ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
    			
    			SystemObject vater = hoVater.getSystemObjekt(0);
    			Data data = ((ConfigSystemObject)vater).getConfigurationData(atgAP, lookup);
    			if (data != null)
    			{	
    				long portNummerAP = data.getItem(attribut).asUnscaledValue().longValue();
	    			long portNummerAPKP = daten.getItem(attribut).asUnscaledValue().longValue();
	    			
	    			for (HierarchieObjekt hoKind: hierarchie.getUntergeordneteObjekte(hoGeraet))
	    			{
	    				if (HOT_TLS.Inselbus.equals(hoKind.getHierarchieObjektTyp()))
	    				{
	    					SystemObject ap = hoKind.getSystemObjekt(0);    				
	    					Data dataAP = ((ConfigSystemObject)ap).getConfigurationData(atgAP, lookup);
	  		  			if (dataAP != null)
	  		  			{
	  							long nummerAP = dataAP.getItem("PortNummer").asUnscaledValue().longValue();
	  							for (HierarchieObjekt hoSM: hierarchie.getUntergeordneteObjekte(hoKind))
	  							{
	  								if (HOT_TLS.SM.equals(hoSM.getHierarchieObjektTyp()) && hoSM != ho)
	  		    				{
	  									SystemObject apkp = hoSM.getSystemObjekt(1);    				
	  		    					Data dataAPKP = ((ConfigSystemObject)apkp).getConfigurationData(atgAPKP, lookup);
	  		  		  			if (dataAPKP != null)
	  		  		  			{
	  		  							long nummerAPKP = dataAPKP.getItem("PortNummer").asUnscaledValue().longValue();
	  		  							vorhandeneKombinationen.add(nummerAP + "/" + nummerAPKP);
	  		  		  			}	
	  		    				}
	  							}
	  		  			}	
	    				}
	    			}
    			
	    			if (vorhandeneKombinationen.contains(portNummerAP + "/" + portNummerAPKP))
	    			{
	    				meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
	    				"Die Kombination " + portNummerAP + "/" + portNummerAPKP + " fr Portnummer AP/Portnummer APKP wird fr " +
	    				hoGeraet + " bereits verwendet"));
	    			}
	    			else
	    			{
	    				meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
	    				"Prfen Eindeutigkeit der Kombination Portnummer AP/Portnummer APKP fr bergeordnetes Gert: OK"));
	    			}
    			}	
    		}
			}
  		
  		status.add(meldung);
  	}
  }
	
	/**
	 * Prfen - BW-spezifisch - der Daten fr die ATG atg.anschlussPunkt 
	 * bei HOT Inselbus
	 * 
	 * @param ho das Hierarchieobhekt, zu dem das bergebene Systemobjekt gehrt
	 * @param so	so das Systemobjekt, zu dem ATG-Daten bernommen werden sollen
   * @param atgInfo Info ber die ATG der zu bernehmenden Daten
   * @param daten Datensatz der Daten, die geprft werden sollen
	 * @param status Multimeldung, die mit Meldungen ergnzt wird
	 */
	private void pruefeDatenAtgAPBeiInselbus(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten, MultiStatus status)
  {
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
//  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten));
  		boolean pflichtFeldFlag = "ProtokollTyp".equals(attInfo.getName()); // ProtokollTyp in BW KEIN Pflichtfeld
  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten, pflichtFeldFlag)); 
  		
  		if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
  			if ("PortNummer".equals(attribut))
    		{
  				Assert.isTrue(HOT_TLS.Inselbus.equals(ho.getHierarchieObjektTyp()));
  				HierarchieObjekt hoVater = (HierarchieObjekt)getHierarchie().getUebergeordnetesObjekt(ho);
  				  				
    			Collection<String> vorhandeneKombinationen = new HashSet<String>();
    			IHierarchie hierarchie = getHierarchie();
    			ConfigDataModel dataModel = hierarchie.getDataModel();
    			AttributeGroup atgAP = dataModel.getAttributeGroup("atg.anschlussPunkt");
    			AttributeGroup atgAPKP = dataModel.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
    			ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
    			
    			long portNummerAP = daten.getItem(attribut).asUnscaledValue().longValue();
		    			
	    		for (HierarchieObjekt hoKind: hierarchie.getUntergeordneteObjekte(hoVater))
    			{
    				if (HOT_TLS.Inselbus.equals(hoKind.getHierarchieObjektTyp()) && hoKind != ho)
    				{
    					SystemObject ap = hoKind.getSystemObjekt(0);    				
    					Data dataAP = ((ConfigSystemObject)ap).getConfigurationData(atgAP, lookup);
  		  			if (dataAP != null)
  		  			{
  							long nummerAP = dataAP.getItem("PortNummer").asUnscaledValue().longValue();
  							for (HierarchieObjekt hoSM: hierarchie.getUntergeordneteObjekte(hoKind))
  							{
  								if (HOT_TLS.SM.equals(hoSM.getHierarchieObjektTyp()))
  		    				{
  									SystemObject apkp = hoSM.getSystemObjekt(1);    				
  		    					Data dataAPKP = ((ConfigSystemObject)apkp).getConfigurationData(atgAPKP, lookup);
  		  		  			if (dataAPKP != null)
  		  		  			{
  		  							long nummerAPKP = dataAPKP.getItem("PortNummer").asUnscaledValue().longValue();
  		  							vorhandeneKombinationen.add(nummerAP + "/" + nummerAPKP);
  		  		  			}	
  		    				}
  							}
  		  			}	
    				}
    			}
	    		
	    		for (HierarchieObjekt hoSM: hierarchie.getUntergeordneteObjekte(ho))
	    		{
	    			if (HOT_TLS.SM.equals(hoSM.getHierarchieObjektTyp()))
    				{
							SystemObject apkp = hoSM.getSystemObjekt(1);    				
    					Data dataAPKP = ((ConfigSystemObject)apkp).getConfigurationData(atgAPKP, lookup);
  		  			if (dataAPKP != null)
  		  			{
  							long portNummerAPKP = dataAPKP.getItem("PortNummer").asUnscaledValue().longValue();
  							if (vorhandeneKombinationen.contains(portNummerAP + "/" + portNummerAPKP))
  							{
  								meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
  				    				"Die Kombination " + portNummerAP + "/" + portNummerAPKP + " fr Portnummer AP/Portnummer APKP wird fr " +
  				    				hoVater + " bereits verwendet"));
  							}	
  		  			}	
    				}
	    		}
	    		
	    		if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
	    		{
    				meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfen Eindeutigkeit der Kombination Portnummer AP/Portnummer APKP fr bergeordnetes Gert: OK"));
    			}
    		}
			}
  		
  		status.add(meldung);
  	}
  }
	
	/**
	 * Prfen der Daten fr die ATG atg.anschlussPunktKommunikationsPartner
	 * 
	 * @param ho das Hierarchieobhekt, zu dem das bergebene Systemobjekt gehrt
	 * @param so	so das Systemobjekt, zu dem ATG-Daten bernommen werden sollen
   * @param atgInfo Info ber die ATG der zu bernehmenden Daten
   * @param daten Datensatz der Daten, die geprft werden sollen
	 * @param status Multimeldung, die mit Meldungen ergnzt wird
	 */
  private void pruefeDatenAtgAPKPBeiVRZUZKRI(HierarchieObjekt ho, SystemObject so, AtgInfo atgInfo, Data daten, MultiStatus status)
  {
  	for (String attribut: atgInfo.getAttribute())
  	{
  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
    			attribut, null);
  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
//		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten));
  		boolean pflichtFeldFlag = "ProtokollTyp".equals(attInfo.getName()); // ProtokollTyp in BW KEIN Pflichtfeld
  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten, pflichtFeldFlag)); 
  		
  		if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
			{
  			if ("PortNummer".equals(attribut))
    		{
  				long portNummer = daten.getItem(attribut).asUnscaledValue().longValue();
  				Collection<Long> vorhandeneNummern = new HashSet<Long>();
    			IHierarchieObjektTyp hot = ho.getHierarchieObjektTyp();
    			SystemObject ap = null;
    			IHierarchie hierarchie = getHierarchie();

    			if (hierarchie.getUebergeordnetesObjekt(ho) instanceof HierarchieObjekt) // Nicht fr HO unmittelbar unter KB
    			{	
		  			if (hot.equals(HOT_TLS.VRZ) || hot.equals(HOT_TLS.UZ) || hot.equals(HOT_TLS.KRI))
		  			{
	  					ap = ho.getSystemObjekt(1);
		  			}	
    			}
    			
    			if (ap != null)
    			{
  					NonMutableSet apkpMenge = ((ConfigurationObject)ap).getNonMutableSet("AnschlussPunkteKommunikationsPartner");
  					if (apkpMenge != null)
  					{
  						for (SystemObject apkp: apkpMenge.getElementsInModifiableVersion())
  						{
  							if (apkp == null)
	  	          {
	  	          	_debug.warning("null-Referenz in Menge " + apkpMenge.getName() + " von " + ap.getPid());
	  	          	continue;
	  	          }
  							if (apkp != so)
  							{
  								ConfigDataModel dataModel = getHierarchie().getDataModel();
  		      			AttributeGroup atg = dataModel.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
  		      			ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
  		    				
		              Data data = ((ConfigSystemObject)apkp).getConfigurationData( atg, lookup );
		              if (data != null)
		              {	
		              	long nummer = data.getItem("PortNummer").asUnscaledValue().longValue();
	    							vorhandeneNummern.add(nummer);
		              }  
  							}  
  						}
  					}
  				}
    			
    			if (vorhandeneNummern.contains(portNummer))
    			{
    				meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
    				"Die Portnummer " + portNummer + " wird bereits verwendet"));
    			}
    			else
    			{
    				meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
    				"Prfen Eindeutigkeit Portnummer fr AnschlussPunktKomnunikationsPartner: OK"));
    			}
    		}
			}
  		
  		status.add(meldung);
  	}
  	
//  	for (String attribut: atgInfo.getAttribute())
//  	{
//  		MultiStatus	meldung = new MultiStatus(Activator.PLUGIN_ID, Status.OK, "Prfe Wert fr Attribut " +
//    			attribut, null);
//  		IAttributInfo attInfo = atgInfo.getAttributInfo(attribut);
////  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten));
//  		boolean pflichtFeldFlag = "ProtokollTyp".equals(attInfo.getName()); // ProtokollTyp in BW KEIN Pflichtfeld
//  		meldung.merge(PruefungenTools.pruefeDatenDefinition(attInfo, daten, pflichtFeldFlag)); 
//  		
//  		if (!meldung.matches(IStatus.ERROR | IStatus.CANCEL))
//			{
//  			if ("PortNummer".equals(attribut))
//    		{
//    			long portNummer = daten.getItem(attribut).asUnscaledValue().longValue(); 
//    			
//    			Collection<Long> vorhandeneNummern = new HashSet<Long>();
//    			ConfigDataModel dataModel = getHierarchie().getDataModel();
//    			AttributeGroup atg = dataModel.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
//    			ObjectLookup lookup = LookupFactory.createLookupForModifiableVersion(dataModel);
//    			HierarchieObjekt hoVater = (HierarchieObjekt)getHierarchie().getUebergeordnetesObjekt(ho);
//    			
//    			for (HierarchieObjekt hoKind: getHierarchie().getUntergeordneteObjekte(hoVater))
//    	  	{
//    	  		if (hoKind != ho)
//    	  		{
//    	  			SystemObject apkp = null;
//    	  			IHierarchieObjektTyp hot = hoKind.getHierarchieObjektTyp();
//    	  			if (hot.equals(HOT_TLS.VRZ) || hot.equals(HOT_TLS.UZ) || hot.equals(HOT_TLS.KRI))
//    	  			{
//    	  				apkp = hoKind.getSystemObjekt(2);
//    	  			}
//    	  			else if (hot.equals(HOT_TLS.SM))
//    	  			{
//    	  				apkp = hoKind.getSystemObjekt(1);
//    	  			}
//    	  				
//    	  			if (apkp != null)
//    	  			{	
//    		  			Data data = ((ConfigSystemObject)apkp).getConfigurationData(atg, lookup);
//    		  			if (data != null)
//    		  			{
//    							long nummer = data.getItem("PortNummer").asUnscaledValue().longValue();
//    							vorhandeneNummern.add(nummer);
//    						}
//    	  			}	
//    	  		}	
//    	  	}
//    			
//    			if (vorhandeneNummern.contains(portNummer))
//    			{
//    				meldung.add(new Status(IStatus.ERROR, Activator.PLUGIN_ID, 
//    				"Die Portnummer " + portNummer + " wird bereits verwendet"));
//    			}
//    			else
//    			{
//    				meldung.add(new Status(IStatus.OK, Activator.PLUGIN_ID, 
//    				"Prfen Eindeutigkeit Portnummer fr AnschlussPunktKomnunikationsPartner: OK"));
//    			}
//    		}
//			}
//  		
//  		status.add(meldung);
//  	}
  }
	
}
