/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.tkaufd;

import de.bsvrz.buv.plugin.tkabasis.IModelInfo;
import de.bsvrz.buv.plugin.tkabasis.TKABasisView;
import de.bsvrz.buv.plugin.tkabasis.hierarchie.IHierarchieManager;
import de.bsvrz.buv.plugin.tkabasis.pruefungen.IPruefungen;
import de.bsvrz.buv.plugin.tkabasis.regeln.IBildungsRegeln;
import de.bsvrz.puk.config.configFile.datamodel.ConfigDataModel;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Dient der Kommunikation mit dem DaV bzgl. der SystemkalenderVerwaltung.
 * 
 * @author HofmannJ
 * 
 */
public class DatenSpeicher implements IModelInfo
{

  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private TKABasisView _tkaBasisView = null;

  private ConfigDataModel _configDataModel = null;

  private String _dateiPfad = null;
  
  private String _secId = null;
 
  private IHierarchieManager _hierarchieManager;
   
  private IBildungsRegeln _bildungsRegeln;
  
  private IPruefungen _pruefungen;
  
  private UFDSTypenManager _ufdsTypenManager;
  
	/**
   * Singleton-Instanz
   */
  private static DatenSpeicher _instanz = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static DatenSpeicher getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new DatenSpeicher();
    }
    return _instanz;
  }

  /**
   * Konstruktor, setzt die Verbindung zum DaV.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  private DatenSpeicher()
  {

  }

  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public void setzeTKABasisView(TKABasisView tkaBasisView)
  {
    this._tkaBasisView = tkaBasisView;
  }

  /**
   * Holt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public TKABasisView holeTKABasisView()
  {
    return this._tkaBasisView;
  }
  
  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public ConfigDataModel holeConfigDataModel()
  {
    return _configDataModel;
  }

  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public void setzeConfigDataModel(ConfigDataModel configDataModel)
  {
    _configDataModel = configDataModel;
  }


  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public String holeDateiPfad()
  {
	  return _dateiPfad;
  }
  
  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public void setzeDateiPfad(String pfad)
  {
	  _dateiPfad = pfad;
  }

  public IHierarchieManager getHierarchieManager()
	{
		return _hierarchieManager;
	}

	void setHierarchieManager(IHierarchieManager hierarchieManager)
	{
		_hierarchieManager = hierarchieManager;
	}

	public IBildungsRegeln getBildungsRegeln()
	{
		return _bildungsRegeln;
	}

	void setBildungsRegeln(IBildungsRegeln bildungsRegeln)
	{
		_bildungsRegeln = bildungsRegeln;
	}
	
	public IPruefungen getPruefungen()
	{
		return _pruefungen;
	}

	public void setPruefungen(IPruefungen pruefungen)
	{
		_pruefungen = pruefungen;
	}
	
	public UFDSTypenManager getUFDSTypenManager()
	{
		return _ufdsTypenManager;
	}

	public void setUFDSTypenManager(UFDSTypenManager ufdsTypenManager)
	{
		_ufdsTypenManager = ufdsTypenManager;
	}
	
  public String holeSecId()
  {
    return _secId;
  }

  public void setzeSecId(String secId)
  {
    _secId = secId;
  }
	
}
