/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.lib.debug;

//~ NICHT JDK IMPORTE =========================================================

import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;

//~ KLASSEN ===================================================================

/**
 * Klasse stellt das GUI 'Protokollmeldungen' dar.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 1.4 $ / $Date: 2010/11/18 15:02:17 $ / ($Author: Pittner $)
 */
public class ProtokollGUI extends Composite {

    /** Font der Meldungen. */
    private static final Font FONT_COURIER_9_NORMAL = new Font(Display.getCurrent(), "Courier", 9, SWT.NORMAL);

    //~ FELDER ================================================================

    /** Schaltflche 'Lschen'. */
    private Button _btnLoeschen;

    /** Schaltflche 'Speichern'. */
    private Button _btnSpeichern;

    /** Verwaltungsinstanz. */
    private ProtokollGUIVerwaltung _guiVew;

    /** Baum mit den Meldungen. */
    private TreeViewer _treeViewerMeldungen;

    /** Meldungstext. */
    private Text _txtMeldung;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse ProtokollGUI.
     *
     * @param parent Parent-Composite.
     */
    public ProtokollGUI(Composite parent, Map<IStatus, Object[]> listeMeldung) {
        super(parent, SWT.NONE);
        _guiVew = new ProtokollGUIVerwaltung(this, listeMeldung);

        final GridData gd_compVerwaltungsBereich = new GridData(SWT.FILL, SWT.FILL, true, true);

        setLayoutData(gd_compVerwaltungsBereich);

        final GridLayout gridLayout = new GridLayout();

        gridLayout.marginWidth  = 0;
        gridLayout.marginHeight = 0;
        setLayout(gridLayout);
        initGUI();
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'btnLoeschen'.
     *
     * @return Wert der Eigenschaft 'btnLoeschen'.
     *
     * @see #_btnLoeschen
     */
    public Button getBtnLoeschen() {
        return _btnLoeschen;
    }

    /**
     * Getter fr Eigenschaft 'btnSpeichern'.
     *
     * @return Wert der Eigenschaft 'btnSpeichern'.
     *
     * @see #_btnSpeichern
     */
    public Button getBtnSpeichern() {
        return _btnSpeichern;
    }

    /**
     *     Getter fr Eigenschaft 'guiVew'.
     *
     *     @return Wert der Eigenschaft 'guiVew'.
     *
     *     @see #_guiVew
     */
    public ProtokollGUIVerwaltung getGuiVew() {
        return _guiVew;
    }

    /**
     * Getter fr Eigenschaft 'treeViewerMeldungen'.
     *
     * @return Wert der Eigenschaft 'treeViewerMeldungen'.
     *
     * @see #_treeViewerMeldungen
     */
    public TreeViewer getTreeViewerMeldungen() {
        return _treeViewerMeldungen;
    }

    /**
     * Getter fr Eigenschaft 'txtMeldung'.
     *
     * @return Wert der Eigenschaft 'txtMeldung'.
     *
     * @see #_txtMeldung
     */
    public Text getTxtMeldung() {
        return _txtMeldung;
    }

    //~ METHODEN ==============================================================

    /**
     * Initialisiere das GUI.
     */
    private void initGUI() {
        final Composite compAllgemein = new Composite(this, SWT.NONE);

        compAllgemein.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

        final GridLayout gridLayout_1 = new GridLayout();

        gridLayout_1.marginHeight = 0;
        gridLayout_1.marginWidth  = 0;
        compAllgemein.setLayout(gridLayout_1);

        final SashForm sashComp = new SashForm(compAllgemein, SWT.BORDER | SWT.VERTICAL);

        sashComp.SASH_WIDTH = 1;
        sashComp.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

        //
        final Composite compBaum = new Composite(sashComp, SWT.NONE);

        compBaum.setLayout(new GridLayout());
        _treeViewerMeldungen = new TreeViewer(compBaum, SWT.FULL_SELECTION | SWT.BORDER);
        _treeViewerMeldungen.addSelectionChangedListener(new ISelectionChangedListener() {
            public void selectionChanged(final SelectionChangedEvent event) {
                _guiVew.selektionImBaumGeaendert();
            }
        });
        _treeViewerMeldungen.setContentProvider(new ProtokollGUIVerwaltung.LogTreeContentProvider());
        _treeViewerMeldungen.setLabelProvider(new ProtokollGUIVerwaltung.LogTreeLabelProvider());

        Tree tree = _treeViewerMeldungen.getTree();

        tree.setLinesVisible(true);
        tree.setHeaderVisible(true);
        tree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

        TreeColumn col1 = new TreeColumn(tree, SWT.LEFT);

        col1.setText("Meldung");
        col1.setWidth(500);

        TreeColumn col2 = new TreeColumn(tree, SWT.LEFT);

        col2.setText("Plug-in");
        col2.setWidth(200);

        TreeColumn col3 = new TreeColumn(tree, SWT.LEFT);

        col3.setText("Meldungszeit");
        col3.setWidth(120);
        
        TreeColumn col4 = new TreeColumn(tree, SWT.LEFT);
        
        col4.setText("Konfiguration");
        col4.setWidth(120);

        final Composite compText = new Composite(sashComp, SWT.BORDER);

        compText.setLayout(new GridLayout());
        _txtMeldung = new Text(compText, SWT.H_SCROLL | SWT.V_SCROLL | SWT.CANCEL | SWT.MULTI);
        _txtMeldung.setFont(FONT_COURIER_9_NORMAL);
        _txtMeldung.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        sashComp.setWeights(new int[] { 1, 1 });

        final Composite compSchaltflaeche = new Composite(compAllgemein, SWT.NONE);

        compSchaltflaeche.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, true, false));

        final GridLayout gridLayout = new GridLayout();

        gridLayout.makeColumnsEqualWidth = true;
        gridLayout.numColumns            = 2;
        compSchaltflaeche.setLayout(gridLayout);
        _btnSpeichern = new Button(compSchaltflaeche, SWT.NONE);
        _btnSpeichern.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(final SelectionEvent e) {
                _guiVew.btnSpeichernSelektiert();
            }
        });
        _btnSpeichern.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
        _btnSpeichern.setText("Speichern");
        _btnLoeschen = new Button(compSchaltflaeche, SWT.NONE);
        _btnLoeschen.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
        _btnLoeschen.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(final SelectionEvent e) {
                _guiVew.btnLoeschenSelektiert();
            }
        });
        _btnLoeschen.setText("Lschen");
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 01.03.10
